/*
 coding: UTF-8
*/
/*
 * badscan.c - Milena Audiobook Creator
 * Copyright (C) Bohdan R. Rau 2010 <ethanak@polip.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write see:
 *               <http://www.gnu.org/licenses/>.
 */
#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>
#include "gmilena.h"
#include <milena.h>
#include <ctype.h>
#include <iconv.h>
#include <glib.h>

static char **verbs;
static int many_verbs,verbs_size;
static char *verb_memo;
static int verb_memo_pos;

static char **vexts;
static int vexts_no;

static char *internal_bwords[]={
    "łan","łana","łanowi","łanem","łanie","ładownik*","Ładownik*","byty","en:memoriał","en:Memoriał","Byty",
    "en:Annę","en:Irenę","en:Marię","fr:Marię","fr:Annę",
    NULL};

static struct bad_word {
    struct bad_word *next;
    short leng;
    short langid;
    char str[1];
} *bad_words;


static void free_bad_words(void)
{
    struct bad_word *bw;
    while (bw=bad_words) {
        bad_words=bad_words->next;
        free(bw);
    }
}

static int dic_cmp(const void *v1,const void *v2)
{
	return strcmp(*(char **)v1,*(char **)v2);
}

static void add_ibword(char *str)
{
    int n,len,i;char *c;
    int langid=-1;
    struct bad_word *bw;
    c=strchr(str,':');
    if (c) {
        *c++=0;
        for (i=0;i<NUM_LANGS;i++) if (!strcmp(str,lang_codes[i])) {
            langid=i;
            break;
        }
        str=c;
    }
    if (!*str) return;
    n=strlen(str)-1;
    if (str[n]=='*') {
        str[n]=0;
    }
    else {
        n=0;
    }
    bw=malloc(sizeof(*bw)+strlen(str));
    bw->next=bad_words;
    bad_words=bw;
    strcpy(bw->str,str);
    bw->leng=n;
    bw->langid=langid;
}        

static int read_bad_words(char *fname)
{
    char ibuf[256];
    FILE *f=fopen(fname,"r");
    if (!f) return 0;
    free_bad_words();
    while(fgets(ibuf,256,f)) {
        char *c;
        c=strstr(ibuf,"//");
        if (c) *c=0;
        c=trim(ibuf);
        if (*c) add_ibword(c);
    }
    fclose(f);
    return 1;
}

char scanner_path[256];
int readScanner(void)
{
    GtkWidget *dialog=gtk_file_chooser_dialog_new(
			"Wybierz plik",
			GTK_WINDOW(main_window),GTK_FILE_CHOOSER_ACTION_OPEN,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			NULL);
    GtkFileFilter *filter;
    gtk_file_chooser_set_local_only((gpointer)dialog,TRUE);
    if (scanner_path[0]) {
    	gtk_file_chooser_set_filename((gpointer)dialog,scanner_path);
    }
    if (gtk_dialog_run((gpointer)dialog)!=GTK_RESPONSE_ACCEPT) {
        gtk_widget_destroy(dialog);
        return 0;
    }
    strcpy(scanner_path,gtk_file_chooser_get_filename((gpointer)dialog));
    gtk_widget_destroy(dialog);
    if (read_bad_words(scanner_path)) {
        Info("Informacja","Plik został wczytany");
        return 1;
    }
    Error("Błąd","Nie mogę wczytać pliku podejrzanych");
    return 0;
}

#define VERB_MEMO_SIZE 65536


static int read_verbs(void)
{
    char ibuf[256];
    char wbuf[512];
    FILE *f;
    char *s,*c,*vr;
    static int was_error=0;
    int i,lverb,lext;
    iconv_t ic;

    int v_append_verb(char *vr)
    {
        char *dst,*c;
        size_t slen,dlen;
        int len;
        slen=strlen(vr);
        dlen=511;
        dst=wbuf;
        iconv(ic,&vr,&slen,&dst,&dlen);
        if (*vr) {
            Error("Błąd","Błąd działania Iconv");
            was_error=1;
            return 0;
        }
        *dst=0;
        len=strlen(wbuf)+1;
        if (verb_memo_pos+len >= VERB_MEMO_SIZE) {
            verb_memo=malloc(VERB_MEMO_SIZE);
            verb_memo_pos=0;
        }
        c=verb_memo+verb_memo_pos;
        strcpy(c,wbuf);
        verb_memo_pos+=len;
        if (many_verbs >= verbs_size) {
            verbs_size+=8192;
            verbs=realloc(verbs,sizeof(*verbs)*verbs_size);
        }
        verbs[many_verbs++]=c;
        return 1;
    }

    int v_append_ext(char *vr)
    {
        char *dst,*c;
        size_t slen,dlen;
        int len;
        slen=strlen(vr);
        dlen=511;
        dst=wbuf;
        iconv(ic,&vr,&slen,&dst,&dlen);
        if (*vr) {
            Error("Błąd","Błąd działania Iconv");
            was_error=1;
            return 0;
        }
        *dst=0;
        len=strlen(wbuf)+1;
        if (verb_memo_pos+len >= VERB_MEMO_SIZE) {
            verb_memo=malloc(VERB_MEMO_SIZE);
            verb_memo_pos=0;
        }
        c=verb_memo+verb_memo_pos;
        strcpy(c,wbuf);
        verb_memo_pos+=len;
        vexts[vexts_no++]=c;
        return 1;
    }


    if (was_error) return 0;
    ic=iconv_open("UTF-8","ISO-8859-2");
    if (ic == (iconv_t)-1) {
        Error("Błąd","Iconv nie działa");
        was_error=1;
        return 0;
    }
    s=milena_FilePath("pl_dict.dat",ibuf);
    f=fopen(s,"r");
    if (!f) {
        Error("Błąd","Nie mogę wczytać pliku słownika");
        was_error=1;
        return 0;
    }
    many_verbs=0;
    verbs_size=8192;
    verbs=malloc(sizeof(*verbs) * verbs_size);
    verb_memo=malloc(VERB_MEMO_SIZE);
    verb_memo_pos=0;
    while (fgets(ibuf,256,f)) {
        c=ibuf;
        while (*c && isspace(*c)) c++;
        if (!*c) continue;
        vr=c;
        while (*c && !isspace(*c)) c++;
        s=c-1;
        if (*c=='\xe6') continue;
        if (!*c) continue;
        *c++=0;
        for (;*c && *c!='v' && *c!='V';c++);
        if (!*c) continue;
        if (!v_append_verb(vr)) {
            fclose(f);
            return 0;
        }
    }
    fclose(f);
    s=milena_FilePath("pl_verbs.dat",ibuf);
    f=fopen(s,"r");
    if (!f) goto final;
    fgets(ibuf,256,f);
    lverb=strtol(ibuf,NULL,10);
    fgets(ibuf,256,f);
    lext=strtol(ibuf,NULL,10);
    for (i=0;i<lverb;i++) {
        if (!fgets(ibuf,256,f)) break;
        c=trim(ibuf);
        if (!c) continue;
        if (c[strlen(c)-1] == '\xe6') continue;
        if (!v_append_verb(c)) {
            fclose(f);
            return 0;
        }
    }
    vexts=malloc(lext * sizeof(*vexts));
    vexts_no=0;
    while (fgets(ibuf,256,f)) {
        if (vexts_no > lext) break;
        c=trim(ibuf);
        if (*c=='\xe6') continue;
        if (!v_append_ext(c)) {
            fclose(f);
            return 0;
        }
    }
    qsort(vexts,vexts_no,sizeof(*vexts),dic_cmp);
    fclose(f);
final:    
    qsort(verbs,many_verbs,sizeof(*verbs),dic_cmp);
    if (bad_words) return 1;
    if (scanner_path[0]) {
        if (read_bad_words(scanner_path)) return 1;
    }
    for (i=0;internal_bwords[i];i++) {
        strcpy(ibuf,internal_bwords[i]);
        add_ibword(ibuf);
    }
    return 1;
}

static char *my_get_word(char *str,char *wrdbuf,int *que)
{
    gunichar g;
    char *c=str;
    if (que) *que=0;
    while (*c) {
        g=g_utf8_get_char(c);
        if (!g_unichar_isalnum(g)) break;
        c=g_utf8_next_char(c);
    }
    if (!que || g != '?') goto fine;
    g=g_utf8_get_char(c+1);
    if (!g_unichar_isalnum(g)) goto fine;
    c++;
    while (*c) {
        g=g_utf8_get_char(c);
        if (!g_unichar_isalnum(g)) break;
        c=g_utf8_next_char(c);
    }
    *que=1;
    
fine:    
    memcpy(wrdbuf,str,c-str);
    wrdbuf[c-str]=0;
    return c;
}

static int wordbuf_is_verb(char *wbuf)
{
    char buffy[1024];
    char *c;
    int mid,lo,hi,n;
    gunichar g;
    c=buffy;
    while (*wbuf) {
        g=g_unichar_tolower(g_utf8_get_char(wbuf));
        c+=g_unichar_to_utf8(g,c);
        wbuf=g_utf8_next_char(wbuf);
    }
    *c=0;
    lo=0;
    hi=many_verbs-1;
    while (lo <=hi) {
        mid=(lo+hi)/2;
        n=strcmp(buffy,verbs[mid]);
        if (n<0) hi=mid-1;
        else if (n>0) lo=mid+1;
        else return 1;
    }
    c=g_utf8_strreverse(buffy,-1);
    strcpy(buffy,c);
    g_free(c);
    lo=0;
    hi=vexts_no-1;
    while (lo <=hi) {
        mid=(lo+hi)/2;
        n=strncmp(buffy,vexts[mid],strlen(vexts[mid]));
        if (n<0) hi=mid-1;
        else if (n>0) lo=mid+1;
        else return 1;
    }
    
    return 0;
}

static unsigned int langmask;

static char *bad_strings[]={ "ąu","ąi",
    NULL};

static int vdic_list_ok;
static struct vdic_l {
    struct vdic_l *next;
    char *str;
} *vdic_list;

static void free_vdic_list(void)
{
    struct vdic_l *vl;
    while (vl=vdic_list) {
        vdic_list=vdic_list->next;
        free(vl->str);
        free(vl);
    }
    vdic_list_ok=0;
}

static void get_vdic_list(void)
{
	struct vdic_l *vl;
        char *c,*d,*cs;
        if (!current_editor->dic_editor) return;
        char *body=get_editor_body(current_editor->dic_editor);
        for (c=body;c && *c;c=cs) {
            cs=strchr(c,'\n');
            if (!cs) break;
            *cs++=0;
            d=strstr(c,"//");
            if (d) *d=0;
            for (d=c;*d && !isspace(*d);*d++);
            if (!*d) continue;
            *d++=0;
            if (!*c) continue;
            while (*d && isspace(*d)) d++;
            if (!*d) continue;
            vl=malloc(sizeof(*vl));
            vl->str=strdup(c);
            vl->next=vdic_list;
            vdic_list=vl;
        }
        g_free(body);
    
}
static int word_is_known_to_dic(char *str)
{
    struct vdic_l *vl;
    if (!vdic_list_ok) {
        get_vdic_list();
        vdic_list_ok=1;
    }
    for (vl=vdic_list;vl;vl=vl->next) if (!strcmp(vl->str,str)) return 1;
    return 0;
    
}
int find_bad_construction(char *str,int *wrlen)
{
    char wordbuf[1024];
    gunichar g;
    char *c=str;
    int pos,i,que;
    struct bad_word *bw;
    if (!verbs && !read_verbs()) return -1;
    free_vdic_list();
    for (i=0,langmask=0;i<NUM_LANGS;i++) {
        if (gtk_toggle_button_get_active((gpointer)lang_cb[i])) {
            langmask |= 1<<i;
        }
    }
    while (*c) {
        g=g_utf8_get_char(c);
        if (!g_unichar_isalnum(g)) break;
        c=g_utf8_next_char(c);
    }
    while (*c) {
        while (*c) {
            g=g_utf8_get_char(c);
            if (g_unichar_isalnum(g)) break;
            c=g_utf8_next_char(c);
        }
        if (!*c) return -1;
        pos=c-str;
        c=my_get_word(c,wordbuf,&que);
        if (que) {
            if (wrlen) *wrlen=g_utf8_strlen(wordbuf,-1);
            return pos;
        }
        //printf("%s\n",wordbuf);
        if (!strcasecmp(wordbuf,"nic")) {
            while (*c && *c==' ') c++;
            if (c) g=g_utf8_get_char(c);
            else g=' ';
            if (g_unichar_isalnum(g)) {
                int pos1=c-str;
                c=my_get_word(c,wordbuf,&que);
                if (que) {
                    if (wrlen) *wrlen=g_utf8_strlen(wordbuf,-1);
                    return pos1;
                }
                if (wordbuf_is_verb(wordbuf)) {
                    if (wrlen) *wrlen=3;
                    return pos;
                }
                pos=pos1;
            }
        }
        if (word_is_known_to_dic(wordbuf)) continue;
        if (wrlen) *wrlen=g_utf8_strlen(wordbuf,-1);
        for (i=0;bad_strings[i];i++) if (strstr(wordbuf,bad_strings[i])) return pos;
        for (bw=bad_words;bw;bw=bw->next) {
            if (bw->langid>=0) {
                if (!(langmask & (1<<bw->langid))) continue;
            }
            if (bw->leng) {
                if (!strncmp(wordbuf,bw->str,bw->leng)) return pos;
            }
            else {
                if (!strcmp(wordbuf,bw->str)) return pos;
            }
        }
    }
    return -1;
}
