/*
 * dykcjonarz.c - Milena Audiobook Creator
 * Copyright (C) Bohdan R. Rau 2009 <ethanak@polip.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write see:
 *               <http://www.gnu.org/licenses/>.
 */
#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>
#include "gmilena.h"
#include <milena.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

extern char *to_utf8(char *fbuf,int flen,char *cset,int freeme);

static char lci[256]={
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	'0','1','2','3','4','5','6','7','8','9',0,0,0,0,0,0,
	0,'a','b','c','d','e','f','g','h','i','j','k','l','m','n','o',
	'p','q','r','s','t','u','v','w','x','y','z',0,0,0,0,0,
	0,'a','b','c','d','e','f','g','h','i','j','k','l','m','n','o',
	'p','q','r','s','t','u','v','w','x','y','z',0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,'',0,'',0,'','',0,0,'','','','',0,'','',
	0,'',0,'',0,'','',0,0,'','','','',0,'','',
	'','','','','','','','','','','','','','','','',
	'','','','','','','',0,'','','','','','','','',
	'','','','','','','','','','','','','','','','',
	'','','','','','','',0,'','','','','','','',0};

static char uci[256]={
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,1,0,1,0,1,1,0,0,1,1,1,1,0,1,1,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	
#define STRBLOCK_SIZE 16300
static struct strblock {
	struct strblock *next;
	int pos_str,pos_txt;
	char buf[STRBLOCK_SIZE];
} *memo;

static struct slowo {
	struct slowo *l,*r;
	int flags;
	char *str;
} *slowa;

static void alloc_block(void)
{
	struct strblock *s=malloc(sizeof(*s));
	s->next=memo;
	memo=s;
	s->pos_txt=STRBLOCK_SIZE;
	s->pos_str=0;
}

static char *dups(char *c)
{
	int l=strlen(c)+1;
	if (memo->pos_str+l>memo->pos_txt) alloc_block();
	memo->pos_txt-=l;
	strcpy(memo->buf+memo->pos_txt,c);
	return memo->buf+memo->pos_txt;
}

static int dash_word_num;

static struct slowo *allocword(void)
{
	
	struct slowo *s;
	if (memo->pos_str+sizeof(struct slowo)>memo->pos_txt) alloc_block();
	s=(struct slowo *)(memo->buf+memo->pos_str);
	memo->pos_str+=sizeof(struct slowo);
	return s;
}

static void insert_word(struct slowo **s,char *str,int flags)
{
	while (*s) {
		int n=strcmp((*s)->str,str);
		if (n<0) s=&(*s)->l;
		else if (n>0) s=&(*s)->r;
		else break;
	}
	if (!*s) {
		dash_word_num++;
		*s=allocword();
		(*s)->str=str;
		(*s)->l=(*s)->r=NULL;
		(*s)->flags=flags;
	}
	else (*s)->flags |= flags;
}

static int get_dash_word(char **str)
{
	char *s;
	int flags;
	if (!**str) return 0;
	flags=0;
	for (;**str;(*str)++) {
		if (lci[(**str) & 255]) break;
	}
	if (!**str) return 0;
	s=*str;
	for (;**str;(*str)++) {
		if (lci[(**str) & 255]) {
			**str=lci[(**str) & 255];
			continue;
		}
		if (**str !='\'' && **str != '-') break;
		if (!lci[(*str)[1] & 255]) break;
		if (**str=='-') flags=1;
	}
	if (**str) {
		*(*str)++=0;
	}
	if (flags) insert_word(&slowa,s,0);
	return 1;
	
}

static char *dash_word_memo;
static int dash_word_len,dash_word_size;

static void append_buf(char *buf,int len)
{
	if (len<0) len=strlen(buf);
	if (dash_word_len+len >= dash_word_size-1) {
		dash_word_size+=8192;
		dash_word_memo=realloc(dash_word_memo,dash_word_size);
	}
	memcpy(dash_word_memo+dash_word_len,buf,len);
	dash_word_len+=len;
}

static void dash_walk(struct slowo *s)
{
	char buf[1024];
	int n;
	while (s) {
		dash_walk(s->r);
		strcpy(buf,s->str);
		strcat(buf,"\n");
		append_buf(buf,-1);
		s=s->l;
	}
}

extern int ignore_oor;
char *get_current_iso2(void)
{
	int n;
	char *str,*istr;
	n=get_current_editor();
	if (n<0) return NULL;

	if (n) {
		Error("Błędne polecenie","Polecenie nie jest możliwe dla trybów DASH i DIC");
		return NULL;
	}
	if (!gtk_text_buffer_get_char_count(tresc)) return NULL;
	str=get_actual_body();
	ignore_oor=1;
	n=to_iso2(str,NULL);
	if (!n) {
		g_free(str);
		return NULL;
	}
	istr=malloc(n);
	to_iso2(str,istr);
	g_free(str);
	return istr;
}


void make_dash(GtkWidget *dummy, void *data)
{
	 
	struct strblock *s;
	struct MyGtkEditor *edi;
	char *bstr,*str=get_current_iso2();
	if (!str) return;
	memo=NULL;
	slowa=NULL;
	alloc_block();
	bstr=str;
	dash_word_num=0;
	while(get_dash_word(&bstr));
	if (!dash_word_num) {
		Info("Informacja","Brak podejrzanych myślników");
		free(str);
		return;
	}
	dash_word_size=8192;
	dash_word_memo=malloc(dash_word_size);
	dash_word_len=0;
	dash_walk(slowa);
	free(str);
	str=to_utf8(dash_word_memo,dash_word_len,"ISO-8859-2",1);
	while (s=memo) {
		memo=memo->next;
		free(s);
	}		
	edi=get_subeditor(current_editor,MILEDIT_MODE_DASH);
	set_buffer_text(edi->buf,str,-1,0);
	free(str);
}

static char **words,*wrdbdy;
static int wordcnt,wordsiz;

static int morfologik_rbase(void)
{
	int fd,i;
	size_t len;
	char *bdy;
	struct stat sb;

	char fname[PATH_MAX+1];
	sprintf(fname,"%s-words/pl_basewords.dat",milena_GetDataPath());
	if (wordcnt) return 1;
	if (stat(fname,&sb)) {
		g_perror(fname);
		return 0;
	}
	len=sb.st_size;
	fd=open(fname,O_RDONLY);
	if (fd<0) {
		g_perror(fname);
		return 0;
	}
	
	bdy=malloc(len);
	if (read(fd,bdy,len)!=len) {
		free(bdy);
		g_perror(fname);
		return 0;
	}
	close(fd);
	wrdbdy=bdy;
	wordcnt=wordsiz=strtol(bdy,NULL,10);
	bdy+=10;
	words=malloc(wordcnt*sizeof(*words));
	for (i=0;i<wordcnt;i++) {
		words[i]=bdy;
		bdy+=strlen(bdy)+1;
	}
	return 1;
}
	
static int morfologik_find(char *c)
{
	int lo,hi,mid,n;
	lo=0;hi=wordcnt-1;
	while (lo<=hi) {
		mid=(lo+hi)/2;
		n=strcmp(words[mid],c);
		if (!n) return 1;
		if (n>0) hi=mid-1;
		else lo=mid+1;
	}
	return 0;
}

static struct dict {
	struct dict *next;
	char *wrd;
	char *trans;
	int mode;
} *sdict,**ssdict;


static char *trans(char *str)
{
	static char buf[8192],buf2[8192];
	int dummy;
	milena_GetPhrase(milena,&str,buf,8192,&dummy);
	milena_Prestresser(milena,buf,buf2,8192);
	milena_TranslatePhrase(milena,(unsigned char *)buf2,buf,8192,0);
	milena_Poststresser(buf,buf2,8192);
	return buf2;
}

static void outtrans(struct slowo *s)
{
	char buf[8192];
	strcpy(buf,s->str);
	strcat(buf," //");
	strcat(buf,trans(s->str));
	strcat(buf,"\n");
	append_buf(buf,-1);
}

static void walk_nomorf(struct slowo *s)
{
	struct dict *d;
	while (s) {
		walk_nomorf(s->r);
		for (d=sdict;d;d=d->next) if (!strcmp(d->wrd,s->str)) break;
		if (d || milena_IsIgnoredWord(milena,s->str)) {
			s->flags |= 16;
		}
		else {
			if (!morfologik_find(s->str)) {
				outtrans(s);
				s->flags |= 16;
			}
		}
		s=s->l;
	}
}
static void walk(struct slowo *s)
{
	static char *pstr="qvx'-";
	static char *estr="aeiouy";
	struct dict *d;
	while (s) {
		walk(s->r);
		if (!(s->flags & 16)) {
			for (d=sdict;d;d=d->next) if (!strcmp(d->wrd,s->str)) break;
			if (!d) {
				if ((s->flags & 3)==1 || strpbrk(s->str,pstr) || !strpbrk(s->str,estr)) {
					outtrans(s);
				}
			}
		}
		s=s->l;
	}
}

static int get_word(char **str)
{
	char *s;
	int flags;
	int bof;
	if (!**str) return 0;
	if (!(bof=milena_SkipToUnknownWord(milena,str))) {
		return 0;
	}
	s=*str;
	if (bof==2) flags=0;
	else if (uci[(**str) & 255]) flags=1;
	else flags = 2;
	for (;**str;(*str)++) {
		if (lci[(**str) & 255]) {
			if (isdigit(**str)) flags |= 4;
			else flags |= 8;
			**str=lci[(**str) & 255];
			continue;
		}
		if (**str !='\'' && **str != '-') break;
		if (!lci[(*str)[1] & 255]) break;
	}
	bof=0;
	if (**str) {
		if (strchr(".?!\n",**str)) bof=1;
		*(*str)++=0;
	}
	if ((flags & 12)==4) return 1;
	if (strlen(s)<3) return 1;
	insert_word(&slowa,s,flags);
	return 1;
	
}

static struct dict *allocdict(void)
{
	
	struct dict *s;
	if (memo->pos_str+sizeof(struct dict)>memo->pos_txt) alloc_block();
	s=(struct dict *)(memo->buf+memo->pos_str);
	memo->pos_str+=sizeof(struct dict);
	return s;
}

void read_dict(char *line,int mode)
{
	char *c,*c1,*c2;
	struct dict *d;
	c=strstr(line,"//");
	if (c) {*c++=0;*c++=0;}
	while (*line && isspace(*line)) line++;
	if (!*line) return;
	for (c=line;*c && !isspace(*c); c++);
	*c++=0;
	while (*c && isspace(*c)) c++;
	for (c1=c2=c;*c1;c1++) if (!isspace(*c1)) c2=c1+1;
	*c2=0;
	d=allocdict();
	d->next=NULL;
	*ssdict=d;
	ssdict=&d->next;
	d->wrd=dups(line);
	d->trans=NULL;
	d->mode=mode;
	if (mode && c && *c) d->trans=dups(c);
}

static int read_dic_file(char *fname,int mode)
{
	FILE *f;
	char buf[1024];
	f=fopen(fname,"r");
	if(!f) {
		g_perror(fname);
		return 0;
	}
	if (mode) {
		if (fgets(buf,1024,f)) {
			if (!auto_set_langs(buf)) read_dict(buf,mode);
		}
	}
	while(fgets(buf,1024,f)) read_dict(buf,mode);
	fclose(f);
	return 1;
}

static void free_memo(void)
{
	struct strblock *s;
	while (s=memo) {
		memo=memo->next;
		free(s);
	}
}

static void outdic(struct dict *d)
{
	char buf[8192];
	strcpy(buf,d->wrd);
	strcat(buf," ");
	if (d->trans) {
		strcat(buf,d->trans);
	}
	strcat(buf,"//");
	strcat(buf,trans(d->wrd));
	strcat(buf,"\n");
	append_buf(buf,-1);
}

void make_dict(GtkWidget *dummy, void *data)
{
	 
	struct strblock *s;
	struct MyGtkEditor *edi;
	char *bstr,*str;
	char buf[1024];
	char *extra=NULL;
	struct dict *d;
	int i,nw;struct miltheme *mt;
	
	if (get_current_editor() != 0) return;
	if (current_editor->dic_editor) {
		if (!Ask("Pytanko","Słownik już istnieje, czy mam go zastąpić?")) return;
	}
	if (data) {
		GtkWidget *dialog=gtk_file_chooser_dialog_new("Wybierz plik dodatkowy",
			GTK_WINDOW(main_window),GTK_FILE_CHOOSER_ACTION_OPEN,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			NULL);
		GtkFileFilter *filter;
		gtk_file_chooser_set_local_only((gpointer)dialog,TRUE);
		gtk_file_chooser_add_filter((gpointer) dialog,
			make_filter("Pliki DIC","*.dic",NULL));
		gtk_file_chooser_add_filter((gpointer) dialog,
			make_filter("Pliki tekstowe","*.txt","*.TXT",NULL));
		gtk_file_chooser_add_filter((gpointer) dialog,
			filter=make_filter("Wszystkie pliki","*",NULL));
		gtk_file_chooser_set_filter((gpointer) dialog,filter);
		char *cs=gtk_file_chooser_get_filename((gpointer)dic_filechooser);
		if (!cs || !*cs) {
			if (last_file_path[0]) {
				cs=last_file_path;
			}
		}
		if (cs && *cs) gtk_file_chooser_set_filename((gpointer)dialog,cs);
		if (gtk_dialog_run((gpointer)dialog)==GTK_RESPONSE_ACCEPT) {
			extra=gtk_file_chooser_get_filename((gpointer)dialog);
		}
		gtk_widget_destroy(dialog);
		if (!extra || !*extra) return;
	}
	else {
		if (!Ask("Pytanko","Na pewno? To potrwa kilka minut.")) return;
	}
	if (!init_milena_libs(1)) {
		Error("Hm...","Milena niezainicjalizowana");
		return;
	}
	if (!(str=get_current_iso2())) {
		Error("Hm...","Brak Current ISO2");
		close_milena_libs();
		return;
	}
	start_busy_cursor();
	memo=NULL;
	slowa=NULL;
	alloc_block();
	sdict=NULL;
	ssdict=&sdict;
	if (!read_dic_file(milena_FilePath("pl_dict.dat",buf),0)) {
		free(str);
		close_milena_libs();
		free_memo();
		end_busy_cursor();
		return;
	}
	if (extra && *extra) {
		if (!read_dic_file(extra,1)) {
			free(str);
			close_milena_libs();
			free_memo();
			end_busy_cursor();
			return;
		}
		if (!milena_ReadUserDicWithFlags(milena,extra,MILENA_UDIC_DICTMODE)) {
			free(str);
			close_milena_libs();
			free_memo();
			end_busy_cursor();
			return;
		}
	}
	if (!morfologik_rbase()) {
		free(str);
		close_milena_libs();
		free_memo();
		end_busy_cursor();
		return;
	}
	bstr=str;
	dash_word_num=0;
	while(get_word(&bstr));

	dash_word_size=8192;
	dash_word_memo=malloc(dash_word_size);
	dash_word_len=0;

	strcpy(buf,"//#LANGS:");
	for (i=nw=0;i<NUM_LANGS;i++) if (gtk_toggle_button_get_active((gpointer)lang_cb[i])) {
		if (nw) strcat(buf,",");
		nw++;
		strcat(buf,lang_codes[i]);
	}
	strcat(buf,"\n//#THEMES:");
	for (mt=themes,nw=0;mt;mt=mt->next) if (gtk_toggle_button_get_active((gpointer)mt->cb)) {
		if (nw) strcat(buf,",");
		nw++;
		strcat(buf,mt->tname);
	}
	strcat(buf,"\n\n");
	append_buf(buf,-1);
	if (extra && *extra) {
		nw=0;
		for (d=sdict;d;d=d->next) {
			if (!d->mode) continue;
			if (d->trans) {
				if (!nw) {
					char *c;
					c=strrchr(extra,'/');
					if (c) c++;else c=extra;
					sprintf(buf,"// KOPIA WYMOWY z %s\n\n",c);
					append_buf(buf,-1);
					nw=1;
				}
				outdic(d);
			}
		}
		append_buf("\n\n",-1);
	}


	append_buf("// BRAK W MORFOLOGIKU\n\n",-1);
	walk_nomorf(slowa);
	append_buf("\n\n// WYSTEPUJA W MORFOLOGIKU\n\n",-1);
	walk(slowa);


	if (extra && *extra) {
		int nw=0;
		for (d=sdict;d;d=d->next) {
			if (!d->mode) continue;
			if (!d->trans) {
				if (!nw) {
					char *c;
					c=strrchr(extra,'/');
					if (c) c++;else c=extra;
					sprintf(buf,"\n\n// POMIJANE z %s\n\n",c);
					append_buf(buf,-1);
					nw=1;
				}
				outdic(d);
			}
		}
	}

	close_milena_libs();
	free(str);
	str=to_utf8(dash_word_memo,dash_word_len,"ISO-8859-2",1);
	free_memo();
	edi=get_subeditor(current_editor,MILEDIT_MODE_DIC);
	set_buffer_text(edi->buf,str,-1,0);
	free(str);
	end_busy_cursor();
	
}

static char *find_dash(char *beg,char *end,char *dash)
{
	for (;;) {
		char *c,*d;
		while (*dash && isspace(*dash)) dash++;
		if (*dash == '#') {
			dash=strchr(dash,'\n');
			if (!dash) return NULL;
			continue;
		}
		if (!*dash) return NULL;
		for (c=beg,d=dash;c<end;c++,d++) {
			if (!*d) break;
			if (*c=='\'' && *d=='\'') continue;
			if (*c=='-' && (*d=='~' || *d=='-')) continue;
			if (lci[(*c)&255] != lci[(*d)&255]) break;
		}
		if (c==end) {
			if (!*d || my_space(*d)) return dash;
		}
		while (*dash && !my_space(*dash)) dash++;
	}
		
}

static char *dash_output;
static int dash_output_size;
static int dash_output_length;

static void dash_put_char(int c)
{
	if (dash_output_length>dash_output_size-2) {
		dash_output_size+=8192;
		dash_output=realloc(dash_output,dash_output_size);
	}
	dash_output[dash_output_length++]=c;
}



void apply_dash(GtkWidget *widget,void *dummy)
{
	char *str=get_current_iso2(),*bdy,*ibdy;
	int n;
	struct MyGtkEditor *edi;
	char *ic,*sc,*c;int ws;
	if (!str) return;
	edi=current_editor->dash_editor;
	if (!edi) {
		Error("Błąd","Utwórz DASH i usuń nieprawidłowe");
		free(str);
		return;
	}
	bdy=get_editor_body(edi);
	ignore_oor=1;
	n=to_iso2(bdy,NULL);
	if (!n) ibdy=strdup("");
	else {
		ibdy=malloc(n);
		to_iso2(bdy,ibdy);
	}
	g_free(bdy);
	ic=str;
	dash_output_size=8192;
	dash_output=malloc(dash_output_size);
	dash_output_length=0;
	while (*ic) {
		if (!my_alnum(*ic)) {
			dash_put_char(*ic++);
			continue;
		}
		sc=ic;
		ws=0;
		while (*sc) {
			if (my_alnum(*sc)) {
				sc++;
				continue;
			}
			if (*sc != '-' && *sc != '\'') break;
			if (!sc[1] || !my_alnum(sc[1])) break;
			if (*sc=='-') ws=1;
			sc++;
		}
		if (!ws) {
			while (ic<sc) dash_put_char(*ic++);
			continue;
		}
		c=find_dash(ic,sc,ibdy);
		if (!c) {
			while (ic<sc) {
				if (*ic != '-') dash_put_char(*ic);
				ic++;
			}
			continue;
		}
		while (ic < sc) {
			if (*c=='+') {
				dash_put_char(' ');
				dash_put_char('-');
				dash_put_char(' ');
			}
			else if (*c != '~') dash_put_char(*ic);
			ic++;
			c++;
		}
	}
	free(str);
	free(ibdy);
	str=to_utf8(dash_output,dash_output_length,"ISO-8859-2",1);
	set_buffer_text(current_editor->buf,str,-1,1);
	free(str);
	remove_subeditor(current_editor->dash_editor);
	current_editor->dash_editor=NULL;
}

