/*
 * editor.c - Milena Audiobook Creator
 * Copyright (C) Bohdan R. Rau 2009 <ethanak@polip.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write see:
 *               <http://www.gnu.org/licenses/>.
 */
#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>
#include "gmilena.h"
#include <gtksourceview/gtksourceiter.h>
#include <gdk/gdkkeysyms.h>
#include <pcre.h>
#include <glib.h>
#include <glib/gunicode.h>

#define F_ALONE 0x01
#define F_EMPTY 0x02
#define F_DIALOG 0x04
#define F_SPLIT 0x08
#define F_HYPHEN 0x10
#define F_SHORT 0x20

#define F_NONE 0x100
#define F_ENTER 0x200
#define F_SPACE 0x400



extern char *to_utf8(char *fbuf,int flen,char *cset,int freeme);
extern int ignore_oor;
static GtkWidget *find_dialog;
static GtkWidget *fd_entry_find,*fd_entry_replace;
static GtkWidget *fd_cb_regexp,*fd_cb_nocase,*fd_cb_word;

char *get_actual_body(void)
{
	GtkTextIter start,end;
	gtk_text_buffer_get_start_iter((gpointer)tresc,&start);
	gtk_text_buffer_get_end_iter((gpointer)tresc,&end);
	return gtk_text_buffer_get_text((gpointer)tresc,&start,&end,FALSE);
	
}


char *get_editor_body(struct MyGtkEditor *editor)
{
	GtkTextIter start,end;
	gtk_text_buffer_get_start_iter((gpointer)editor->buf,&start);
	gtk_text_buffer_get_end_iter((gpointer)editor->buf,&end);
	return gtk_text_buffer_get_text((gpointer)editor->buf,&start,&end,FALSE);
}

void convert_to_iso2(GtkWidget *widget,void *dummy)
{
	int n;
	n=get_current_editor();
	if (n<0) return;
	if (n) {
		Error("Błędne polecenie","Konwersja nie jest możliwa dla trybów DASH i DIC");
		return;
	}
	if (!gtk_text_buffer_get_char_count(tresc)) return;
	char *ustr,*istr,*str=get_actual_body();
	ignore_oor=2;
	n=to_iso2(str,NULL);
	if (!n) {
		g_free(str);
		return;
	}
	istr=malloc(n);
	ignore_oor=2;
	to_iso2(str,istr);
	ustr=to_utf8(istr,strlen(istr),"ISO-8859-2",1);
	if (!strcmp(str,ustr)) {
		Info("Informacja","Plik nie zawierał znaków spoza ISO2");
	}
	else {
		set_buffer_text(tresc,ustr,-1,1);
		if (count_bad_char) {
			char buf[256];
			sprintf(buf,"Konwersja wykonana, nieskonwertowane znaki: %d",count_bad_char);
			Info ("Informacja",buf);
		}
		else Info("Informacja","Konwersja wykonana");
	}
	g_free(str);
	free(ustr);
	return;
}

static char *unformat(char *str,int no_pages,int autopar)
{
	char *istr,*ostr,*mstr,*lch,*body;
	int i,j,nline,nenter;
	struct {
		int flags;
		char *buf;
	} *linie;
	ostr=mstr=body=str;
	
	for (;*body && my_space(*body);body++);
	for (istr=body,nline=1;*istr;istr++) {
		if (*istr=='\r') {
			if (istr[1]!='\n') nline++;
			continue;
		}
		if (*istr=='\n') nline++;
	}
			
	linie=calloc(nline,sizeof(*linie));
	istr=body;nline=0;
	while (*istr) {
		while (*istr && *istr != '\n' && *istr !='\r' && my_space(*istr)) istr++;
		if (!*istr) break;
		linie[nline].flags=0;
		linie[nline++].buf=ostr;
		lch=ostr;
		
		while (*istr && *istr != '\n' && *istr !='\r') {
			if (!my_space(*istr)) lch=ostr+1;
			*ostr++=*istr++;
		}
		if (*istr=='\r') istr++;
		if (*istr=='\n') istr++;
		*lch=0;
		ostr=lch+1;
	}
	if (no_pages) {
		
		for (i=j=0;i<nline;i++) {
			char *c=linie[i].buf;
			if (!*c) goto loop;
			for (;*c;c++) if (!my_digit(*c)) goto loop;
			for (;j>0 && !*linie[j-1].buf;j--);
			while (i<nline-1 && !*linie[i+1].buf) i++;
			continue; 
		loop:	linie[j].flags = linie[i].flags;
			linie[j++].buf=linie[i].buf;
		}
		nline=j;
	}
	i=0;
	for (;i<nline;i++) {
		int len;
		if (!*linie[i].buf) {
			linie[i].flags = F_EMPTY;
			continue;
		}
		if (!readable(linie[i].buf)) {
			linie[i].flags = F_ALONE;
			continue;
		}
		if (linie[i].buf[0]=='-') linie[i].flags |= F_DIALOG;
		len=strlen(linie[i].buf);
		if (autopar && len <= autopar) {
			linie[i].flags |= F_SHORT;
		}
		else if (i<nline-1 && len>10 && (linie[i].buf[len-1] & 255)==0xad) {
			linie[i].buf[len-1]=0;
			linie[i].flags |= F_HYPHEN;
		}
		else if (i<nline-1 && len>10 && linie[i].buf[len-1]=='-') {
			int ns,tr;
			ns=len-2;
			if (my_alpha(linie[i].buf[ns]) && my_alpha(linie[i+1].buf[0])) {
				tr=0;
				for (;ns>=0;ns--) {
					if (!my_alnum(linie[i].buf[ns])) break;
					if (!my_alpha(linie[i].buf[ns])) {
						tr=1;
						break;
					}
				}
				if (!tr) {
					for (ns=0;linie[i+1].buf[ns];ns++) {
						if (!my_alnum(linie[i+1].buf[ns])) break;
						if (!my_alpha(linie[i+1].buf[ns])) {
							tr=1;
							break;
						}
					}
				}
				if (!tr) {
					linie[i].buf[len-1]=0;
					linie[i].flags |= F_HYPHEN;
				}
			}
		}
	}
	for (i=0;i<nline-1;i++) {
		char *c,*d;
		for (c=d=linie[i].buf;*c;c++) if ((*c & 255)!=0xad) *d++=*c;
		*d=0;
		if (linie[i+1].flags & (F_ALONE|F_EMPTY|F_DIALOG|F_SPLIT)) {
			linie[i].flags |= F_ENTER;
			if (!autopar && (linie[i+1].flags & F_EMPTY)) {
				linie[i+1].flags |= F_NONE;
				if (i<nline-2 && linie[i+2].flags & F_EMPTY) {
					linie[i+1].flags |= F_ENTER;
					i++;
				}
			}
			continue;
		}
		if (linie[i].flags & (F_ALONE | F_EMPTY | F_SPLIT | F_SHORT)) {
			linie[i].flags |= F_ENTER;
			continue;
		}
		linie[i].flags |= F_SPACE;
	}
	linie[i].flags |= F_ENTER;
	nenter=0;
	for (i=0;i<nline;i++) {
		char *c=linie[i].buf;
		if (linie[i].flags & F_NONE) continue;
		while (*c) *mstr++=*c++;
		if (linie[i].flags & F_ENTER) {
			if (linie[i].flags & F_EMPTY) {
				if (nenter) continue;
				nenter++;
			}
			else nenter=0;
			*mstr++='\n';
		}
		else {
			nenter=0;
			if ((linie[i].flags & (F_SPACE | F_HYPHEN))==F_SPACE) *mstr++=' ';
		}
	}
	*mstr=0;
	free(linie);
}

static GtkWidget *cbx_autonum;
static GtkWidget *spin_wiersz;

static void set_minwiersz_active(void)
{
	gtk_widget_set_sensitive(spin_wiersz,
		gtk_toggle_button_get_active((gpointer)cbx_autonum));
}

void correct_jacosub(GtkWidget *dummy,void *data)
{
	char *str=get_current_iso2(),buf[64],*str2;
	if (!str) return;
	int timeres=jacosub_timeres(str);
	if (!timeres) {
		Error("Hm...","To nie wygląda na plik Jacosub");
		free(str);
		return;
	}
	sprintf(buf,"Korygujesz Jacosub z TIMERES %d?",timeres);
	if (!Ask("Pytanko",buf)) {
		free(str);
		return;
	}
	str2=jacosub_simplify(str,timeres);
	if (str2) {
		str2=to_utf8(str2,strlen(str2),"ISO-8859-2",1);
		set_buffer_text(current_editor->buf,str2,-1,1);
		free(str2);
	}
	free(str);
}

void unformat_text(GtkWidget *dummy, void *data)
{
	
	char *str=get_current_iso2();
	if (!str) return;
	GtkWidget *dialog=gtk_dialog_new_with_buttons(
		"Ustaw odformatowanie",
		(gpointer)main_window,
		GTK_DIALOG_MODAL,
		GTK_STOCK_OK,GTK_RESPONSE_ACCEPT,
		GTK_STOCK_CANCEL,GTK_RESPONSE_REJECT,
		NULL);
	GtkWidget *box=gtk_dialog_get_content_area((gpointer)dialog);
	GtkWidget *vbox=gtk_vbox_new(0,2);
	gtk_box_pack_start(GTK_BOX(box),(gpointer)vbox,FALSE,FALSE,0);
	
	GtkWidget *cbx_autopages=gtk_check_button_new_with_label("Usuń numerację stron");
	gtk_box_pack_start(GTK_BOX(vbox),cbx_autopages,FALSE,FALSE,0);
	GtkWidget *hbox=gtk_hbox_new(0,2);
	gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0);
	cbx_autonum=gtk_check_button_new_with_label("Minimalna długość wiersza");
	gtk_box_pack_start(GTK_BOX(hbox),cbx_autonum,FALSE,FALSE,0);
	gtk_toggle_button_set_active((gpointer)cbx_autonum,TRUE);
	spin_wiersz=gtk_spin_button_new_with_range(20.0,120.0,1.0);
	gtk_spin_button_set_value((gpointer)spin_wiersz,50);
	atk_object_set_name(gtk_widget_get_accessible((gpointer)spin_wiersz),"Minimalna długość wiersza");
	atk_object_set_description(gtk_widget_get_accessible((gpointer)spin_wiersz),"Ustawia minimalną długość wiersza, poniżej której będzie on traktowany jako kończący akapit");
	//connect_label(cbx_autonum,spin_wiersz);
	g_signal_connect(G_OBJECT(cbx_autonum),"toggled",G_CALLBACK(set_minwiersz_active),NULL);
	gtk_box_pack_start(GTK_BOX(hbox),spin_wiersz,FALSE,FALSE,0);
	gtk_widget_show_all(box);
	int n=gtk_dialog_run((gpointer)dialog);
	if (n!=GTK_RESPONSE_ACCEPT) {
		gtk_widget_destroy(dialog);
		free(str);
		return;
	}
	
	int autopar=0;
	int nopages=gtk_toggle_button_get_active((gpointer)cbx_autopages);
	if (gtk_toggle_button_get_active((gpointer)cbx_autonum)) {
		autopar=gtk_spin_button_get_value_as_int((gpointer)spin_wiersz);
	}
	gtk_widget_destroy(dialog);
	unformat(str,nopages,autopar);
	str=to_utf8(str,strlen(str),"ISO-8859-2",1);

	set_buffer_text(current_editor->buf,str,-1,1);
	free(str);
	return;
}

static GtkWidget *search_dialog;
static GtkWidget *search_entry,*replace_label,*replace_entry;
static GtkWidget *search_regex,*search_case,*search_word,*search_bword,*search_eword,*search_backward,*search_extw;
static GtkWidget *search_replace_selection,*replace_box;
static int editor_do_replace,search_dialog_return_code;

static int words_extended;

static int iter_starts_word(GtkTextIter *iter)
{
	if (gtk_text_iter_starts_line(iter)) return 1;
	GtkTextIter sword=*iter;
	if (!gtk_text_iter_backward_char(&sword)) return 1;
	gunichar z=gtk_text_iter_get_char(&sword);
	if (g_unichar_isalnum(z)) return 0;
	if (!words_extended) return 1;
	if (z != '\'' && z!='-') return 1;
	if (!gtk_text_iter_backward_char(&sword)) return 1;
	z=gtk_text_iter_get_char(&sword);
	return !g_unichar_isalnum(z);
}

static int iter_ends_word(GtkTextIter *iter)
{
	if (gtk_text_iter_ends_line(iter)) return 1;
	GtkTextIter sword=*iter;
	//if (!gtk_text_iter_backward_char(&sword)) return;
	gunichar z=gtk_text_iter_get_char(&sword);
	if (g_unichar_isalnum(z)) return 0;
	if (!words_extended) return 1;
	if (z != '\'' && z!='-') return 1;
	if (!gtk_text_iter_forward_char(&sword)) return 1;
	z=gtk_text_iter_get_char(&sword);
	if (!z) return 1;
	return !g_unichar_isalnum(z);
}

int search_in_buffer(struct MyGtkEditor *editor,char *str,int insens,int bkward,int as_word)
{
	struct MyGtkEditor *edi;
	GtkTextIter pos,start,end;
	if (editor) edi=editor;
	else {
		get_current_editor();
		edi=current_editor;
	}
	GtkTextMark *sel;
	if (bkward) {
		sel=gtk_text_buffer_get_insert(edi->buf);
	}
	else {
		sel=gtk_text_buffer_get_selection_bound(edi->buf);
	}
	gtk_text_buffer_get_iter_at_mark(edi->buf,&pos,sel);
	int flags = GTK_SOURCE_SEARCH_TEXT_ONLY | GTK_SOURCE_SEARCH_VISIBLE_ONLY;
	if (insens) flags |= GTK_SOURCE_SEARCH_CASE_INSENSITIVE;
	words_extended=gtk_toggle_button_get_active((gpointer)search_extw);
	for(;;) {
		if (bkward) {
			if (!gtk_source_iter_backward_search(&pos,str,flags,&start,&end,NULL)) return 0;
		}
		else {
			if (!gtk_source_iter_forward_search(&pos,str,flags,&start,&end,NULL)) return 0;
		}
		if (!as_word) break;
		if ((as_word & 1) && !iter_starts_word(&start)) {
			if (bkward) pos=start;else pos=end;
			continue;
		}
		if ((as_word & 2) && !iter_ends_word(&end)) {
			if (bkward) pos=start;else pos=end;
			continue;
		}
		break;
	}
	gtk_text_buffer_select_range(edi->buf,&start,&end);
	gtk_text_view_scroll_to_iter(edi->view,&start,0.2,FALSE,0.0,0.0);
	return 1;
}



struct editor_history {
	struct editor_history *succ,*pred;
	int flags;
	char *search;
	char *replace;
} editor_history={&editor_history,&editor_history,0x80};

static void insert_editor_history(int is_replace)
{
	struct editor_history *eh;
	char const *str=gtk_entry_get_text((gpointer)search_entry);
	char const *rep=gtk_entry_get_text((gpointer)replace_entry);
	int flags = 0;
	if (gtk_toggle_button_get_active((gpointer)search_regex)) flags |=1;
	if (gtk_toggle_button_get_active((gpointer)search_word)) flags |=2;
	if (gtk_toggle_button_get_active((gpointer)search_bword)) flags |=4;
	if (gtk_toggle_button_get_active((gpointer)search_eword)) flags |=8;
	if (gtk_toggle_button_get_active((gpointer)search_case)) flags |=16;
	if (gtk_toggle_button_get_active((gpointer)search_extw)) flags |=32;
	for (eh=editor_history.pred;eh && eh->flags != 0x80;eh=eh->succ) {
		if (eh->flags != flags) continue;
		if (strcmp(eh->search,str)) continue;
		if (is_replace) {
			if (strcmp(eh->replace,rep)) continue;
		}
		break;
	}
	if (eh && eh->flags != 0x80) {
		eh->succ->pred=eh->pred;
		eh->pred->succ=eh->succ;
	}
	else {
		eh=malloc(sizeof(*eh));
		eh->search=strdup(str);
		if (is_replace) {
			eh->replace=strdup(rep);
		}
		else {
			eh->replace=strdup("");
		}
		eh->flags=flags;
	}
	eh->succ=editor_history.succ;
	eh->pred=&editor_history;
	eh->succ->pred=eh;
	eh->pred->succ=eh;
}

static struct editor_history *editor_history_pointer;

static void next_editor_history(int dir)
{
	struct editor_history *eh;
	if (dir) {
		eh=editor_history_pointer->succ;
	}
	else {
		eh=editor_history_pointer->pred;
	}
	if (eh->flags & 0x80) return;
	editor_history_pointer=eh;
	gtk_entry_set_text((gpointer)search_entry,eh->search);
	gtk_entry_set_text((gpointer)replace_entry,eh->replace);
	gtk_toggle_button_set_active((gpointer)search_regex,(eh->flags & 1)?1:0);
	gtk_toggle_button_set_active((gpointer)search_word,(eh->flags & 2)?1:0);
	gtk_toggle_button_set_active((gpointer)search_bword,(eh->flags & 4)?1:0);
	gtk_toggle_button_set_active((gpointer)search_eword,(eh->flags & 8)?1:0);
	gtk_toggle_button_set_active((gpointer)search_case,(eh->flags & 16)?1:0);
	gtk_toggle_button_set_active((gpointer)search_extw,(eh->flags & 32)?1:0);
	if (eh->flags & 1) {
		gtk_toggle_button_set_active((gpointer)search_backward,FALSE);
	}
}


static void set_searcher_active(GtkWidget *widget,void *data)
{
	/*
	if (widget == (gpointer) search_backward) {
		if (gtk_toggle_button_get_active((gpointer)search_backward)) {
			gtk_toggle_button_set_active((gpointer)search_regex,0);
		}
	}
	if (widget == (gpointer) search_regex) {
		if (gtk_toggle_button_get_active((gpointer)search_regex)) {
			gtk_toggle_button_set_active((gpointer)search_backward,0);
		}
	}
	gtk_widget_set_sensitive(search_regex,
		!gtk_toggle_button_get_active((gpointer)search_backward));
	gtk_widget_set_sensitive(search_backward,
		!gtk_toggle_button_get_active((gpointer)search_regex));
						      */
	int n;
	n= gtk_toggle_button_get_active((gpointer)search_bword) |
		gtk_toggle_button_get_active((gpointer)search_eword);
	gtk_widget_set_sensitive(search_word,!n);
	n= gtk_toggle_button_get_active((gpointer)search_word) |
		gtk_toggle_button_get_active((gpointer)search_eword);
	gtk_widget_set_sensitive(search_bword,!n);
	n= gtk_toggle_button_get_active((gpointer)search_bword) |
		gtk_toggle_button_get_active((gpointer)search_word);
	gtk_widget_set_sensitive(search_eword,!n);
	
}


static int search_key_press(GtkWidget *widget,GdkEventKey *event,void *data)
{
	int sym=event->keyval;
	if (sym == GDK_Escape) {
		gtk_dialog_response((gpointer)search_dialog,GTK_RESPONSE_REJECT);
		return 1;
	}
	if (sym == GDK_Up) {
		//printf("Up\n");
		next_editor_history(1);
		return 1;
	}
	if (sym == GDK_Down) {
		//printf("Down\n");
		next_editor_history(0);
		return 1;
	}
	if ((sym == GDK_Return || sym == GDK_KP_Enter) && !editor_do_replace) {
		gtk_dialog_response((gpointer)search_dialog,GTK_RESPONSE_ACCEPT);
		return 1;
	}
	
	return 0;
}

static void set_searcher_return(GtkWidget *widget,void *dummy)
{
	gtk_dialog_response((gpointer)search_dialog,(int)dummy);
}

static void create_search_dialog(void)
{
	if (search_dialog) return;
	search_dialog=gtk_dialog_new_with_buttons(
		"Szukaj",
		(gpointer)main_window,
		GTK_DIALOG_MODAL,
		GTK_STOCK_OK,GTK_RESPONSE_ACCEPT,
		GTK_STOCK_CANCEL,GTK_RESPONSE_REJECT,
		NULL);
	GtkWidget *box=gtk_dialog_get_content_area((gpointer)search_dialog);
	GtkWidget *vbox=gtk_vbox_new(0,2);
	gtk_box_pack_start(GTK_BOX(box),(gpointer)vbox,FALSE,FALSE,0);
	
	GtkWidget *stable=gtk_table_new(2,2,0);
	gtk_box_pack_start(GTK_BOX(vbox),stable,FALSE,FALSE,0);
	GtkWidget *lab1=gtk_label_new("Szukaj");
	gtk_table_attach_defaults((gpointer)stable,lab1,0,1,0,1);
	search_entry=gtk_entry_new();
	gtk_table_attach_defaults((gpointer)stable,search_entry,1,2,0,1);
	gtk_widget_set_size_request((gpointer)search_entry, 320,-1);
	
	replace_label=gtk_label_new("Zmień na");
	gtk_table_attach_defaults((gpointer)stable,replace_label,0,1,1,2);
	replace_entry=gtk_entry_new();
	gtk_table_attach_defaults((gpointer)stable,replace_entry,1,2,1,2);
	
	
	g_signal_connect(G_OBJECT(search_entry),"key-press-event",G_CALLBACK(search_key_press),NULL);
	connect_label(lab1,search_entry);
	connect_label(replace_label,replace_entry);
	
	GtkWidget *sbox=gtk_hbox_new(0,2);
	gtk_box_pack_start(GTK_BOX(vbox),sbox,FALSE,FALSE,0);
	
	search_regex=gtk_check_button_new_with_mnemonic("_RegExp");
	gtk_box_pack_start(GTK_BOX(sbox),search_regex,FALSE,FALSE,0);
	
	search_case=gtk_check_button_new_with_mnemonic("_Wielkość liter");
	gtk_box_pack_start(GTK_BOX(sbox),search_case,FALSE,FALSE,0);

	search_word=gtk_check_button_new_with_mnemonic("_Słowo");
	gtk_box_pack_start(GTK_BOX(sbox),search_word,FALSE,FALSE,0);

	search_bword=gtk_check_button_new_with_mnemonic("_Początek słowa");
	gtk_box_pack_start(GTK_BOX(sbox),search_bword,FALSE,FALSE,0);

	search_eword=gtk_check_button_new_with_mnemonic("_Koniec słowa");
	gtk_box_pack_start(GTK_BOX(sbox),search_eword,FALSE,FALSE,0);
	
	GtkWidget *xbox=gtk_hbox_new(0,2);
	gtk_box_pack_start(GTK_BOX(vbox),xbox,FALSE,FALSE,0);
	
	search_backward=gtk_check_button_new_with_mnemonic("Ws_tecz");
	gtk_box_pack_start(GTK_BOX(xbox),search_backward,FALSE,FALSE,0);
	search_extw=gtk_check_button_new_with_mnemonic("E_xt. słowo");
	gtk_box_pack_start(GTK_BOX(xbox),search_extw,FALSE,FALSE,0);

	g_signal_connect(G_OBJECT(search_regex),"toggled",G_CALLBACK(set_searcher_active),NULL);
	g_signal_connect(G_OBJECT(search_word),"toggled",G_CALLBACK(set_searcher_active),NULL);
	g_signal_connect(G_OBJECT(search_bword),"toggled",G_CALLBACK(set_searcher_active),NULL);
	g_signal_connect(G_OBJECT(search_eword),"toggled",G_CALLBACK(set_searcher_active),NULL);
	g_signal_connect(G_OBJECT(search_backward),"toggled",G_CALLBACK(set_searcher_active),NULL);
	
	replace_box=gtk_frame_new("Zamień wszystkie");
	GtkWidget *replace_ibox=gtk_hbox_new(0,2);
	gtk_box_pack_end(GTK_BOX(xbox),replace_box,FALSE,FALSE,0);
	gtk_container_add(GTK_CONTAINER(replace_box),replace_ibox);
	
	//gtk_box_pack_start(GTK_BOX(replace_ibox),(gpointer)gtk_label_new("Zmień wszystkie"),FALSE,FALSE,0);
	GtkWidget *but=gtk_button_new_with_mnemonic("W _całym tekście");
	gtk_box_pack_start(GTK_BOX(replace_ibox),but,FALSE,FALSE,0);
	g_signal_connect(G_OBJECT(but),"clicked",G_CALLBACK(set_searcher_return),(void *)100);
	search_replace_selection=gtk_button_new_with_mnemonic("W _zaznaczeniu");
	gtk_box_pack_start(GTK_BOX(replace_ibox),search_replace_selection,FALSE,FALSE,0);
	g_signal_connect(G_OBJECT(search_replace_selection),"clicked",G_CALLBACK(set_searcher_return),(void *)101);
	
	gtk_widget_show_all(box);
}


int search_ask(int do_replace)
{
	if (get_current_editor()<0) return;
	create_search_dialog();
	gtk_entry_set_text((gpointer)search_entry,"");
	gtk_entry_set_text((gpointer)replace_entry,"");
	gtk_toggle_button_set_active((gpointer)search_regex,FALSE);
	gtk_toggle_button_set_active((gpointer)search_word,FALSE);
	gtk_toggle_button_set_active((gpointer)search_bword,FALSE);
	gtk_toggle_button_set_active((gpointer)search_eword,FALSE);
	gtk_toggle_button_set_active((gpointer)search_case,FALSE);
	gtk_toggle_button_set_active((gpointer)search_regex,FALSE);
	gtk_toggle_button_set_active((gpointer)search_extw,FALSE);
	editor_do_replace=do_replace;
	gtk_widget_grab_focus(search_entry);
	gtk_widget_set_sensitive(search_replace_selection,
		gtk_text_buffer_get_has_selection(tresc));
	if (do_replace) {
		gtk_widget_show(replace_entry);
		gtk_widget_show(replace_label);
		gtk_widget_show_all(replace_box);
	}
	else {
		gtk_widget_hide(replace_entry);
		gtk_widget_hide(replace_label);
		gtk_widget_hide(replace_box);
	}
	editor_history_pointer=&editor_history;
	gtk_window_resize(GTK_WINDOW(search_dialog),1,1);
	int n=gtk_dialog_run(GTK_DIALOG(search_dialog));
	gtk_widget_hide(search_dialog);
	if (n!=GTK_RESPONSE_ACCEPT && n!=100 && n !=101) return 0;
	insert_editor_history(do_replace);
	search_dialog_return_code=n;
	return gtk_entry_get_text_length((gpointer)search_entry);
}

static char *preproc_regex(char *c)
{
	char *d,*e;
	static char buf[1024];
	int in_class;
	int m;
	for (d=c,e=buf,in_class=0;*d;d++) {
		if (*d=='[') {
			*e++=*d;
			if (in_class) {
				in_class |= 4;
				continue;
			}
			in_class=1;
			m=0;
			if (d[1]=='^') {
				in_class=2;
				d++;
				*e++='^';
			}
			continue;
		}
		if (*d==']' && in_class) {
			if (in_class & 4) {
				*e++=*d;
				in_class &= 3;
				continue;
			}
			if (in_class==2) {
				if (!(m & 1)) {
					*e++='\\';*e++='r';
				}
				if (!(m & 2)) {
					*e++='\\';*e++='n';
				}
			}
			*e++=']';
			in_class=0;
			continue;
		}
		if (*d!='\\') {
			*e++=*d;
			continue;
		}
		if (d[1] !='s' || in_class) {
			*e++='\\';d++;
			if (in_class==2) {
				if (*d=='r') m|=1;
				if (*d=='n') m|=2;
			}
			*e++=*d;
			continue;
		}
		d++;
		strcpy(e,"[^\\S\\n\\r]");
		e+=9;
	}
	*e=0;
//	printf("RE <%s>\n",buf);
	return buf;
}


pcre *compile_preg_pattern(const char *str)
{
	int flags=PCRE_MULTILINE | PCRE_UTF8 | PCRE_NEWLINE_ANY;
	const char *err;int eoffset;
	pcre *pcr;
	if (!gtk_toggle_button_get_active((gpointer)search_case)) {
		flags |= PCRE_CASELESS;
	}
	str=preproc_regex((char *)str);
	pcr=pcre_compile(str,flags,&err,&eoffset,NULL);
	if (!pcr) {
		Error("Błąd kompilacji",(char *)err);
		return NULL;
	}
	words_extended=gtk_toggle_button_get_active((gpointer)search_extw);
	return pcr;
}

static int unichar_dec(char *str,char *buf)
{
	int z,i;
	for (z=i=0;i<4;i++) {
		z<<=4;
		if (!str[i] || !isxdigit(str[i])) return 0;
		else if (isdigit(str[i])) z+=str[i]-'0';
		else z+=tolower(str[i])-'a'+10;
	}
	i=0;
	if (z<128) {
		if (buf) buf[i++]=z;
		else return 1;
	}
	else if (z<0x800) {
		if (!buf) return 2;
		buf[i++]= 0xc0 | (z>>6);
		buf[i++]= 0x80 | (z & 0x3f);
	}
	else {
		if (!buf) return 3;
		buf[i++]= 0xe0 | (z >> 12);
		buf[i++]= 0x80 | ((z>>6) & 0x3f);
		buf[i++]= 0x80 | (z & 0x3f);
	}
	buf[i]=0;
	return i;
}

static char *create_replacement(char *repstr,int *xmatch,char *str)
{
	int n=0,t;char buf[4];
	char *r=repstr,*c,*d;
	while (*r) {
		if (*r != '\\') {
			n++;
			r++;
			continue;
		}
		r++;
		if (!*r) {
			n++;
			break;
		}
		if (xmatch && *r>='0' && *r<='9') {
			int v=(*r) - '0';
			n+=xmatch[2*v+1]-xmatch[2*v];
		}
		else if (*r=='u' && (t=unichar_dec(r+1,NULL))) {
			r+=4;
			n+=t;
		}
		else {
			n++;
		}
		r++;
	}
	c=d=malloc(n+1);
	r=repstr;
	while (*r) {
		if (*r != '\\') {
			*d++=*r++;
			continue;
		}
		r++;
		if (!*r) {
			*d++='\\';
			break;
		}
		if (xmatch && *r>='0' && *r<='9') {
			int v=(*r) - '0';
			int i;
			for (i=xmatch[2*v];i<xmatch[2*v+1];i++) *d++=str[i];
		}
		else if (*r == 'u' && unichar_dec(r+1,buf)) {
			char *s=buf;
			r+=4;
			while (*s) *d++=*s++;
		}
		else if (*r=='n') {
			*d++='\n';
		}
		else if (*r=='t') {
			*d++='\t';
		}
		else {
			*d++=*r;
		}
		r++;
	}
	*d=0;
	return c;
}

static int is_start_word(char *str,int pos)
{
	gunichar ut;
	char *c=g_utf8_find_prev_char(str,str+pos);
	if (!c) return TRUE;
	ut=g_utf8_get_char(c);
	
	if (g_unichar_isalnum(ut)) return 0;
	if (!words_extended) return TRUE;
	if (ut != '\'' && ut != '-') return TRUE;
	c=g_utf8_find_prev_char(str,c);
	if (!c) return TRUE;
	ut=g_utf8_get_char(c);
	return !g_unichar_isalnum(ut);
	
}

static int is_end_word(char *str,int pos)
{
	gunichar ut;
	if (!str[pos]) return TRUE;
	ut=g_utf8_get_char(str+pos);
	if (g_unichar_isalnum(ut)) return 0;
	if (!words_extended) return TRUE;
	if (ut != '\'' && ut != '-') return TRUE;
	pos++;
	if (!str[pos]) return TRUE;
	ut=g_utf8_get_char(str+pos);
	return !g_unichar_isalnum(ut);
}

static void not_found()
{
	Info("Informacja","Nic nie znaleziono");
}

static void found_occurences(int n)
{
	char buf[256];
	sprintf(buf,"Znaleziono %d wystąpie",n);
	if (n==1) strcat(buf,"nie");
	else {
		n%=100;
		if (n>10 && n<20) strcat(buf,"ń");
		else {
			n%=10;
			if (n>=2 && n<=4) strcat(buf,"nia");
			else strcat(buf,"ń");
		}
	}
	Info("Informacja",buf);
}

static int find_backward_by_regex(GtkTextIter *start,GtkTextIter *end,int as_word,char **repstr)
{
	
	int len,found;
	int ovector[30],fvector[30];
	pcre *pat;
	GtkTextIter plast;
	int naddch,reallen;
	pat=compile_preg_pattern((gpointer)gtk_entry_get_text((gpointer)search_entry));
	if (!pat) {
		return 0;
	}
	GtkTextMark *sel=gtk_text_buffer_get_insert(tresc);
	gtk_text_buffer_get_iter_at_mark(tresc,end,sel);
	
	naddch=0;
	plast=*end;
	if (gtk_text_iter_forward_char(&plast)) {
		naddch=1;
		if (gtk_text_iter_forward_char(&plast)) naddch=2;
	}
	
	gtk_text_buffer_get_start_iter(tresc,start);
	char *str=gtk_text_iter_get_text(start,&plast);
	len=strlen(str);
	reallen=len;
	while (naddch>0) {
		naddch--;
		char *c=g_utf8_find_prev_char(str,str+len);
		if (c) len=c-str;
	}
	int pos=len/2;
	int lastpos=0;
	int endpos=len;
	found=-1;
	int this_found,n,i,epos;
	for (;;) {
		if (lastpos >= endpos) break;
		while (pos >= lastpos && (str[pos] & 0xc0)==0x80) pos--;
		if (pos <lastpos) break;
		this_found=0;
		epos=pos;
		for (;;) {
			
			n=pcre_exec(pat,NULL,str,reallen,epos,PCRE_NOTEMPTY,ovector,30);
			if (n<0) break;
			if (ovector[0]>=len) break;
			if ((as_word & 1) && !is_start_word(str,ovector[0])) {
				epos=ovector[0];
				if (str[epos++] & 0x80) {
					while((str[epos] & 0xc0)==0x80) epos++;
				}
				continue;
			}
			if ((as_word & 2) && !is_end_word(str,ovector[1])) {
				epos=ovector[0];
				if (str[epos++] & 0x80) {
					while((str[epos] & 0xc0)==0x80) epos++;
				}
				continue;
			}
			this_found=1;
			if (ovector[0]>found) {
				found=ovector[0];
				for (i=0;i<30;i++) fvector[i]=ovector[i];
			}
			break;
		}
		if (!this_found) {
			endpos=pos;
			pos-=(pos-lastpos+1)/2;
			continue;
		}
		lastpos=pos+1;
		pos+=(endpos-pos+1)/2;
	}
	if (found >= 0) {
		*end=*start;
		n=g_utf8_strlen(str,fvector[0]);
		gtk_text_iter_forward_chars(start,n);
		n=g_utf8_strlen(str,fvector[1]);
		gtk_text_iter_forward_chars(end,n);
		
		if (repstr) {
			*repstr=create_replacement(
				(char *)gtk_entry_get_text((gpointer)replace_entry),
				fvector,str);
		}
		gtk_text_buffer_select_range(tresc,start,end);
		gtk_text_view_scroll_to_iter(tresc_v,start,0.2,FALSE,0.0,0.0);
	}
	g_free(str);
	pcre_free(pat);
	return found >= 0;
}

static int find_by_regex(GtkTextIter *start,GtkTextIter *end,int as_word,char **repstr)
{
	
	int len,found;
	int ovector[30];
	pcre *pat;
	pat=compile_preg_pattern((gpointer)gtk_entry_get_text((gpointer)search_entry));
	if (!pat) {
		return 0;
	}
	GtkTextMark *sel=gtk_text_buffer_get_selection_bound(tresc);
	gtk_text_buffer_get_iter_at_mark(tresc,start,sel);
	int pos=gtk_text_iter_get_line_index(start);
	while (!gtk_text_iter_starts_line(start)) {
		gtk_text_iter_backward_cursor_position(start);
	}
	gtk_text_buffer_get_end_iter(tresc,end);
	char *str=gtk_text_iter_get_text(start,end);
	found=0;len=strlen(str);
	for (;;) {
		int n=pcre_exec(pat,NULL,str,len,pos,PCRE_NOTEMPTY,ovector,30);
		if (n<0) break;
		if ((as_word & 1) && !is_start_word(str,ovector[0])) {
			pos=ovector[0];
			if (str[pos++] & 0x80) {
				while((str[pos] & 0xc0)==0x80) pos++;
			}
			continue;
		}
		if ((as_word & 2) && !is_end_word(str,ovector[1])) {
			pos=ovector[0];
			if (str[pos++] & 0x80) {
				while((str[pos] & 0xc0)==0x80) pos++;
			}
			continue;
		}
		found=1;
		*end=*start;
		n=g_utf8_strlen(str,ovector[0]);
		gtk_text_iter_forward_chars(start,n);
		n=g_utf8_strlen(str,ovector[1]);
		gtk_text_iter_forward_chars(end,n);
		
		if (repstr) {
			*repstr=create_replacement(
				(char *)gtk_entry_get_text((gpointer)replace_entry),
				ovector,str);
		}
		break;
	}	
	g_free(str);
	pcre_free(pat);
	if (found) {
		gtk_text_buffer_select_range(tresc,start,end);
		gtk_text_view_scroll_to_iter(tresc_v,start,0.2,FALSE,0.0,0.0);
	}

	return found;
}


static char *u_strstr(char *str,gunichar *needle,char **estr)
{
	char *slo,*sup,*s;gunichar *st;
	for (;;) {
		slo=g_utf8_strchr(str,-1,*needle);
		sup=g_utf8_strchr(str,-1,g_unichar_toupper(*needle));
		if (!slo) {
			if (!sup) {
				return NULL;
			}
			s=sup;
			
		}
		else if (!sup) {
			if (!slo) {
				return NULL;
			}
			s=slo;
		}
		else {
			s=slo;
			if (s>sup) s=sup;
		}
		str=s;
		s=g_utf8_next_char(s);
		st=needle+1;
		while (*st) {
			if (!*s) return NULL;
			if (*st != g_unichar_tolower(g_utf8_get_char(s))) break;
			s=g_utf8_next_char(s);
			st++;
		}
		if (!*st) {
			*estr=s;
			return str;
		}
		str=g_utf8_next_char(str);
	}
}

static void replace_all(char *search,
		char *replace,
		int insen,
		int as_word,
		int is_regex)

{
	pcre *pat;
	pcre_extra *pe;
	char const *pe_error;
	int ovector[30];
	GtkTextMark *m1,*m2;
	GtkTextIter start,finish,ps,end;
	int in_selection=0;
	char *str;
	char *ostr,*rc;
	int ostr_len,ostr_size,ostr_adder;
	int rtc;
	int found;
	int pos,slen,last;
	gunichar *u_search=NULL;

	void ostr_add(char *str,int len)
	{
		if (ostr_len+len>=ostr_size-2) {
			int adder=ostr_adder;
			if (len > adder/2) adder+=len;
			ostr_adder*=2;
			ostr_size+=adder;
			ostr=realloc(ostr,ostr_size);
		}
		memcpy(ostr+ostr_len,str,len);
		ostr_len+=len;
	}
	if (is_regex) {
		if (!(pat=compile_preg_pattern(search))) {
			return;
		}
		pe=pcre_study(pat,0,&pe_error);
	}
	else if (insen) {
		gunichar *gc;
		u_search=g_utf8_to_ucs4(search,-1,NULL,NULL,NULL);
		if (!u_search) return;
		for (gc=u_search;*gc;gc++) *gc=g_unichar_tolower(*gc);
		
	}
	
	if (search_dialog_return_code == 101) {
		in_selection = 1;
	}
	if (!in_selection) {
		gtk_text_buffer_get_start_iter(tresc,&start);
		gtk_text_buffer_get_end_iter(tresc,&finish);
	}
	else {
		m1=gtk_text_buffer_get_insert(tresc);
		m2=gtk_text_buffer_get_selection_bound(tresc);
		gtk_text_buffer_get_iter_at_mark(tresc,&start,m1);
		gtk_text_buffer_get_iter_at_mark(tresc,&finish,m2);
	}
	str=gtk_text_buffer_get_text(tresc,&start,&finish,FALSE);
	slen=strlen(str);
	ostr_size=slen+1;
	ostr=malloc(ostr_size);
	ostr_len=0;
	rtc=0;
	pos=0;
	last=0;
	ostr_adder=1024;
	start_busy_cursor();
	yield();
	if (!is_regex) replace=create_replacement(replace,NULL,NULL);
	for (;;) {
		char *fstr,*estr;
		found=0;
		for (;;) {
			if (is_regex) {
				if (pcre_exec(pat,pe,str,slen,pos,PCRE_NOTEMPTY,ovector,30)<0) break;
			}
			else {
				if (!insen) {
					fstr=strstr(str+pos,search);
					estr=fstr+strlen(search);
				}
				else {
					fstr=u_strstr(str+pos,u_search,&estr);
				}
				if (!fstr) break;
				ovector[0]=(fstr-str);
				ovector[1]=(estr-str);
			}
			if ((as_word & 1) && !is_start_word(str,ovector[0])) {
				pos=ovector[0];
				if (str[pos++] & 0x80) {
					while((str[pos] & 0xc0)==0x80) pos++;
				}
				continue;
			}
			if ((as_word & 2) && !is_end_word(str,ovector[1])) {
				pos=ovector[0];
				if (str[pos++] & 0x80) {
					while((str[pos] & 0xc0)==0x80) pos++;
				}
				continue;
			}
			found=1;
			break;
		}
		if (!found) break;
		if (is_regex) {
			rc=create_replacement(
				replace,
				ovector,str);
		}
		else {
			rc=replace;
		}
		ostr_add(str+last,ovector[0]-last);
		
		ostr_add(rc,strlen(rc));
		if (is_regex) {
			free(rc);
		}
		pos=last=ovector[1];
		rtc++;
	}
	if (is_regex) {
		pcre_free(pat);
		if (pe) pcre_free(pe);
	}
	else (free(replace));
	if (rtc) {
		if (last < slen) {
			ostr_add(str+last,slen-last);
		}
		gtk_text_buffer_begin_user_action(tresc);
		if (!in_selection) {
			gtk_text_buffer_set_text(tresc,ostr,ostr_len);
			gtk_text_buffer_get_end_iter(tresc,&ps);
			gtk_text_buffer_place_cursor(tresc,&ps);
			gtk_text_view_scroll_to_iter(tresc_v,&ps,0.2,FALSE,0.0,0.0);
		}
		else {
			gtk_text_buffer_delete(tresc,&start,&finish);
			gtk_text_buffer_insert(tresc,&start,ostr,ostr_len);
			gtk_text_buffer_place_cursor(tresc,&start);
			gtk_text_view_scroll_to_iter(tresc_v,&start,0.2,FALSE,0.0,0.0);
			ps=start;
		}
		gtk_text_buffer_end_user_action(tresc);
	}
	
	g_free(str);
	free(ostr);
	if (u_search) g_free(u_search);
	end_busy_cursor();
	if (a11y_not_found) {
		if (rtc) found_occurences(rtc);
		else not_found();
	}
}

void StartReplace(GtkWidget *widget,void *data)
{
	int bkwd,insen,inwor;
	char *replace;
	GtkTextIter pos,end;
	if (!search_ask(1)) return;
	get_current_editor();
	bkwd=gtk_toggle_button_get_active((gpointer)search_backward);
	insen=!gtk_toggle_button_get_active((gpointer)search_case);
	inwor=0;
	if (gtk_toggle_button_get_active((gpointer)search_word)) inwor|=3;
	if (gtk_toggle_button_get_active((gpointer)search_bword)) inwor|=1;
	if (gtk_toggle_button_get_active((gpointer)search_eword)) inwor|=2;
	replace=(gpointer) gtk_entry_get_text((gpointer)replace_entry);
	if (search_dialog_return_code == GTK_RESPONSE_ACCEPT) {
		if (gtk_toggle_button_get_active((gpointer)search_regex)) {
			if (bkwd) {
				if (!find_backward_by_regex(&pos,&end,inwor,&replace)) {
					if (a11y_not_found) not_found();
					return;
				}
			}
			else {
				if (!find_by_regex(&pos,&end,inwor,&replace)) {
					if (a11y_not_found) not_found();
					return;
				}			}
		}
		else {
			if (!search_in_buffer(NULL,(gpointer)gtk_entry_get_text((gpointer)search_entry),
				insen,
				bkwd,
				inwor)) {
					if (a11y_not_found) not_found();
					return;
			}
			replace=create_replacement(replace,NULL,NULL);
		}
		gtk_text_buffer_begin_user_action (tresc);
		gtk_text_buffer_delete_selection (tresc, FALSE, TRUE);
		gtk_text_buffer_insert_at_cursor (tresc, replace, strlen (replace));
		gtk_text_buffer_end_user_action (tresc);
		free(replace);
		return;
	}
	replace_all((gpointer)gtk_entry_get_text((gpointer)search_entry),
		replace,
		insen,
		inwor,
		gtk_toggle_button_get_active((gpointer)search_regex));
}
void StartSearch(GtkWidget *widget,void *data)
{
	int _data=(int)data;
	int bkwd=0,insen,inwor;
	GtkTextIter pos,end;
	if (_data == 2) bkwd=1;
	get_current_editor();
	if (!data || !search_dialog || !gtk_entry_get_text_length((gpointer)search_entry)) {
		if (!search_ask(0)) return;
		bkwd=gtk_toggle_button_get_active((gpointer)search_backward);
	}
	insen=!gtk_toggle_button_get_active((gpointer)search_case);
	inwor=0;
	if (gtk_toggle_button_get_active((gpointer)search_word)) inwor|=3;
	if (gtk_toggle_button_get_active((gpointer)search_bword)) inwor|=1;
	if (gtk_toggle_button_get_active((gpointer)search_eword)) inwor|=2;
	
	if (!gtk_toggle_button_get_active((gpointer)search_regex)) {
		if (!search_in_buffer(NULL,(gpointer)gtk_entry_get_text((gpointer)search_entry),
			insen,
			bkwd,
			inwor) && a11y_not_found) not_found();
		return;
	}
	int n;
	if (bkwd) n=find_backward_by_regex(&pos,&end,inwor,NULL);
	else n=find_by_regex(&pos,&end,inwor,NULL);
	if (!n && a11y_not_found) not_found();
}

void find_entity(GtkWidget *widget,void *data)
{
	int t=get_current_editor();
	GtkTextIter start,end,pos;
	if (t <0) return;
	create_search_dialog();
	gtk_entry_set_text((gpointer)search_entry,"&((#(([0-9]{1,6})|(x[0-9a-f]{1,4})))|([a-z]{2,8}));");
	gtk_entry_set_text((gpointer)replace_entry,"");
	gtk_toggle_button_set_active((gpointer)search_regex,1);
	gtk_toggle_button_set_active((gpointer)search_word,0);
	gtk_toggle_button_set_active((gpointer)search_bword,0);
	gtk_toggle_button_set_active((gpointer)search_eword,0);
	gtk_toggle_button_set_active((gpointer)search_extw,0);
	gtk_toggle_button_set_active((gpointer)search_case,0);
	gtk_toggle_button_set_active((gpointer)search_backward,FALSE);
	int inwor=0;
	if (!find_by_regex(&pos,&end,inwor,NULL) && a11y_not_found) not_found();
	//insert_editor_history(0);
	return;
}

void find_current_word(GtkWidget *widget,void *data)
{
	int t=get_current_editor();
	GtkTextIter start,end,pos;
	if (t != MILEDIT_MODE_DIC && t != MILEDIT_MODE_DASH) return;
	GtkTextMark *sel=gtk_text_buffer_get_insert(tresc);
	gtk_text_buffer_get_iter_at_mark(tresc,&start,sel);
	while (!gtk_text_iter_starts_line(&start)) {
		gtk_text_iter_backward_cursor_position(&start);
	}
	end=start;
	gtk_text_iter_forward_to_line_end(&end);
	char *str=gtk_text_buffer_get_text(tresc,&start,&end,0);
	char *cs,*ds,*ecs=NULL,*eds;
	int n;
	gunichar ut,uu;int has_up=0,has_star=0;
	if (!str) return;
	cs=strstr(str,"//");
	if (cs) *cs=0;
	cs=str;
	while (*cs && isspace(*cs)) cs++;
	if (!*cs) {
		g_free(str);
		return;
	}
	for (ds=cs;*ds;) {
		ut=g_utf8_get_char(ds);
		if (g_unichar_isupper(ut)) has_up=1;
		if (!g_unichar_isalnum(ut) && ut!='-' && ut!='\'') break;
		ds=g_utf8_next_char(ds);
	}
	if (*ds=='*') has_star=1;
	*ds=0;
	if (!*cs) {
		g_free(str);
		return;
	}
	if (has_up) {
		ecs=malloc(8*strlen(cs));
		for (ds=cs,eds=ecs;*ds;ds=g_utf8_next_char(ds)) {
			ut=g_utf8_get_char(ds);
			if (g_unichar_islower(ut)) {
				*eds++='[';
				n=g_unichar_to_utf8(ut,eds);
				eds+=n;
				n=g_unichar_to_utf8(g_unichar_toupper(ut),eds);
				eds+=n;
				*eds++=']';
			}
			else {
				n=g_unichar_to_utf8(ut,eds);
				eds+=n;
			}
		}
		*eds=0;
		cs=ecs;
	}
	create_search_dialog();
	gtk_entry_set_text((gpointer)search_entry,cs);
	gtk_entry_set_text((gpointer)replace_entry,"");
	gtk_toggle_button_set_active((gpointer)search_regex,has_up);
	gtk_toggle_button_set_active((gpointer)search_word,has_star?0:1);
	gtk_toggle_button_set_active((gpointer)search_bword,has_star);
	gtk_toggle_button_set_active((gpointer)search_eword,0);
	gtk_toggle_button_set_active((gpointer)search_extw,1);
	gtk_toggle_button_set_active((gpointer)search_case,has_up);
	gtk_toggle_button_set_active((gpointer)search_backward,FALSE);
	show_editor(current_editor->owneditor);
	get_current_editor();
	int insen=!gtk_toggle_button_get_active((gpointer)search_case);
	int inwor=0;
	if (gtk_toggle_button_get_active((gpointer)search_word)) inwor|=3;
	if (gtk_toggle_button_get_active((gpointer)search_bword)) inwor|=1;
	if (gtk_toggle_button_get_active((gpointer)search_eword)) inwor|=2;
	
	gtk_text_buffer_get_start_iter((gpointer)tresc,&start);
	gtk_text_buffer_place_cursor((gpointer)tresc,&start);
	gtk_text_view_scroll_to_iter((gpointer)tresc_v,&start,FALSE,FALSE,0.0,0.0);
	if (!gtk_toggle_button_get_active((gpointer)search_regex)) {
		if (!search_in_buffer(NULL,(gpointer)gtk_entry_get_text((gpointer)search_entry),
			insen,
			0,
			inwor) && a11y_not_found) not_found();
	}
	else {
		if (!find_by_regex(&pos,&end,inwor,NULL) && a11y_not_found) not_found();
	}
	//insert_editor_history(0);
	g_free(str);
	if (ecs) free(ecs);
	return;
}


void set_buffer_text(gpointer buf,char *str,int len,int undoable)
{
	GtkTextIter pos;
	if (undoable) {
		gtk_text_buffer_begin_user_action(buf);
	}
	else {
		gtk_source_buffer_begin_not_undoable_action(buf);
	}
	gtk_text_buffer_set_text(buf,str,len);
	gtk_text_buffer_get_start_iter(buf,&pos);
	gtk_text_buffer_place_cursor(buf,&pos);
	if (undoable) {
		gtk_text_buffer_end_user_action(buf);
	}
	else {
		gtk_source_buffer_end_not_undoable_action(buf);
		gtk_text_buffer_set_modified((gpointer)tresc,FALSE);
	}
}

void go_to_line(GtkWidget *widget,void *dummy)
{
	if (get_current_editor()<0) return;
	int n=GetNumber("Skocz do linii","Linia");
	if (n<=0) return;
	GtkTextIter line;
	gtk_text_buffer_get_iter_at_line(tresc,&line,n-1);
	gtk_text_buffer_place_cursor(tresc,&line);
	gtk_text_view_scroll_to_iter((gpointer)tresc_v,&line,FALSE,TRUE,0.0,0.5);
}

void editor_delete_line(GtkWidget *widget,void *data)
{
	GtkTextMark *m;
	GtkTextIter start,finisz;
	int cnt;
	if (get_current_editor()<0) return;
	m=gtk_text_buffer_get_insert(tresc);
	gtk_text_buffer_get_iter_at_mark(tresc,&start,m);
	cnt=gtk_text_iter_get_line_offset(&start);
	if (cnt) gtk_text_iter_backward_chars(&start,cnt);
	finisz=start;
	gtk_text_iter_forward_line(&finisz);
	gtk_text_buffer_begin_user_action(tresc);
	gtk_text_buffer_delete(tresc,&start,&finisz);
	gtk_text_buffer_end_user_action(tresc);
	
}
void editor_ulcase(GtkWidget *widget,void *data)
{
	GtkTextMark *m1,*m2;
	GtkTextIter start,finish;
	char *str,*ustr;
	int sc;
	if (get_current_editor()<0) return;
	m1=gtk_text_buffer_get_insert(tresc);
	m2=gtk_text_buffer_get_selection_bound(tresc);
	gtk_text_buffer_get_iter_at_mark(tresc,&start,m1);
	gtk_text_buffer_get_iter_at_mark(tresc,&finish,m2);
	str=gtk_text_buffer_get_text(tresc,&start,&finish,FALSE);
	if (data) {
		ustr=g_utf8_strup(str,-1);
	}
	else {
		ustr=g_utf8_strdown(str,-1);
	}
	sc=strcmp(str,ustr);
	//printf("A1=%s, A2=%s, S=%d\n",str,ustr,sc);
	g_free(str);
	if (sc) {
		gtk_text_buffer_begin_user_action(tresc);
		gtk_text_buffer_delete(tresc,&start,&finish);
		gtk_text_buffer_insert(tresc,&start,ustr,strlen(ustr));
		gtk_text_buffer_place_cursor(tresc,&start);
		//gtk_text_view_scroll_to_iter(tresc_v,&start,0.2,FALSE,0.0,0.0);
		gtk_text_buffer_end_user_action(tresc);
	}
	g_free(ustr);	
}

static char recoder_uname[32];
static int recoder_ux[128];

static int recoder_read_encoding(char *enc)
{
	FILE *f;int znak,nr,w;char *c;
	char line[256];
	if(!strcmp(recoder_uname,enc)) return 1;
	sprintf(line,"zcat /usr/share/i18n/charmaps/%s.gz",enc);
	f=popen(line,"r");
	if (!f) {
		perror("Popen");
		return 0;
	}
	recoder_uname[0]=0;
	w=0;
	while(fgets(line,256,f)) {
		if (line[0]!='<') continue;
		if (line[1]!='U') continue;
		znak=strtol(line+2,&c,16);
		if (*c++!='>') continue;
		while (*c && *c!='/') c++;
		c++;
		if (*c++!='x') continue;
		nr=strtol(c,NULL,16)-128;
		if (!nr) w |=1;
		else if (nr==127) w |=2;
		if (nr>=0 && nr<128) recoder_ux[nr]=znak; 
	}
	pclose(f);
	if (w !=3) {
		Error("Błąd","Nie mogę wczytać tablicy kodowania");
		return 0;
	}
	strcpy(recoder_uname,enc);
	return 1;
}



void editor_fake_latin(GtkWidget *widget,void *data)
{
	GtkTextIter start,finish;
	char *str,*ustr,*c,*d;
	int sc,len;
	
	int gzn(int sc)
	{
		if (sc>=128 && sc <256) sc=recoder_ux[sc&0x7f];
		else if (sc == 0x153) sc=0x15b;
		else if (sc == 0x152) sc=0x15a;
		else if (sc == 0x178) sc=0x17a;
		/* a jak jest duże 'Ź'? */
		return sc;
	}
	
	if (get_current_editor()<0) return;
	if (!recoder_read_encoding((char *)data)) return;
	gtk_text_buffer_get_start_iter(tresc,&start);
	gtk_text_buffer_get_end_iter(tresc,&finish);
	
	
	str=gtk_text_buffer_get_text(tresc,&start,&finish,FALSE);
	len=1;
	for (c=str;*c;c=g_utf8_next_char(c)) {
		sc=g_utf8_get_char(c);
		sc=gzn(sc);
		if (sc < 128) len++;
		else if (sc <=0x7ff) len+=2;
		else if (sc <=0xffff) len+=3;
		else continue;
	}
	ustr=g_malloc(len);
	for (c=str,d=ustr;*c;c=g_utf8_next_char(c)) {
		sc=g_utf8_get_char(c);
		sc=gzn(sc);
		if (sc < 128) {
			*d++=sc;
		}
		else if (sc <= 0x7ff) {
			*d++=(sc>>6) | 0xc0;
			*d++=(sc & 0x3f) | 0x80;
		}
		else if (sc <= 0xffff) {
			*d++=(sc>>12) | 0xe0;
			*d++=((sc >> 6) & 0x3f) | 0x80;
			*d++=(sc & 0x3f) | 0x80;
		}
		else continue;
	}
	*d=0;
	g_free(str);
	gtk_text_buffer_begin_user_action(tresc);
	gtk_text_buffer_set_text(tresc,ustr,-1);
	gtk_text_buffer_get_start_iter(tresc,&start);
	gtk_text_buffer_place_cursor(tresc,&start);
	gtk_text_buffer_end_user_action(tresc);
	g_free(ustr);	
}

void find_dialog_connection(GtkWidget *widget,void *dummy)
{

	GtkTextIter pos,end;int n,size;
	if (get_current_editor()) return;
	int tres=get_current_timeres();
	if (!tres) {
		Error("Błąd","Plik nie zaczyna się od #TIMERES");
		return;
	}
	GtkTextMark *sel=gtk_text_buffer_get_selection_bound(tresc);
	gtk_text_buffer_get_iter_at_mark(tresc,&pos,sel);
	int cnt=gtk_text_iter_get_line_offset(&pos);
	if (cnt) gtk_text_iter_backward_chars(&pos,cnt);
	gtk_text_buffer_get_end_iter(tresc,&end);
	char *txt=gtk_text_buffer_get_slice(tresc,&pos,&end,FALSE);
	n=jacosub_find_dialcon(txt,&size,tres);
	if (txt) g_free(txt);
	if (n<0) {
		if (a11y_not_found) {
			not_found();
			return;
		}
	}
	gtk_text_iter_forward_chars(&pos,n);
	end=pos;
	gtk_text_iter_forward_chars(&end,size);
	gtk_text_buffer_select_range(tresc,&pos,&end);
	gtk_text_view_scroll_to_iter(tresc_v,&pos,0.2,FALSE,0.0,0.0);
	return;
}

void find_scanner_error(GtkWidget *widget, void *dummy)
{
	GtkTextIter start,end;
	char *str;
	int len;
	
	if (get_current_editor()) return;

	GtkTextMark *sel=gtk_text_buffer_get_selection_bound(tresc);
	gtk_text_buffer_get_iter_at_mark(tresc,&start,sel);
	gtk_text_buffer_get_end_iter(tresc,&end);
	str=gtk_text_iter_get_text(&start,&end);
	int pos=find_bad_construction(str,&len);
	if (pos>=0) {
		int ofs=g_utf8_strlen(str,pos);
		gtk_text_iter_forward_chars(&start,ofs);
		end=start;
		gtk_text_iter_forward_chars(&end,len);
		gtk_text_buffer_select_range(tresc,&start,&end);
		gtk_text_view_scroll_to_iter(tresc_v,&start,0.2,FALSE,0.0,0.0);
	}
	else {
		not_found();
	}
	g_free(str);
}
