#include "gmilena.h"
#include <stdarg.h>
#include <string.h>
#include <math.h>
#include <enca.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/stat.h>
#include <ctype.h>

extern int ignore_oor;

static int my_compare(char *s1,char *s2)
{
	while (*s1) {
		while (*s1 && !isalnum(*s1)) s1++;
		if (!*s1) break;
		if (*s2++ != tolower(*s1++)) return 0;
	}
	if (*s2) return 0;
	return 1;
}



extern char *read_rtf(char *str,int len);
extern char *to_utf8(char *fbuf,int flen,char *cset,int freeme);
#define TESTBUF_SIZE 100

static const char *get_encoding(char *buf,size_t len)
{
	EncaAnalyser an;
	const double mu = 0.005;
	const double m = 15.0;
	size_t sgnf;
	const char *c;
	EncaEncoding result;
	for (c=buf;*c;c++) if ((*c) & 0x80) break;
	if (!*c) return "UTF-8";
	an=enca_analyser_alloc("pl");
	if (!an) {
		return NULL;
	}
	enca_set_threshold(an, 1.38);
	enca_set_multibyte(an, 1);
	enca_set_ambiguity(an, 1);
	enca_set_garbage_test(an, 1);
	sgnf = ceil((double)len/(len/m + 1.0/mu));
	enca_set_significant(an, sgnf);
	enca_set_termination_strictness(an, 1);
	enca_set_filtering(an, sgnf > 2);

	result = enca_analyse_const(an, (unsigned char *)buf, len);	
	if (!enca_charset_is_known(result.charset)) {
		enca_analyser_free(an);
		return NULL;
	}
	c=enca_charset_name(result.charset,ENCA_NAME_STYLE_ICONV);
	enca_analyser_free(an);
	return c;
}

static int check_program(char *program,char *pakiet)
{
	char *c=getenv("PATH");
	if (!c) {
		Error("Błąd","Brak ustawionej ścieżki PATH");
		return 0;
	}
	char **path=g_strsplit(c,":",0);
	int i,fnd;
	for (i=fnd=0;path[i];i++) {
		char buf[PATH_MAX];
		sprintf(buf,"%s/%s",path[i],program);
		if (!g_access(buf,X_OK|R_OK)) {
			fnd=1;
			break;
		}
		
	}
	g_strfreev(path);
	if (!fnd) {
		char buf[512];
		sprintf(buf,"Brak programu %s",program);
		if (pakiet) sprintf(buf+strlen(buf),".\nZainstaluj pakiet %s",pakiet);
		Error("Błąd",buf);
	}
	return fnd;
	
}

void autocompose_psp(char *buf)
{
	char *obuf,*s,*bs;
	obuf=bs=buf;
	int gtch()
	{
		int z,n;
		if (!*buf) return 0;
		z=(*buf++) & 255;
		if (!(z & 0x80)) return z;
		if ((z & 0xe0) == 0xc0) {n=1;z&=0x1f;}
		else if ((z & 0xf0) == 0xe0) {n=2;z&=0x0f;}
		else if ((z & 0xf8) == 0xf0) {n=3;z&=0x07;}
		else if ((z & 0xfc) == 0xf8) {n=4;z&=0x03;}
		else if ((z & 0xfe) == 0xfc) {n=5;z&=0x01;}
		else return 0;
		
		while(n && *buf) {
			z=(z<<6) | ((*buf++) & 0x3f);
			n--;
		}
		return z;
	}
	for (;;) {
		int z1,z2,z3;
		s=buf;
		z1=gtch();
		if (!z1) break;
		//printf("%d %d\n",z1,L'´');
		if (z1 != L'´' && z1 != L'˙' && z1 !=L'˛') {
loop:			while (s<buf) *obuf++=*s++;
			continue;
		}
		z2=gtch();
		if (!z2) break;
		if (z2 >= 0x80) goto loop;
		if (z1==L'´') {
			if (z2=='S') z3=L'Ś';
			else if (z2=='s') z3=L'ś';
			else if (z2=='C') z3=L'Ć';
			else if (z2=='c') z3=L'ć';
			else if (z2=='N') z3=L'Ń';
			else if (z2=='n') z3=L'ń';
			else if (z2=='Z') z3=L'Ź';
			else if (z2=='z') z3=L'ź';
			else goto loop;
		}
		else if (z1 == L'˙') {
			if (z2=='Z') z3=L'Ż';
			else if (z2=='z') z3=L'ż';
			else goto loop;
		}
		else if (z1 == L'˛') {
			if (z2=='A') z3=L'Ą';
			else if (z2=='a') z3=L'ą';
			else if (z2=='E') z3=L'Ę';
			else if (z2=='e') z3=L'ę';
			else goto loop;
			if (obuf != bs && obuf[-1] == ' ') obuf--;
		}
		else goto loop;
		*obuf++=(z3>>6) | 0xc0;
		*obuf++=(z3 & 0x3f) | 0x80;
	}
	*obuf=0;
}

static char *read_file(char *name,int *is_g,int new_pdf)
{
	int fd;
	struct stat sb;
	int flen,n,inb,fmode;
	char *fbuf,*isostr,*rstr,*exten;
	char tempo[64];
	static unsigned char *isword[3]={
		(unsigned char *)"\004\376\067\0\043",
		(unsigned char *)"\x8\320\317\021\340\241\261\032\341",
		(unsigned char *)"\x6\333\245-\0\0\0"};
	char testbuf[TESTBUF_SIZE];
	int i;
	char *encoding=NULL;

	if (is_g) *is_g=0;
	if (stat(name,&sb)) {
		g_perror(name);
		return NULL;
	}
	exten=strrchr(name,'.');
#ifdef HAVE_EPUB
	if (exten && !strcasecmp(exten,".epub")) {
		char *erc=NULL;
		fbuf=read_epub(name,&erc);
		if (!fbuf) {
			if (!erc) erc="Nieznany błąd";
			Error("EPub",erc);
			return NULL;
		}
		return fbuf;
	}
#endif
	if ((fd=open(name,O_RDONLY))<0) {
		g_perror(name);
		return NULL;
	}
	flen=sb.st_size;
	inb=0;fmode=0;
	if (flen > TESTBUF_SIZE) {
		if (read(fd,testbuf,TESTBUF_SIZE)!=TESTBUF_SIZE) {
			g_perror(name);
			return NULL;
		}
		inb=TESTBUF_SIZE;
		fmode=0;
		if (!strncmp(testbuf,"{\\rtf",5)) fmode=1;
		else if (!strncmp(testbuf,"%!PS-Adobe",10)) fmode=5;
		else if (!strncmp(testbuf,"%PDF-1",6)) fmode=4;
		else {
			for (i=0;i<3;i++) if (!memcmp(testbuf,isword[i]+1,isword[i][0])) {
				fmode=2;
				break;
			}

		}
		if (!fmode) {
			if (!memcmp(testbuf,"PK\003\004",4)) {
				for (i=0;i<TESTBUF_SIZE-27;i++) {
					if (!memcmp(testbuf+i,"vnd.oasis.opendocument.text",27)) {
						fmode=3;
						break;
					}
				}
			}
		}
	}
	if (fmode == 5) {
		char sbuf[1024],*c,*d;
		close(fd);
		if (!check_program(new_pdf?"pdftohtml":"pdftotext","poppler-utils lub xpdf-utils")) return NULL;
		if (!check_program("ps2pdf","ghostscript")) return NULL;
		strcpy(sbuf,"ps2pdf ");
		c=name;
		d=sbuf+strlen(sbuf);
		for (;*c;c++) {
		if (!isalnum(*c) && !strchr("./",*c)) *d++='\\';
			*d++=*c;
		}
		*d++=' ';
		strcpy(tempo,"/tmp/milenizer_XXXXXX");
		fd=mkstemp(tempo);
		if (fd<0) {
			g_perror("mkstemp");
			return NULL;
		}
		close(fd);
		strcpy(d,tempo);
		strcat(sbuf,".pdf");
		system(sbuf);
		if (stat(d,&sb)) {
			g_perror(d);
			remove(tempo);
			remove(d);
			return NULL;
		}
		if (new_pdf) strcpy(sbuf,"pdftohtml -i -noframes -xml -stdout -enc UTF-8 ");
		else strcpy(sbuf,"pdftotext -raw -enc UTF-8 ");
		strcat(sbuf,tempo);
		strcat(sbuf,".pdf ");
		if (new_pdf) strcat(sbuf,">");
		strcat(sbuf,tempo);
		system(sbuf);
		strcpy(sbuf,tempo);
		strcat(sbuf,".pdf");
		remove(sbuf);
		if (stat(tempo,&sb)) {
			g_perror(d);
			remove(tempo);
			return NULL;
		}
		flen=sb.st_size;
		if (!flen) {
			remove(tempo);
			Error("Błąd","Wynikowy plik jest pusty");
			return NULL;
		}
		fd=open(tempo,O_RDONLY);
		if (fd<0) {
			g_perror(tempo);
			remove(tempo);
			return NULL;
		}
		inb=0;
		remove(tempo);
		encoding="UTF-8";
	}
	else if (fmode==2 || fmode==3 || fmode==4) {
		char sbuf[1024],*c,*d;
		close(fd);
		if (fmode==2) {
			if (!check_program("antiword",NULL)) return NULL;
			strcpy(sbuf,"antiword -w 0 -m UTF-8.txt ");
		}
		else if (fmode == 3) {
			if (!check_program("odt2txt",NULL)) return NULL;
			strcpy(sbuf,"odt2txt --width=-1 --encoding=UTF-8 ");
		}
		else {
			if (!check_program(new_pdf?"pdftohtml":"pdftotext","poppler-utils lub xpdf-tools")) return NULL;
			if (new_pdf) strcpy(sbuf,"pdftohtml -i -noframes -xml -stdout -enc UTF-8 ");
			else strcpy(sbuf,"pdftotext -raw -enc UTF-8 ");
		}
		c=name;
		d=sbuf+strlen(sbuf);
		for (;*c;c++) {
		if (!isalnum(*c) && !strchr("./",*c)) *d++='\\';
			*d++=*c;
		}
		if (fmode !=4 || new_pdf) strcpy(d," > ");
		else strcpy(d," ");
		strcpy(tempo,"/tmp/milenizer_XXXXXX");
		fd=mkstemp(tempo);
		if (fd<0) {
			g_perror("mkstemp");
			return NULL;
		}
		close(fd);
		strcat(sbuf,tempo);
		system(sbuf);
		if (stat(tempo,&sb)) {
			g_perror(tempo);
			remove(tempo);
			return NULL;
		}
		flen=sb.st_size;
		if (!flen && fmode == 4) {
			remove(tempo);
			if (!Ask("Pytanie","Plik PDF wygląda na zabezpieczony.\nCzy mam próbować wczytać go mimo to?\nNie zawsze się to udaje...")) return NULL;
			yield();
			if (!check_program("pdftops","poppler-utils lub xpdf-tools")) return NULL;
			if (!check_program("ps2pdf","ghostscript")) return NULL;
			
			strcpy(sbuf,"pdftops ");
			c=name;
			d=sbuf+strlen(sbuf);
			for (;*c;c++) {
			if (!isalnum(*c) && !strchr("./",*c)) *d++='\\';
				*d++=*c;
			}
			*d++=' ';
			strcpy(d,tempo);
			strcat(d,".ps");
			//printf("\nWykonuję %s\n",sbuf);
			system(sbuf);
			strcpy(sbuf,"ps2pdf ");
			strcat(sbuf,tempo);
			strcat(sbuf,".ps ");
			strcat(sbuf,tempo);
			strcat(sbuf,".pdf");
			//printf("Wykonuję %s\n",sbuf);
			system(sbuf);
			strcpy(sbuf,tempo);
			strcat(sbuf,".ps");
			remove(sbuf);
			if (new_pdf) strcpy(sbuf,"pdftohtml -i -noframes -xml -stdout -enc UTF-8 ");
			else strcpy(sbuf,"pdftotext -raw -enc UTF-8 ");
			strcat(sbuf,tempo);
			strcat(sbuf,".pdf ");
			if (new_pdf) strcat(sbuf,">");
			strcat(sbuf,tempo);
			//printf("Wykonuję %s\n",sbuf);
			system(sbuf);
			strcpy(sbuf,tempo);
			strcat(sbuf,".pdf");
			remove(sbuf);
			if (stat(tempo,&sb)) {
				g_perror(tempo);
				remove(tempo);
				return NULL;
			}
			flen=sb.st_size;
		}
		fd=open(tempo,O_RDONLY);
		if (fd<0) {
			g_perror(tempo);
			remove(tempo);
			return NULL;
		}
		inb=0;
		remove(tempo);
		encoding="UTF-8";
		//fmode=0;
	}
	fbuf=malloc(flen+1);
	if (inb) memcpy(fbuf,testbuf,inb);
	if (read(fd,fbuf+inb,flen-inb)!=flen-inb) {
		g_perror(name);
		return NULL;
	}
	close(fd);
	fbuf[flen]=0;
	if (!fmode && exten && (!strcasecmp(exten,".html") || !strcasecmp(exten,".xhtml") || !strcasecmp(exten,".htm"))) {
		int n=html2txt(fbuf,NULL);
		if (n>0) {
			char *c=malloc(n);
			html2txt(fbuf,c);
			free(fbuf);
			return c;
		}
	}
	if (!fmode) {
		char *c=strstr(fbuf,"<!DOCTYPE pdf2xml");
		if (c && c < fbuf+100) {
			c=pdxml2txt(fbuf);
			free(fbuf);
			fbuf=c;
		}
	}
	if (fmode==1) {
		char *c=read_rtf(fbuf,flen);
		free(fbuf);
		return c;
	}
	if (!encoding) encoding=(char *)get_encoding(fbuf,flen);
	if (!encoding) {
		encoding=ask_encoding();
		if (!encoding) {
			free(fbuf);
			return NULL;
		}
	}
	if (!my_compare(encoding,"utf8")) {
		fbuf=to_utf8(fbuf,flen,encoding,1);
	}
	if (fmode == 4 || fmode == 5) {
		if (new_pdf) {
			char *cs=pdxml2txt(fbuf);
			free(fbuf);
			fbuf=cs;
		}
		autocompose_psp(fbuf);
	}
	return fbuf;
}



char last_file_path[PATH_MAX+1];

GtkFileFilter *make_filter(char *name,...)
{
	va_list ap;
	va_start(ap,name);
	char *c;
	GtkFileFilter *filter=gtk_file_filter_new();
	gtk_file_filter_set_name(filter,name);
	for (;;) {
		c=va_arg(ap,char *);
		if (!c) break;
		gtk_file_filter_add_pattern(filter,c);
	}
	va_end(ap);
	return filter;
}
int textbuf_is_empty(gpointer buf)
{
	int n=gtk_text_buffer_get_char_count(buf);
	return !n;
	
}

void new_file(GtkWidget *dummy,void *data)
{
	CreateEditor(0,NULL,"nowy");
}

static int get_save_path(char *path,int is_dic)
{
	int n;
	GtkWidget *dialog=gtk_file_chooser_dialog_new(
		is_dic?"Wybierz plik do zapisu słownika":"Wybierz plik do zapisu",
		GTK_WINDOW(main_window),GTK_FILE_CHOOSER_ACTION_SAVE,
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
		NULL);
	GtkFileFilter *filter;
	gtk_file_chooser_set_local_only((gpointer)dialog,TRUE);
	if (!is_dic) gtk_file_chooser_add_filter((gpointer) dialog,
		filter=make_filter("Pliki tekstowe","*.txt","*.TXT",NULL));
	if (is_dic) gtk_file_chooser_add_filter((gpointer) dialog,
		filter=make_filter("Pliki DIC","*.dic","*.DIC",NULL));
	gtk_file_chooser_add_filter((gpointer) dialog,
		make_filter("Wszystkie pliki","*",NULL));
	gtk_file_chooser_set_filter((gpointer) dialog,filter);
	
	if (*path) {
		char *c=strrchr(path,'/');
		if (c) {
			*c++=0;
			gtk_file_chooser_set_current_folder((gpointer)dialog,path);
			gtk_file_chooser_set_current_name((gpointer)dialog,c);
		}
	}
	gtk_file_chooser_set_do_overwrite_confirmation((gpointer)dialog,TRUE);
#if GTK_CHECK_VERSION(2,18,0)
	gtk_file_chooser_set_create_folders((gpointer)dialog,TRUE);
#endif
	if ((n=gtk_dialog_run((gpointer)dialog))==GTK_RESPONSE_ACCEPT){
		strcpy(path,gtk_file_chooser_get_filename((gpointer)dialog));
	}
	gtk_widget_destroy(dialog);
	return n==GTK_RESPONSE_ACCEPT;
}

extern int ignore_oor;
static void save_dic_file(int do_ask)
{
	struct MyGtkEditor *main_editor,*dic_editor;
	char path[PATH_MAX+1],*fname;
	int cp=0;char *c;
	if (current_editor->editor_type == MILEDIT_MODE_NORMAL) {
		main_editor=current_editor;
		dic_editor=current_editor->dic_editor;
	}
	if (current_editor->editor_type == MILEDIT_MODE_DIC) {
		main_editor=current_editor->owneditor;
		dic_editor=current_editor;
	}
	else return;
	if (!gtk_text_buffer_get_char_count(dic_editor->buf)) return;
	if (!main_editor || !dic_editor) return;
	path[0]=0;
	if (do_ask || !dic_editor->save_name[0]) {
		if (dic_editor->save_name[0]) strcpy(path,dic_editor->save_name);
		else if (dic_editor->load_name[0]) strcpy(path,dic_editor->load_name);
		else if (main_editor->save_name[0]) {
			strcpy(path,main_editor->save_name);
			cp=1;
		}
		else if (main_editor->load_name[0]) {
			strcpy(path,main_editor->load_name);
			cp=1;
		}
		if (path[0]) {
			fname=strrchr(path,'/');
			if (!fname) path[0]=0;
			else {
				fname++;
			}
			if (cp) {
				c=strrchr(fname,'.');
				if (!c) c=fname+strlen(fname);
				strcpy(c,".dic");
			}
		}
		if (!get_save_path(path,1)) return;
	}
	else strcpy(path,dic_editor->save_name);
	
	GtkTextIter start,end;
	gtk_text_buffer_get_start_iter((gpointer)dic_editor->buf,&start);
	gtk_text_buffer_get_end_iter((gpointer)dic_editor->buf,&end);
	char *str=gtk_text_buffer_get_text((gpointer)dic_editor->buf,&start,&end,FALSE);
	//str=get_actual_body();
	ignore_oor=1;
	int n=to_iso2(str,NULL);
	if (!n) {
		g_free(str);
		return;
	}
	char *istr=malloc(n);
	to_iso2(str,istr);
	g_free(str);
	FILE *f=fopen(path,"w");
	if (!f) {
		g_perror(path);
		free(istr);
		return;
	}
	fwrite(istr,1,strlen(istr),f);
	fclose(f);
	free(istr);
	strcpy(dic_editor->save_name,path);
	gtk_text_buffer_set_modified((gpointer)dic_editor->buf,FALSE);
	Info("Potwierdzenie","Plik słownika został zapisany");
	
}


void notek_set_tab_label(char *text)
{
	get_current_editor();
	GtkWidget *label=gtk_label_new(text);
	gtk_notebook_set_tab_label((gpointer)notek,(gpointer)current_editor->window,label);
	atk_object_set_name(gtk_widget_get_accessible((gpointer)label),text);
	atk_object_set_name(gtk_widget_get_accessible((gpointer)tresc_v),text);
	//printf("New label %s\n",text);
}

void save_file(GtkWidget *dummy,void *data)
{
	int edimode=get_current_editor();
	char cp[PATH_MAX+1];
	FILE *f;
	int do_rename_tab=0;
	if (edimode<0) return;
	if (edimode == MILEDIT_MODE_DIC) {
		save_dic_file(data?1:0);
		return;
	}
	
	if (edimode != 0) return;
	if (!gtk_text_buffer_get_char_count(tresc)) return;
	
	if (data || !current_editor->save_name[0]) {
		char *c;
		cp[0]=0;
		if (current_editor->save_name[0]) strcpy(cp,current_editor->save_name);
		else if (current_editor->load_name[0]) {
			strcpy(cp,current_editor->load_name);
		}
		else if (last_file_path[0]) {
			strcpy(cp,last_file_path);
		}
		if (cp[0]) {
			c=strrchr(cp,'/');
			if (!c) c=cp;
			c=strrchr(c,'.');
			if (c) strcpy(c,".txt");
			else strcat(cp,".txt");
		}
		if (!get_save_path(cp,0)) return;
		do_rename_tab=1;
	}
	else strcpy(cp,current_editor->save_name);
	f=fopen(cp,"w");
	if (!f) {
		g_perror(cp);
		return;
	}
	char *str=get_current_iso2();
	fwrite(str,1,strlen(str),f);
	fclose(f);
	free(str);
	strcpy(current_editor->save_name,cp);
	gtk_text_buffer_set_modified((gpointer)tresc,FALSE);
	if (do_rename_tab) {
		char *c=strrchr(cp,'/');
		if (c) c++;else c=cp;
		notek_set_tab_label(c);
#ifdef USE_SUBMIXER
		enable_video_menues(current_editor->editor_type ==0,current_editor);
#endif
		/*
		gtk_notebook_set_tab_label_text((gpointer)notek,
				(gpointer)current_editor->window,
				c);
		*/
	}
	Info("Potwierdzenie","Plik został zapisany");
	
	
}

static char *crlf2lf(char *body)
{
	char *d,*c;
	int ncp=0;
	d=c=body;
	while (*c && isspace(*c)) c++;
	while (*c) {
		if (*c=='\n') {
			if (ncp<2) ncp++;
			c++;
			continue;
		}
		if (*c!='\r') {
			while (ncp > 0) {
				*d++='\n';
				ncp--;
			}
			*d++=*c++;
			continue;
		}
		if (ncp<2) ncp++;
		c++;
		if (*c=='\n') c++;
	}
	*d=0;
	return body;
}

static char *deshy(char *body)
{
	char *c,*d;
	for (d=c=body;*c;) {
		if (*c != '\xc2') {
			*d++=*c++;continue;
		}
		c++;
		if (!*c) {
			*d++=0xc2;
			break;
		}
		if (*c != '\xad') {
			*d++=0xc2;
			*d++=*c++;
			continue;
		}
		c++;
		while (*c==' ') c++;
		if (*c=='\n') c++;
	}
	*d=0;
	return body;
}

static void simplify_path(char *fname)
{
	char *c,*d;
	for(c=d=fname;*c;) {
		if (*c!='/') {
			*d++=*c++;
			continue;
		}
		if (c[1]=='.') {
			if (c[2]=='/') {
				c+=2;
				continue;
			}
			if (c[2]=='.' && c[3]=='/') {
				if (d>fname) d--;
				while (d>fname) {
					if (*d == '/') break;
					d--;
				}
				c+=3;
				continue;
			}
		}
		*d++=*c++;
	}
	*d=0;
}

int auto_set_langs(char *s)
{
	int flags_set=0;
	int i;char *c,*d,oh[NUM_LANGS];
	struct miltheme *mt;
	for (;;) {
		while (*s && isspace(*s)) s++;
		if (strncmp(s,"//",2)) break;
		s+=2;
		while (*s==' ' || *s=='\t') {
			s++;
		}
		if (*s++!='#') break;
		if (!(flags_set & 1) && !strncmp(s,"LANGS:",6)) {
			s+=6;
			for (i=0;i<NUM_LANGS;i++) oh[i]=0;
			while (*s) {
				if (!islower(s[0]) || !islower(s[1]) || (s[2]!=',' && s[2] != '\n')) break;
				for (i=0;i<NUM_LANGS;i++) if (!strncmp(s,lang_codes[i],2)) {
					oh[i]=1;
					break;
				}
				s+=2;
				if (*s != ',') break;
				s++;
			}
			for (i=0;i<NUM_LANGS;i++) gtk_toggle_button_set_active((gpointer)lang_cb[i],oh[i]!=0);
			flags_set |= 1;
			continue;
		}
		if (!(flags_set & 2) && !strncmp(s,"THEMES:",7)) {
			s+=7;
			for (mt=themes;mt;mt=mt->next) {
				gtk_toggle_button_set_active((gpointer)mt->cb,0);
			}
			for (;;) {
				if (!*s || !islower(*s)) break;
				for (mt=themes;mt;mt=mt->next) {
					for (c=s,d=mt->tname;*d;c++,d++) {
						if (*c!=*d) break;
					}
					if (*d && (*c!=',' && *c!='\n')) continue;
					s=c;
					if (*s==',') s++;
					gtk_toggle_button_set_active((gpointer)mt->cb,1);
					break;
				}
			}
			flags_set |= 2;
			continue;
		}
		break;
	}
	return 1;
		
}
int auto_set_langs_old(char *s)
{
	int i;char *c,oh[NUM_LANGS];
	
	if (strncmp(s,"//#LANGS:",9)) return 0;
	s+=9;
	for (i=0;i<NUM_LANGS;i++) oh[i]=0;
	while (*s) {
		if (!islower(s[0]) || !islower(s[1]) || (s[2]!=',' && s[2] != '\n')) break;
		for (i=0;i<NUM_LANGS;i++) if (!strncmp(s,lang_codes[i],2)) {
			oh[i]=1;
			break;
		}
		s+=2;
		if (*s != ',') break;
		s++;
	}
	for (i=0;i<NUM_LANGS;i++) gtk_toggle_button_set_active((gpointer)lang_cb[i],oh[i]!=0);
	return 1;
}

static int fbody_is_hash(char *c)
{
	while (*c && isspace(*c)) c++;
	if (!c) return 0;
	c=strpbrk(c,"\r\n");
	if (!c) return 0;
	while (*c && isspace(*c)) c++;
	return (*c == '#');
}

void open_file(GtkWidget *widget,void *data)
{
	char *fname=NULL;
	char *fbody;
	char dic_path[PATH_MAX+1];
	char buff[PATH_MAX+32];
	struct stat sb;
	int is_include = (widget && (data != NULL));
	int is_g,is_hash;
	static int new_pdf_mode=1;
	static int rtf_foot_mode=0;
	if (is_include && get_current_editor()) return;
	if (widget) {
		GtkWidget *dialog=gtk_file_chooser_dialog_new(
			is_include?"Wybierz plik do wstawienia":"Wybierz plik",
			GTK_WINDOW(main_window),GTK_FILE_CHOOSER_ACTION_OPEN,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			NULL);
		GtkFileFilter *filter;
		GtkWidget *pdf_xml_cb=gtk_check_button_new_with_label("Użyj pdf2xml przy wczytywaniu PDF/PS");
		GtkWidget *rtf_foot_on=combo(rtf_foot_mode,"Ignoruj","W treści","Za akapitem","Na końcu",NULL);
		GtkWidget *rtf_box=gtk_hbox_new(0,2);
		GtkWidget *rtf_label=gtk_label_new("Przypisy w plikach RTF");
		gtk_box_pack_start(GTK_BOX(rtf_box),rtf_label,FALSE,FALSE,0);
		gtk_box_pack_start(GTK_BOX(rtf_box),rtf_foot_on,FALSE,FALSE,0);
		connect_label(rtf_label,rtf_foot_on);
		gtk_toggle_button_set_active((gpointer) pdf_xml_cb,new_pdf_mode);
		gtk_file_chooser_set_local_only((gpointer)dialog,TRUE);
		gtk_file_chooser_add_filter((gpointer) dialog,
			make_filter("Pliki Worda","*.doc","*.DOC","*.rtf","*.RTF",NULL));
		gtk_file_chooser_add_filter((gpointer) dialog,	
			make_filter("Pliki OpenOffice","*.odt",NULL));
		gtk_file_chooser_add_filter((gpointer) dialog,
			make_filter("Pliki tekstowe","*.txt","*.TXT",NULL));
		gtk_file_chooser_add_filter((gpointer) dialog,
			make_filter("Pliki PDF","*.pdf","*.PDF",NULL));
		gtk_file_chooser_add_filter((gpointer) dialog,
			make_filter("Pliki HTML","*.htm","*.HTM","*.html","*.HTML","*.xhtml","*.XHTML",NULL));

#ifdef HAVE_EPUB
		gtk_file_chooser_add_filter((gpointer) dialog,
			make_filter("Pliki ePub","*.epub","*.EPUB",NULL));
#endif
		gtk_file_chooser_add_filter((gpointer) dialog,
			filter=make_filter("Wszystkie pliki","*",NULL));
		gtk_file_chooser_set_filter((gpointer) dialog,filter);
		GtkWidget *ebox=gtk_vbox_new(0,2);
		gtk_box_pack_start(GTK_BOX(ebox),pdf_xml_cb,FALSE,FALSE,0);
		gtk_box_pack_start(GTK_BOX(ebox),rtf_box,FALSE,FALSE,0);
		gtk_file_chooser_set_extra_widget((gpointer) dialog,ebox);
		gtk_widget_show_all(ebox);
		if (last_file_path[0]) {
			gtk_file_chooser_set_filename((gpointer)dialog,last_file_path);
		}
		if (gtk_dialog_run((gpointer)dialog)==GTK_RESPONSE_ACCEPT){
			fname=gtk_file_chooser_get_filename((gpointer)dialog);
		}
		new_pdf_mode=gtk_toggle_button_get_active((gpointer) pdf_xml_cb);
		rtf_foot_mode=gtk_combo_box_get_active((gpointer) rtf_foot_on);
		setFootNote("dine"+rtf_foot_mode);
		gtk_widget_destroy(dialog);
	}
	else {
		fname=data;
		new_pdf_mode = !old_pdf_behav;
	}
	if (!fname) return;
	fbody=read_file(fname,&is_g,new_pdf_mode);
	if (fbody && is_include) {
		fbody=deshy(crlf2lf(fbody));
		gtk_text_buffer_insert_at_cursor(tresc,fbody,strlen(fbody));
		if (is_g)g_free(fbody);
		else free(fbody);
		return;
	}
	char *flabel=strrchr(fname,'/');
	if (!flabel) flabel=fname;
	else flabel++;
	is_hash=0;
	if (fbody) {
		is_hash=fbody_is_hash(fbody);
		get_current_editor();
		if (!current_editor || current_editor->editor_type != 0 || !textbuf_is_empty(tresc)) {
			CreateEditor(0,NULL,flabel);
			atk_object_set_name(gtk_widget_get_accessible((gpointer)tresc_v),flabel);
		}
		else {
			notek_set_tab_label(flabel);
			/*
			gtk_notebook_set_tab_label_text((gpointer)notek,
				(gpointer)current_editor->window,
				flabel);
			*/
		}
		
		set_buffer_text(tresc,deshy(crlf2lf(fbody)),-1,0);
		gtk_widget_grab_focus((gpointer)tresc_v);
		if (is_g) g_free(fbody);
		else free(fbody);
		if (is_hash) {
			char const *xc=gtk_entry_get_text((gpointer)char_split);
			if (*xc == '#') gtk_combo_box_set_active((gpointer)combo_split,1);
		}
	}
	if (g_path_is_absolute(fname)) {
		strcpy(last_file_path,fname);
	}
	else {
		char *cd=g_get_current_dir();
		char *p=g_build_filename(cd,fname,NULL);
		strcpy(last_file_path,p);
		g_free(p);
		g_free(cd);
	}
	simplify_path(last_file_path);
	strcpy(current_editor->load_name,last_file_path);
	strcpy(dic_path,last_file_path);
	g_free(fname);
	
	if (!widget && !auto_load_dic) return;
	char *c=strrchr(dic_path,'/');
	if (!c) return;
	*c=0;
	fname=c+1;
	char *d=strrchr(fname,'.');
	if (!d) d=fname+strlen(fname);
	strcpy(d,".dic");
	*c='/';
	if (stat(dic_path,&sb)) return;
	if (widget) {
		sprintf(buff,"Załadować słownik %s?",fname);
		if (!Ask("Pytanie",buff)) return;
	}
	fbody=read_file(dic_path,&is_g,0);
	struct MyGtkEditor *edi=get_subeditor(current_editor,MILEDIT_MODE_DIC);
	set_buffer_text(edi->buf,crlf2lf(fbody),-1,0);
	strcpy(edi->load_name,dic_path);
	auto_set_langs(fbody);
	if (is_g)g_free(fbody);
	else free(fbody);
	
}

void open_dic(GtkWidget *widget,void *dummy)
{
	char dic_path[PATH_MAX+1];
	if (get_current_editor()) return;
	dic_path[0]=0;
	if (current_editor->save_name[0]) {
		strcpy(dic_path,current_editor->save_name);
	}
	else if (current_editor->load_name[0]) {
		strcpy(dic_path,current_editor->load_name);
	}
	char *fname;
	GtkWidget *dialog=gtk_file_chooser_dialog_new("Wybierz plik",
		GTK_WINDOW(main_window),GTK_FILE_CHOOSER_ACTION_OPEN,
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
		NULL);
	GtkFileFilter *filter;
	gtk_file_chooser_set_local_only((gpointer)dialog,TRUE);
	gtk_file_chooser_add_filter((gpointer) dialog,
		filter=make_filter("Pliki DIC","*.dic","*.DIC",NULL));
	gtk_file_chooser_add_filter((gpointer) dialog,
		make_filter("Pliki tekstowe","*.txt","*.TXT",NULL));
	gtk_file_chooser_add_filter((gpointer) dialog,
		make_filter("Wszystkie pliki","*",NULL));
	gtk_file_chooser_set_filter((gpointer) dialog,filter);
	
	if (dic_path[0]) {
		char *c=strrchr(dic_path,'/');
		struct stat sb;
		if (c) {
			*c=0;
			fname=c+1;
		}
		else {
			fname=dic_path;
		}
		char *d=strrchr(fname,'.');
		if (!d) d=fname+strlen(fname);
		strcpy(d,".dic");
		if (c) *c='/';
		if (!stat(dic_path,&sb)) {
			gtk_file_chooser_set_filename((gpointer)dialog,dic_path);
		}
		else if (c) {
			*c=0;
			gtk_file_chooser_set_current_folder((gpointer)dialog,dic_path);
		}
	}
	fname=NULL;
	if (gtk_dialog_run((gpointer)dialog)==GTK_RESPONSE_ACCEPT){
		fname=gtk_file_chooser_get_filename((gpointer)dialog);
	}
	gtk_widget_destroy(dialog);
	if (!fname) return;
	char *fbody=read_file(fname,NULL,0);
	struct MyGtkEditor *edi=get_subeditor(current_editor,MILEDIT_MODE_DIC);
	set_buffer_text(edi->buf,fbody,-1,0);
	strcpy(edi->load_name,fname);
	auto_set_langs(fbody);
	free(fbody);
	g_free(fname);
}

