/*
 * gmilena.h - Milena Audiobook Creator
 * Copyright (C) Bohdan R. Rau 2009 <ethanak@polip.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write see:
 *               <http://www.gnu.org/licenses/>.
 */
#ifndef GMILENA_H
#define GMILENA_H 1
#include <gtk/gtk.h>
#include <gtksourceview/gtksourceview.h>

#define GMILENA_VERSION "0.1.17"
extern GdkCursor *watch_cursor;

extern GtkWidget *m_ac_search,*m_ac_atk,*m_ac_nop,*m_speaker_follow,*m_tiface;
extern char font_name[];
extern void connect_label(gpointer a,gpointer b);
extern GtkWidget *main_window,*notek;
extern GtkTextBuffer *tresc;
extern GtkTextView *tresc_v;
extern GtkWidget *spin_tempo,*spin_pitch;
#define NUM_LANGS 8
extern GtkWidget *lang_cb[NUM_LANGS];
extern char *lang_codes[NUM_LANGS];
extern GtkWidget *cb_hours,*cb_pro, *cb_ign, *cb_dic, *cb_lector, *cb_spell, *cb_ellip, *cb_equ;
extern GtkWidget *dic_filechooser;
extern GtkWidget *cb_kontrast,*spin_kontrast;
extern GtkWidget *combo_mp3,*cb_stereo,*combo_amr,*cb_dtx,*combo_split,*char_split,
	*combo_autonum,*combo_prolog,*combo_epilog;
extern GtkWidget *statusbar,*statusbar_ove,*statusbar_line,*statusbar_col;
extern GtkFileFilter *make_filter(char *name,...);
extern int to_iso2(char *instr,char *outstr);
extern int to_iso2_mp(char *instr,char *outstr,char **instrpos,int maxpos);
extern int get_unichar(char **s);
extern int znak_is_alnum(int znak);

extern int Ask(char *title,char *question);
extern void Error(char *title,char *error);
extern void Info(char *title,char *info);
extern int Ask2(char *title,char *question,gpointer w);
extern void Error2(char *title,char *error,gpointer w);
extern void Info2(char *title,char *info,gpointer w);
extern int GetNumber(char *title,char *labelstr);
extern void open_file(GtkWidget *dummy,void *data);
extern void new_file(GtkWidget *dummy,void *data);
extern void save_file(GtkWidget *dummy,void *data);
extern void convert_to_iso2(GtkWidget *dummy,void *data);
extern void close_current_editor(GtkWidget *dummy,void *data);
extern void make_dash(GtkWidget *dummy, void *data);
extern void apply_dash(GtkWidget *dummy, void *data);
extern void unformat_text(GtkWidget *dummy, void *data);
extern void correct_jacosub(GtkWidget *dummy, void *data);
extern void make_dict(GtkWidget *dummy, void *data);
extern void swap_editors(GtkWidget *dummy, void *data);
extern void ChooseFont(GtkWidget *widget,void *dummy);
extern void ApplyFont(char *);
extern void StartSearch(GtkWidget *widget,void *dummy);
extern void StartReplace(GtkWidget *widget,void *dummy);
extern void editor_ulcase(GtkWidget *widget,void *data);
extern void editor_fake_latin(GtkWidget *widget,void *data);
extern void menu_active_notek(GtkWidget *widget,void *dummy);
extern void find_current_word(GtkWidget *widget,void *dummy);
extern void find_entity(GtkWidget *widget,void *dummy);
extern void find_scanner_error(GtkWidget *widget,void *dummy);
extern void find_dialog_connection(GtkWidget *widget,void *dummy);
extern void go_to_line(GtkWidget *widget,void *dummy);
extern void save_prefs(GtkWidget *widget,void *dummy);
extern void open_dic(GtkWidget *widget,void *dummy);
extern void set_mbrola_voice(GtkWidget *widget,void *dummy);
extern void book_create(GtkWidget *widget,void *dummy);
extern void acc_make_wrap(void);
void editor_delete_line(GtkWidget *widget,void *data);

extern struct MyGtkEditor *get_subeditor(struct MyGtkEditor *editor,int mode);
extern void g_perror(char *);
extern char *get_actual_body(void);
extern void start_reading(int);
extern void stop_speaking(void);
extern void load_prefs(void);
extern void show_position_on_sb(struct MyGtkEditor *ed);
extern int is_speaking;
extern struct MyGtkEditor *gimma_current_editor(void);

extern struct mbrola_handler {
	int to_mbrola;
	int from_mbrola;
	int pid;
	int flush_status;
	int error;
} *mbr;
int write_MBR(struct mbrola_handler *mbr,char *buf);
int flush_MBR(struct mbrola_handler *mbr);
int read_MBR(struct mbrola_handler *mbr,short *buf,int len,int *outlen);
struct mbrola_handler *init_MBR(char *voice,int freq);
void close_MBR(struct mbrola_handler *mbr);

int audio_play(short *samples,int nsamples);
char *ask_encoding(void);

extern struct MyGtkEditor {
	struct MyGtkEditor *next;
	GtkTextView *view;
	GtkTextBuffer *buf;
	GtkScrolledWindow *window;
	int editor_type;
	struct MyGtkEditor *dash_editor;
	struct MyGtkEditor *dic_editor;
	struct MyGtkEditor *owneditor;
	GtkTextTag *speaktag;
	char load_name[PATH_MAX+1];
	char save_name[PATH_MAX+1];
} *editors,*current_editor;

#define MILEDIT_MODE_NORMAL 0
#define MILEDIT_MODE_DASH 1
#define MILEDIT_MODE_DIC 2

extern int get_current_editor();
extern char *get_current_iso2();
extern char *get_editor_body(struct MyGtkEditor*);

extern struct milena *milena;
extern struct milena_mbrola_cfg *mimbrola;

int init_milena_dic(int is_dic);

extern char last_file_path[];

extern int my_alpha(int);
extern int my_alnum(int);
extern int my_upper(int);
extern int my_space(int);
extern int my_digit(int);
extern int my_vowel(int);
extern int readable(char *);

extern void remove_subeditor(struct MyGtkEditor *editor);
extern void set_buffer_text(gpointer buf,char *str,int len,int undoable);

extern void start_busy_cursor(void);
extern void end_busy_cursor(void);

extern char mbrola_voice[];

extern int ignore_oor;
extern int count_bad_char;

void StartProgressMeter(int count);
void SetProgress(int file,double pos);
void StopProgressMeter(void);
extern int allow_run;

extern int init_paragraph(void);
extern struct phone_buffer *translate_paragraph(char *str);
extern char *translate_subtitle(char *str);
extern int contrast;
extern double contrast_level;
extern gdouble current_mbrola_speed;

extern int init_lame_encoder(char *fname,int rate,char *artist,char *album,char *title,int track);
extern int lame_encode(short *buf,int nsamples);
extern int lame_finish(void);

extern int init_amrwb(void);
extern int start_amrwb(char *fname,int mode,int dtx);
extern int encode_amrwb(short *buf,int nsamples);
extern int finish_amrwb(void);
extern double amr_get_seconds(void);

extern int init_aac_encoder(char *fname);
extern int aac_encode(short *buf,int nsamples);
extern int finish_aac();

extern struct miltheme {
	struct miltheme *next;
	GtkWidget *cb;
	char *fname;
	char *tname;
	char *title;
} *themes;

extern gchar **initial_fnames;
extern gint auto_load_dic;
extern int auto_set_langs(char *);

extern int multipartial_on;
extern double multipartial_value,multipartial_margin;
extern char *read_epub(char *, char **);

extern int id3_tag_mode;

#define word_wrap (gtk_check_menu_item_get_active((gpointer)m_ac_nop)?\
GTK_WRAP_NONE:\
gtk_check_menu_item_get_active((gpointer)m_ac_atk)?\
GTK_WRAP_WORD:GTK_WRAP_WORD_CHAR)

#define a11y_not_found gtk_check_menu_item_get_active((gpointer)m_ac_search)

void *reverb_init(double rvb,double damp,double room,double depth,int upsample);
void reverb_flow(void *vreff, const short * ibuf,short * obuf, size_t len);
void reverb_free(void *vreff);

extern int stereo_mode,stereo_upsample;
extern double stereo_rev,stereo_hfd,stereo_room,stereo_depth;
extern int dont_close_audiodevice;

extern gdouble vocal_track_length;
extern int mbrola_real_freq;
extern gdouble resample_factor;
extern gdouble computed_resample_factor;
extern int use_resampler;
extern int init_resampler(void);
extern int close_resampler(void);
extern int do_resample(short *,int);

extern char *pdxml2txt(char *);

extern gint old_pdf_behav;

extern char *to_utf8(char *fbuf,int flen,char *cset,int freeme);
extern int find_bad_construction(char *str,int *wrlen);
extern char *trim(char *str);
extern char scanner_path[256];
extern int readScanner(void);
extern int debug_level;

extern GtkWidget *combo(int defpos,...);

extern char *jacosub_simplify(char *,int);
extern int jacosub_timeres(char *);
extern int get_current_timeres(void);
extern int jacosub_find_dialcon(char *txt,int *size,int timeres);
extern int get_jaco_timer(char *str,int timeres);

#ifdef USE_SUBMIXER
extern void mplayer_read_movie(GtkWidget *widget, void *dummy);
extern void mplayer_make_audio(GtkWidget *widget, void *dummy);
extern void mplayer_make_video(GtkWidget *widget, void *dummy);
extern void mplayer_settings(GtkWidget *widget, void *dummy);
extern void enable_video_menues(int,struct MyGtkEditor *);


extern double smix_speed;
extern double smix_maxspeed;
extern int smix_droptime,smix_risetime,smix_mindist;
extern int smix_audio_margin;
extern double smix_volume_factor;
extern double smix_max_origin_vol;

#endif

#endif
