/*
 * main.c - Milena Audiobook Creator
 * Copyright (C) Bohdan R. Rau 2009-2010 <ethanak@polip.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write see:
 *               <http://www.gnu.org/licenses/>.
 */
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <locale.h>
#include "gmilena.h"
#include <dirent.h>
#include <milena.h>
GdkCursor *watch_cursor;
GtkWidget *main_window;
GtkWidget *uframe,*notek;
GtkTextBuffer *tresc;
GtkTextView *tresc_v;
GtkWidget *spin_tempo,*spin_pitch;
GtkWidget *lang_cb[NUM_LANGS];
char *lang_codes[NUM_LANGS]={"en","fr","es","it","ru","de","hu","ro"};
char *lang_names[NUM_LANGS]={"Angielski","Francuski","Hiszpański","Włoski",
	"Rosyjski","Niemiecki","Węgierski","Rumuński"};

GtkWidget *cb_hours,*cb_pro, *cb_ign, *cb_dic, *cb_lector, *cb_spell , *cb_ellip, *cb_equ;

GtkWidget *dic_filechooser;


GtkWidget *cb_kontrast,*spin_kontrast;

GtkWidget *combo_mp3,*cb_stereo,*combo_amr,*cb_dtx,*combo_split,*char_split,
	*combo_autonum,*combo_prolog,*combo_epilog;

GtkWidget *menubar,*filemenu,*speechmenu,*toolbar,*helpmenu,*accessmenu,*lookmenu;
#ifdef USE_SUBMIXER
GtkWidget *videomenu;
#endif

GtkWidget *m_play,*t_play,*m_stop,*t_stop,*t_prop,*m_open,*m_include,*m_new,*m_edit,*t_open,*t_new,
	*m_close,*t_close,*m_save,*m_saveas,*m_opendic,*t_save,*t_saveas,*m_search,
	*m_dash,*m_dict,*m_dict2,*m_adash,*m_unfor,*m_edswap,*m_gotodash,
	*m_findcur,*m_voice,*m_stereo,*m_mp3,*m_amr,*m_help,*m_entity,*m_scanner,*m_access,*m_ac_search,
	*m_ac_atk,*m_ac_nop,*m_ac_go_edi,*t_prop,*m_speaker_follow,*m_tiface,*m_togtool,
	*m_fullscreen,*m_keep_audio,*m_unjaco,*m_dialconn
#ifdef USE_SUBMIXER
	,*m_video,*m_video_open,*m_video_caud,*m_video_cmov,*m_video_settings
#endif
#ifdef USE_RESAMPLER	
	,*m_resample
#endif
	;

GtkWidget *statusbar,*statusbar_ove,*statusbar_line,*statusbar_col;

struct miltheme *themes;
gint debug_level=0;

char *trim(char *str)
{
	char *c,*d;
	while (*str && isspace(*str)) str++;
	for (c=d=str;*c;c++) if (!isspace(*c)) d=c+1;
	*d=0;
	return str;
}


int GetNumber(char *title,char *labelstr)
{
	GtkWidget *dialog=gtk_dialog_new_with_buttons(
		title,
		(gpointer)main_window,
		GTK_DIALOG_MODAL,
		GTK_STOCK_OK,GTK_RESPONSE_ACCEPT,
		GTK_STOCK_CANCEL,GTK_RESPONSE_REJECT,
		NULL);
	GtkWidget *box=gtk_dialog_get_content_area((gpointer)dialog);
	GtkWidget *hbox=gtk_hbox_new(0,2);
	gtk_box_pack_start(GTK_BOX(box),(gpointer)hbox,FALSE,FALSE,0);
	GtkWidget *label=gtk_label_new(labelstr);
	GtkWidget *entry=gtk_entry_new();
	connect_label(label,entry);
	gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox),entry,FALSE,FALSE,0);
	gtk_widget_show_all((gpointer)box);
	int n=-1;
	gtk_dialog_set_default_response((gpointer)dialog,GTK_RESPONSE_ACCEPT);
	gtk_entry_set_activates_default((gpointer)entry,TRUE);
	if (gtk_dialog_run((gpointer)dialog) == GTK_RESPONSE_ACCEPT) {
		char const *str=gtk_entry_get_text((gpointer)entry);
		char *estr;
		n=strtol(str,&estr,10);
		if (estr==str) n=-1;
	}
	gtk_widget_destroy(dialog);
	return n;
}

int Ask2(char *title,char *question,gpointer w)
{
	GtkWidget *dialog;
	int n;
	dialog = gtk_message_dialog_new(w,
		GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_MESSAGE_QUESTION,
		GTK_BUTTONS_YES_NO,
            	"%s",question);
	gtk_window_set_title(GTK_WINDOW(dialog), title);
	n=gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
	return n==GTK_RESPONSE_YES;
}

int Ask(char *title,char *question)
{
	return Ask2(title,question,(gpointer)main_window);
}

void Error2(char *title,char *error,gpointer w)
{
	GtkWidget *dialog;
	int n;
	dialog = gtk_message_dialog_new(w,
		GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_MESSAGE_ERROR,
		GTK_BUTTONS_OK,
            	"%s",error);
	gtk_window_set_title(GTK_WINDOW(dialog), title);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
}

void Error(char *title,char *error)
{
	return Error2(title,error,(gpointer)main_window);
}

void Info2(char *title,char *info,gpointer w)
{
	GtkWidget *dialog;
	int n;
	dialog = gtk_message_dialog_new(w,
		GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_MESSAGE_INFO,
		GTK_BUTTONS_OK,
            	"%s",info);
	gtk_window_set_title(GTK_WINDOW(dialog), title);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
}

void Info(char *title,char *info)
{
	return Info2(title,info,(gpointer)main_window);
}

void g_perror(char *s) {
	char buf[1024];
	sprintf(buf,"%s:\n%s",s,strerror(errno));
	Error("Błąd",buf);
}

int close_main_window(void)
{
	audio_play(NULL,0);
	if (!Ask("Pytanie","Naprawdę kończysz?")) return TRUE;
	gtk_main_quit();
	exit(0);
}

static void set_dicfile_active(GtkWidget *dummy)
{
	gtk_widget_set_sensitive(dic_filechooser,
		(!is_speaking) && gtk_toggle_button_get_active((gpointer)cb_dic));
}

static void set_kontrast_active(GtkWidget *dummy)
{
	gtk_widget_set_sensitive(spin_kontrast,
		gtk_toggle_button_get_active((gpointer)cb_kontrast));
}


static void set_lector_active(GtkWidget *dummy)
{
	gtk_widget_set_sensitive(cb_lector,
		!gtk_toggle_button_get_active((gpointer)cb_spell));
	gtk_widget_set_sensitive(cb_spell,
		!gtk_toggle_button_get_active((gpointer)cb_lector));
}

static void toggle_interface_menu(GtkWidget *widget)
{
	if (gtk_check_menu_item_get_active((gpointer)widget)) {
		gtk_toggle_tool_button_set_active((gpointer)t_prop,TRUE);
		gtk_widget_show(uframe);
	}
	else {
		gtk_toggle_tool_button_set_active((gpointer)t_prop,FALSE);
		gtk_widget_hide(uframe);
	}
		
	
}

static void toggle_toolbar(GtkWidget *widget)
{
	if (gtk_check_menu_item_get_active((gpointer)widget)) {
		gtk_widget_show(toolbar);
	}
	else {
		gtk_widget_hide(toolbar);
	}
		
}

static void toggle_fullscreen(GtkWidget *widget)
{
	if (gtk_check_menu_item_get_active((gpointer)widget)) {
		gtk_window_fullscreen(main_window);
	}
	else {
		gtk_window_unfullscreen(main_window);
	}
		
}

static void toggle_iface(GtkWidget *widget)
{
	if (gtk_toggle_tool_button_get_active((gpointer)widget)) {
		gtk_check_menu_item_set_active((gpointer)m_tiface,TRUE);
		gtk_widget_show(uframe);
	}
	else {
		gtk_check_menu_item_set_active((gpointer)m_tiface,FALSE);
		gtk_widget_hide(uframe);
	}
}

static void toggle_audiodevice(GtkWidget *widget)
{
	dont_close_audiodevice=gtk_check_menu_item_get_active((gpointer)widget);
}

void connect_label(gpointer a,gpointer b)
{
	AtkObject *label;
	AtkObject *widget;
	label=gtk_widget_get_accessible(a);
	widget=gtk_widget_get_accessible(b);
	atk_object_add_relationship(label,ATK_RELATION_LABEL_FOR,widget);
	atk_object_add_relationship(widget,ATK_RELATION_LABELLED_BY,label);
	if (gtk_label_get_mnemonic_keyval(a) != GDK_VoidSymbol) {
		gtk_label_set_mnemonic_widget(a,b);
	}
}

GtkWidget *combo(int defpos,...)
{
	GtkWidget *c=gtk_combo_box_new_text();
	va_list ap;
	va_start(ap,defpos);
	for (;;) {
		char *s=va_arg(ap,char *);
		if (!s) break;
		gtk_combo_box_append_text((gpointer)c,s);
	}
	va_end(ap);
	gtk_combo_box_set_active((gpointer)c,defpos);
	return c;
}


static void set_autonum_active(GtkWidget *dummy)
{
	int v1=gtk_combo_box_get_active((gpointer)combo_split);
	int v2=0;
	if (v1) v2=gtk_combo_box_get_active((gpointer)combo_autonum);
	gtk_widget_set_sensitive(char_split,v1);
	gtk_widget_set_sensitive(combo_autonum,v1);
	gtk_widget_set_sensitive(combo_prolog,v2);
	gtk_widget_set_sensitive(combo_epilog,v2);
}


int is_speaking;

void speech_buttons_enable(int is_speaking)
{
	struct MyGtkEditor *editor=gimma_current_editor();
	int mode=-1;
	if (editor) mode=editor->editor_type;
	gtk_widget_set_sensitive(t_stop,is_speaking);
	gtk_widget_set_sensitive(m_stop,is_speaking);
	gtk_widget_set_sensitive(t_play,!is_speaking);
	gtk_widget_set_sensitive(m_play,!is_speaking);
	gtk_widget_set_sensitive(m_new,!is_speaking);
	gtk_widget_set_sensitive(m_open,!is_speaking);
	gtk_widget_set_sensitive(m_include,!is_speaking && (mode==MILEDIT_MODE_NORMAL));
	gtk_widget_set_sensitive(m_opendic,!is_speaking);
	gtk_widget_set_sensitive(m_edit,!is_speaking);
	gtk_widget_set_sensitive(m_search,!is_speaking);
	gtk_widget_set_sensitive(m_save,!is_speaking);
	gtk_widget_set_sensitive(m_saveas,!is_speaking);
	gtk_widget_set_sensitive(t_open,!is_speaking);
	gtk_widget_set_sensitive(t_new,!is_speaking);
	gtk_widget_set_sensitive(t_save,!is_speaking);
	gtk_widget_set_sensitive(t_saveas,!is_speaking);
	gtk_widget_set_sensitive(m_close,!is_speaking);
	gtk_widget_set_sensitive(t_close,!is_speaking);
	gtk_widget_set_sensitive(m_voice,!is_speaking);
	gtk_widget_set_sensitive(m_stereo,!is_speaking);
#ifdef USE_RESAMPLER
	gtk_widget_set_sensitive(m_resample,!is_speaking);
#endif
#ifdef USE_SUBMIXER
	gtk_widget_set_sensitive(m_video,!is_speaking);
#endif
	
	gtk_widget_set_sensitive(m_help,!is_speaking);
	gtk_widget_set_sensitive(m_amr,!is_speaking && (mode==MILEDIT_MODE_NORMAL));
	gtk_widget_set_sensitive(m_mp3,!is_speaking && (mode==MILEDIT_MODE_NORMAL));

	gtk_widget_set_sensitive(dic_filechooser,
		(!is_speaking) && gtk_toggle_button_get_active((gpointer)cb_dic));
	
	gtk_widget_set_sensitive(notek,!is_speaking);
	
}

static void speech_play()
{
	if (is_speaking) return;
	start_reading(0);
}

static void speech_stop()
{
	is_speaking=0;
}

double stereo_rev=50.0,stereo_hfd=50,stereo_room=100,stereo_depth=100;

#ifdef USE_RESAMPLER

void compute_resample_par(void)
{
	resample_factor=computed_resample_factor/vocal_track_length;
	mbrola_real_freq=16000/vocal_track_length;
}

static void resampler_dialog(void)
{
	GtkWidget *dialog=gtk_dialog_new_with_buttons(
		"Resampler",
		(gpointer)main_window,
		GTK_DIALOG_MODAL,
		GTK_STOCK_OK,GTK_RESPONSE_ACCEPT,
		GTK_STOCK_CANCEL,GTK_RESPONSE_REJECT,
		NULL);
	GtkWidget *box=gtk_dialog_get_content_area((gpointer)dialog);
	GtkWidget *vbox=gtk_vbox_new(0,2);
	gtk_box_pack_start(GTK_BOX(box),(gpointer)vbox,FALSE,FALSE,0);

	GtkWidget *usesamp_cb=gtk_check_button_new_with_label("Włącz resampling");
	gtk_box_pack_start(GTK_BOX(vbox),usesamp_cb,FALSE,FALSE,0);
	GtkWidget *stable=gtk_table_new(2,2,0);
	gtk_box_pack_start(GTK_BOX(vbox),stable,FALSE,FALSE,0);
	
	GtkWidget *label=gtk_label_new("Długość toru wokalnego");
	gtk_table_attach_defaults((gpointer)stable,label,0,1,0,1);
	GtkWidget *spin_vtr=gtk_spin_button_new_with_range(0.6,1.6,0.01);
	gtk_table_attach_defaults((gpointer)stable,spin_vtr,1,2,0,1);
	connect_label(label,spin_vtr);
	label=gtk_label_new("Mnożnik częstotliwości");
	gtk_table_attach_defaults((gpointer)stable,label,0,1,1,2);
	GtkWidget *spin_mpx=gtk_spin_button_new_with_range(0.6,1.4,0.01);
	gtk_table_attach_defaults((gpointer)stable,spin_mpx,1,2,1,2);
	connect_label(label,spin_mpx);
	gtk_toggle_button_set_active((gpointer)usesamp_cb,use_resampler);
	gtk_spin_button_set_value((gpointer)spin_vtr,vocal_track_length);
	gtk_spin_button_set_value((gpointer)spin_mpx,computed_resample_factor);
	gtk_widget_show_all(box);
	int n=gtk_dialog_run((gpointer)dialog);
	if (n==GTK_RESPONSE_ACCEPT) {
		use_resampler=gtk_toggle_button_get_active((gpointer)usesamp_cb);
		computed_resample_factor=gtk_spin_button_get_value((gpointer)spin_mpx);
		vocal_track_length=gtk_spin_button_get_value((gpointer)spin_vtr);
		compute_resample_par();
/*
		stereo_rev=gtk_spin_button_get_value((gpointer)spin_rev);
		stereo_hfd=gtk_spin_button_get_value((gpointer)spin_hfd);
		stereo_room=gtk_spin_button_get_value((gpointer)spin_room);
		stereo_depth=gtk_spin_button_get_value((gpointer)spin_depth);
		      */
	}
	gtk_widget_destroy(dialog);
	
}

#endif

static void stereo_dialog(void)
{
	
	GtkWidget *dialog=gtk_dialog_new_with_buttons(
		"Ustawienia trybu Stereo",
		(gpointer)main_window,
		GTK_DIALOG_MODAL,
		GTK_STOCK_OK,GTK_RESPONSE_ACCEPT,
		GTK_STOCK_CANCEL,GTK_RESPONSE_REJECT,
		NULL);
	GtkWidget *box=gtk_dialog_get_content_area((gpointer)dialog);
	GtkWidget *vbox=gtk_vbox_new(0,2);
	gtk_box_pack_start(GTK_BOX(box),(gpointer)vbox,FALSE,FALSE,0);
	
	GtkWidget *stable=gtk_table_new(2,2,0);
	gtk_box_pack_start(GTK_BOX(vbox),stable,FALSE,FALSE,0);
	GtkWidget *label=gtk_label_new("Pogłos");
	gtk_table_attach_defaults((gpointer)stable,label,0,1,0,1);
	GtkWidget *spin_rev=gtk_spin_button_new_with_range(0.0,100.0,1.0);
	gtk_table_attach_defaults((gpointer)stable,spin_rev,1,2,0,1);
	connect_label(label,spin_rev);
	label=gtk_label_new("Damping HF");
	gtk_table_attach_defaults((gpointer)stable,label,0,1,1,2);
	GtkWidget *spin_hfd=gtk_spin_button_new_with_range(0.0,100.0,1.0);
	gtk_table_attach_defaults((gpointer)stable,spin_hfd,1,2,1,2);
	connect_label(label,spin_hfd);
	label=gtk_label_new("Skala pomieszczenia");
	gtk_table_attach_defaults((gpointer)stable,label,0,1,2,3);
	GtkWidget *spin_room=gtk_spin_button_new_with_range(0.0,100.0,1.0);
	gtk_table_attach_defaults((gpointer)stable,spin_room,1,2,2,3);
	connect_label(label,spin_room);
	label=gtk_label_new("Głębokość stereo");
	gtk_table_attach_defaults((gpointer)stable,label,0,1,3,4);
	GtkWidget *spin_depth=gtk_spin_button_new_with_range(0.0,100.0,1.0);
	gtk_table_attach_defaults((gpointer)stable,spin_depth,1,2,3,4);
	connect_label(label,spin_depth);
	gtk_spin_button_set_value((gpointer)spin_rev,stereo_rev);
	gtk_spin_button_set_value((gpointer)spin_hfd,stereo_hfd);
	gtk_spin_button_set_value((gpointer)spin_room,stereo_room);
	gtk_spin_button_set_value((gpointer)spin_depth,stereo_depth);
	
	gtk_widget_show_all(box);
	int n=gtk_dialog_run((gpointer)dialog);
	if (n==GTK_RESPONSE_ACCEPT) {
		stereo_rev=gtk_spin_button_get_value((gpointer)spin_rev);
		stereo_hfd=gtk_spin_button_get_value((gpointer)spin_hfd);
		stereo_room=gtk_spin_button_get_value((gpointer)spin_room);
		stereo_depth=gtk_spin_button_get_value((gpointer)spin_depth);
	}
	gtk_widget_destroy(dialog);
}


static void about_me(void)
{
	GtkWidget *dialog=gtk_about_dialog_new();
	char buf[256];
	gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(dialog),"Milena ABC");
	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(dialog),GMILENA_VERSION);
	gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(dialog),"(C) Bohdan R. Rau 2009-2010");
	sprintf(buf,"Kreator audioksiążek oparty na TTS Milena\nwersja bibliotek %s",milena_GetVersion());
	gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(dialog),buf);
	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(dialog),"http://milena.polip.com");
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
}

static void help_me(void)
{
	Error("Wybacz","Jeszcze nie napisałem...");
	
}

static void show_milena_home(void)
{
	gtk_show_uri(NULL,"http://milena.polip.com",GDK_CURRENT_TIME,NULL);

}

static void acc_go_editor(void)
{
	struct MyGtkEditor *edi=gimma_current_editor();
	if (!edi) return;
	gtk_widget_grab_focus((gpointer)edi->view);
}

static void acc_go_synth(void)
{
	int n=gtk_toggle_tool_button_get_active((gpointer)t_prop);
	if (!n) gtk_toggle_tool_button_set_active((gpointer)t_prop,TRUE);
	gtk_widget_grab_focus(spin_tempo);
}

static void acc_go_conv(void)
{
	int n=gtk_toggle_tool_button_get_active((gpointer)t_prop);
	if (!n) gtk_toggle_tool_button_set_active((gpointer)t_prop,TRUE);
	gtk_widget_grab_focus(cb_kontrast);
}


static void make_main_frame(void)
{
	GtkWidget *frame;
	GtkWidget *lbox;
	GtkScrolledWindow *tresc_sw;
	
	GtkWidget *spins;
	GtkWidget *label_tempo,*label_pitch;
	
	GtkWidget *langframe,*langtb,*leftframe,*riteframe,*mp3tb;
	
	GtkWidget *tfr1,*tfr2,*tfr3,*label;
	
	int i;
	
	gtk_window_set_title(GTK_WINDOW(main_window),"Milena - Kreator audioksiążek");
	frame=gtk_vbox_new(0,2);
	gtk_container_add(GTK_CONTAINER(main_window),frame);

// menu
	GtkAccelGroup *accel=gtk_accel_group_new();
	gtk_window_add_accel_group (GTK_WINDOW (main_window), accel);
	menubar=gtk_menu_bar_new();
	filemenu=gtk_menu_new();
	lookmenu=gtk_menu_new();
	GtkWidget *editmenu=gtk_menu_new();
	GtkWidget *searchmenu=gtk_menu_new();
	speechmenu=gtk_menu_new();
	helpmenu=gtk_menu_new();
	accessmenu=gtk_menu_new();

#ifdef USE_SUBMIXER
	videomenu=gtk_menu_new();
#endif
	
	GtkWidget *m_file = gtk_menu_item_new_with_mnemonic("_Plik");
	m_new = gtk_image_menu_item_new_from_stock(GTK_STOCK_NEW, NULL);
	m_open = gtk_image_menu_item_new_from_stock(GTK_STOCK_OPEN, NULL);
	m_include = gtk_image_menu_item_new_with_label("Wstaw");
	gtk_widget_add_accelerator (m_open, "activate", accel,
                              GDK_o, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	m_close = gtk_image_menu_item_new_from_stock(GTK_STOCK_CLOSE, NULL);
	gtk_widget_add_accelerator (m_close, "activate", accel,
                              GDK_w, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		m_save=gtk_image_menu_item_new_from_stock(GTK_STOCK_SAVE, NULL);
	gtk_widget_add_accelerator (m_save, "activate", accel,
                              GDK_s, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	m_saveas=gtk_image_menu_item_new_from_stock(GTK_STOCK_SAVE_AS, NULL);
	m_opendic=gtk_image_menu_item_new_with_label("Wczytaj DICT");
	GtkWidget *m_quit=gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
	gtk_widget_add_accelerator (m_quit, "activate", accel,
                              GDK_q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(m_file), filemenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), m_new);
	gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), m_open);
	gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), m_include);
	gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), gtk_separator_menu_item_new());
	gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), m_save);
	gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), m_saveas);
	gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), gtk_separator_menu_item_new());
	gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), m_opendic);
	gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), gtk_separator_menu_item_new());
	gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), m_close);
	gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), m_quit);
	gtk_menu_shell_append(GTK_MENU_SHELL(menubar), m_file);
	
	GtkWidget *m_view=gtk_menu_item_new_with_mnemonic("_Wygląd");
	m_tiface=gtk_check_menu_item_new_with_label("Pokaż właściwości");
	m_fullscreen=gtk_check_menu_item_new_with_label("Przełącz pełny ekran");
	gtk_widget_add_accelerator (m_tiface, "activate", accel,
                              GDK_u, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_widget_add_accelerator (m_fullscreen, "activate", accel,
                              GDK_F11, 0, GTK_ACCEL_VISIBLE);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(m_view), lookmenu);
	m_togtool=gtk_check_menu_item_new_with_label("Pokazuj toolbar");
	gtk_menu_shell_append(GTK_MENU_SHELL(lookmenu),m_togtool);
	gtk_menu_shell_append(GTK_MENU_SHELL(lookmenu), m_tiface);
	gtk_menu_shell_append(GTK_MENU_SHELL(lookmenu), m_fullscreen);

	gtk_menu_shell_append(GTK_MENU_SHELL(menubar), m_view);

	
	m_edit=gtk_menu_item_new_with_mnemonic("_Edycja");
	GtkWidget *m_iso2=gtk_menu_item_new_with_label("Sprowadź do LATIN2");
	GtkWidget *m_fake_1250=gtk_menu_item_new_with_label("Popraw fałszywy CP1250");
	GtkWidget *m_fake_iso2=gtk_menu_item_new_with_label("Popraw fałszywy ISO-2");
	m_dash=gtk_menu_item_new_with_label("Utwórz DASH");
	m_dict=gtk_menu_item_new_with_label("Utwórz DICT");
	m_dict2=gtk_menu_item_new_with_label("Utwórz bazowany DICT");
	m_adash=gtk_menu_item_new_with_label("Zastosuj DASH");
	GtkWidget *m_educa=gtk_menu_item_new_with_label("Wielkie litery");
	GtkWidget *m_edlca=gtk_menu_item_new_with_label("Małe litery");
	gtk_widget_add_accelerator (m_educa, "activate", accel,
                              GDK_1, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_widget_add_accelerator (m_edlca, "activate", accel,
                              GDK_2, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	m_unfor=gtk_menu_item_new_with_label("Usuń formatowanie");
	m_unjaco=gtk_menu_item_new_with_label("Korekta Jacosub");
	GtkWidget *m_deline=gtk_menu_item_new_with_label("Usuń linię");
	gtk_widget_add_accelerator (m_deline, "activate", accel,
                              GDK_y, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	GtkWidget *m_font=gtk_menu_item_new_with_label("Czcionka edytora");
	GtkWidget *m_prefs=gtk_menu_item_new_with_label("Zapisz ustawienia");
	m_edswap=gtk_menu_item_new_with_label("Plik <-> DIC");
	gtk_widget_add_accelerator (m_edswap, "activate", accel,
                              GDK_e, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	m_gotodash=gtk_menu_item_new_with_label("Plik -> DASH");
	gtk_widget_add_accelerator (m_gotodash, "activate", accel,
                              GDK_e, GDK_CONTROL_MASK | GDK_SHIFT_MASK, GTK_ACCEL_VISIBLE);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(m_edit), editmenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu), m_iso2);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu), m_fake_1250);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu), m_fake_iso2);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu), gtk_separator_menu_item_new());
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),m_dash);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),m_dict);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),m_dict2);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu), gtk_separator_menu_item_new());
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),m_adash);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),m_unfor);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),m_unjaco);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu), gtk_separator_menu_item_new());
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),m_deline);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu), gtk_separator_menu_item_new());
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),m_edswap);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),m_gotodash);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu), gtk_separator_menu_item_new());
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),m_educa);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),m_edlca);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu), gtk_separator_menu_item_new());
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),m_font);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu), gtk_separator_menu_item_new());
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),m_prefs);
	gtk_menu_shell_append(GTK_MENU_SHELL(menubar), m_edit);
	
	m_search=gtk_menu_item_new_with_mnemonic("_Szukaj");
	GtkWidget *m_find=gtk_menu_item_new_with_label("Znajdź");
	gtk_widget_add_accelerator (m_find, "activate", accel,
                              GDK_f, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	GtkWidget *m_find_n=gtk_menu_item_new_with_label("Znajdź następny");
	gtk_widget_add_accelerator (m_find_n, "activate", accel,
                              GDK_g, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	GtkWidget *m_find_p=gtk_menu_item_new_with_label("Znajdź poprzedni");
	gtk_widget_add_accelerator (m_find_p, "activate", accel,
                              GDK_g, GDK_CONTROL_MASK | GDK_SHIFT_MASK, GTK_ACCEL_VISIBLE);
	GtkWidget *m_find_r=gtk_menu_item_new_with_label("Znajdź i zamień");
	gtk_widget_add_accelerator (m_find_r, "activate", accel,
                              GDK_r, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	m_findcur=gtk_menu_item_new_with_label("Znajdź bieżący");
	gtk_widget_add_accelerator (m_findcur, "activate", accel,
                              GDK_b, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	m_entity=gtk_menu_item_new_with_label("Znajdź encję");
	gtk_widget_add_accelerator (m_entity, "activate", accel,
                              GDK_n, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	m_scanner=gtk_menu_item_new_with_label("Znajdź podejrzanego");
	gtk_widget_add_accelerator (m_scanner, "activate", accel,
                              GDK_n, GDK_CONTROL_MASK|GDK_SHIFT_MASK, GTK_ACCEL_VISIBLE);
	m_dialconn=gtk_menu_item_new_with_label("Znajdź połączenie kwestii");
	gtk_widget_add_accelerator (m_dialconn, "activate", accel,
                              GDK_d, GDK_CONTROL_MASK|GDK_SHIFT_MASK, GTK_ACCEL_VISIBLE);
	
	GtkWidget *m_gotoline=gtk_menu_item_new_with_label("Skocz do linii");
	gtk_widget_add_accelerator (m_gotoline, "activate", accel,
                              GDK_l, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	GtkWidget *m_scanread=gtk_menu_item_new_with_label("Wczytaj plik podejrzanych");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(m_search), searchmenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(searchmenu),m_find);
	gtk_menu_shell_append(GTK_MENU_SHELL(searchmenu),m_find_n);
	gtk_menu_shell_append(GTK_MENU_SHELL(searchmenu),m_find_p);
	gtk_menu_shell_append(GTK_MENU_SHELL(searchmenu),m_find_r);
	gtk_menu_shell_append(GTK_MENU_SHELL(searchmenu), gtk_separator_menu_item_new());
	gtk_menu_shell_append(GTK_MENU_SHELL(searchmenu),m_findcur);
	gtk_menu_shell_append(GTK_MENU_SHELL(searchmenu),m_entity);
	gtk_menu_shell_append(GTK_MENU_SHELL(searchmenu),m_scanner);
	gtk_menu_shell_append(GTK_MENU_SHELL(searchmenu),m_dialconn);
	gtk_menu_shell_append(GTK_MENU_SHELL(searchmenu), gtk_separator_menu_item_new());
	gtk_menu_shell_append(GTK_MENU_SHELL(searchmenu),m_gotoline);
	gtk_menu_shell_append(GTK_MENU_SHELL(searchmenu), gtk_separator_menu_item_new());
	gtk_menu_shell_append(GTK_MENU_SHELL(searchmenu),m_scanread);
	gtk_menu_shell_append(GTK_MENU_SHELL(menubar), m_search);

	GtkWidget *m_speech=gtk_menu_item_new_with_mnemonic("_Mowa");
	m_play=gtk_image_menu_item_new_from_stock(GTK_STOCK_MEDIA_PLAY, NULL);
	m_stop=gtk_image_menu_item_new_from_stock(GTK_STOCK_MEDIA_STOP, NULL);
	m_mp3=gtk_menu_item_new_with_label(
#ifdef USE_FAAC
					   "Utwórz MP3/AAC"
#else
					   "Utwórz MP3"
#endif
					   );
	m_amr=gtk_menu_item_new_with_label("Utwórz Nokia Audiobook");
	m_speaker_follow=gtk_check_menu_item_new_with_label("Podążaj za frazami");
	m_keep_audio=gtk_check_menu_item_new_with_label("Nie zamykaj audio");
	m_voice=gtk_menu_item_new_with_label("Ścieżka do pl1");
	m_stereo=gtk_menu_item_new_with_label("Ustawienia STEREO");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(m_speech), speechmenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(speechmenu), m_play);
	gtk_widget_add_accelerator (m_play, "activate", accel,
                              GDK_p, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_menu_shell_append(GTK_MENU_SHELL(speechmenu), m_stop);
	//gtk_widget_add_accelerator (m_stop, "activate", accel,
         //                     GDK_t, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_widget_add_accelerator (m_stop, "activate", accel,
                              GDK_Escape, 0, GTK_ACCEL_VISIBLE);

	gtk_menu_shell_append(GTK_MENU_SHELL(speechmenu), gtk_separator_menu_item_new());
	gtk_menu_shell_append(GTK_MENU_SHELL(speechmenu), m_mp3);
	gtk_menu_shell_append(GTK_MENU_SHELL(speechmenu), m_amr);
	gtk_menu_shell_append(GTK_MENU_SHELL(speechmenu), gtk_separator_menu_item_new());
	gtk_menu_shell_append(GTK_MENU_SHELL(speechmenu), m_speaker_follow);
	gtk_menu_shell_append(GTK_MENU_SHELL(speechmenu), m_keep_audio);
	gtk_menu_shell_append(GTK_MENU_SHELL(speechmenu), m_voice);
	gtk_menu_shell_append(GTK_MENU_SHELL(speechmenu), m_stereo);
#ifdef USE_RESAMPLER
	m_resample=gtk_menu_item_new_with_label("Ustawienia resamplera");
	gtk_menu_shell_append(GTK_MENU_SHELL(speechmenu), m_resample);
#endif


	gtk_menu_shell_append(GTK_MENU_SHELL(menubar), m_speech);

#ifdef USE_SUBMIXER
	m_video=gtk_menu_item_new_with_mnemonic("_Video");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(m_video), videomenu);
	m_video_open=gtk_menu_item_new_with_label("Otwórz film");
	m_video_caud=gtk_menu_item_new_with_label("Utwórz ścieżkę dźwiękową");
	m_video_cmov=gtk_menu_item_new_with_label("Utwórz film");
	m_video_settings=gtk_menu_item_new_with_label("Ustawienia lektora");
	
	gtk_menu_shell_append(GTK_MENU_SHELL(videomenu), m_video_open);
	gtk_menu_shell_append(GTK_MENU_SHELL(videomenu), m_video_caud);
	gtk_menu_shell_append(GTK_MENU_SHELL(videomenu), m_video_cmov);
	gtk_menu_shell_append(GTK_MENU_SHELL(videomenu), m_video_settings);

	gtk_menu_shell_append(GTK_MENU_SHELL(menubar),m_video);
#endif

	m_access=gtk_menu_item_new_with_mnemonic("_Dostępność");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(m_access), accessmenu);
	
	GtkWidget *m_ac_go_edi=gtk_menu_item_new_with_label("Przejdź do edytora");
	gtk_widget_add_accelerator (m_ac_go_edi, "activate", accel,
                              GDK_h, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_menu_shell_append(GTK_MENU_SHELL(accessmenu),m_ac_go_edi);
	
	GtkWidget *m_ac_go_synth=gtk_menu_item_new_with_label("Przejdź do parametrów syntezy");
	gtk_widget_add_accelerator (m_ac_go_synth, "activate", accel,
                              GDK_k, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_menu_shell_append(GTK_MENU_SHELL(accessmenu),m_ac_go_synth);
	
	GtkWidget *m_ac_go_conv=gtk_menu_item_new_with_label("Przejdź do parametrów konwersji");
	gtk_widget_add_accelerator (m_ac_go_conv, "activate", accel,
                              GDK_k, GDK_CONTROL_MASK | GDK_SHIFT_MASK, GTK_ACCEL_VISIBLE);
	gtk_menu_shell_append(GTK_MENU_SHELL(accessmenu),m_ac_go_conv);
	
	gtk_menu_shell_append(GTK_MENU_SHELL(accessmenu), gtk_separator_menu_item_new());
	m_ac_search=gtk_check_menu_item_new_with_label("Anonsuj wyniki wyszukiwania");
	gtk_menu_shell_append(GTK_MENU_SHELL(accessmenu),m_ac_search);
	m_ac_atk=gtk_check_menu_item_new_with_label("Korekta błedów ATK");
	gtk_menu_shell_append(GTK_MENU_SHELL(accessmenu),m_ac_atk);
	m_ac_nop=gtk_check_menu_item_new_with_label("Nie zawijaj wierszy");
	gtk_menu_shell_append(GTK_MENU_SHELL(accessmenu),m_ac_nop);
	
	gtk_menu_shell_append(GTK_MENU_SHELL(menubar),m_access);
	
	m_help=gtk_menu_item_new_with_mnemonic("Pomo_c");
	GtkWidget *m_about=gtk_image_menu_item_new_from_stock(GTK_STOCK_ABOUT,NULL);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(m_help), helpmenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(helpmenu), m_about);
	GtkWidget *m_hilfe=gtk_image_menu_item_new_from_stock(GTK_STOCK_HELP,NULL);
	//gtk_menu_shell_append(GTK_MENU_SHELL(helpmenu), m_hilfe);
	GtkWidget *m_homepage=gtk_menu_item_new_with_label("Strona Mileny");
	gtk_menu_shell_append(GTK_MENU_SHELL(helpmenu), m_homepage);
	gtk_menu_shell_append(GTK_MENU_SHELL(menubar), m_help);

	gtk_box_pack_start(GTK_BOX(frame), menubar, FALSE, FALSE, 3);

	g_signal_connect(G_OBJECT(m_open),"activate",G_CALLBACK(open_file),NULL);
	g_signal_connect(G_OBJECT(m_include),"activate",G_CALLBACK(open_file),(void *)1);
	g_signal_connect(G_OBJECT(m_new),"activate",G_CALLBACK(new_file),NULL);
	g_signal_connect(G_OBJECT(m_save),"activate",G_CALLBACK(save_file),NULL);
	g_signal_connect(G_OBJECT(m_saveas),"activate",G_CALLBACK(save_file),(void *)1);
	g_signal_connect(G_OBJECT(m_opendic),"activate",G_CALLBACK(open_dic),(void *)1);
	g_signal_connect(G_OBJECT(m_close),"activate",G_CALLBACK(close_current_editor),NULL);
	g_signal_connect(G_OBJECT(m_iso2),"activate",G_CALLBACK(convert_to_iso2),NULL);
	g_signal_connect(G_OBJECT(m_fake_iso2),"activate",G_CALLBACK(editor_fake_latin),(void *)"ISO-8859-2");
	g_signal_connect(G_OBJECT(m_fake_1250),"activate",G_CALLBACK(editor_fake_latin),(void *)"CP1250");
	g_signal_connect(G_OBJECT(m_dash),"activate",G_CALLBACK(make_dash),NULL);
	g_signal_connect(G_OBJECT(m_dict),"activate",G_CALLBACK(make_dict),NULL);
	g_signal_connect(G_OBJECT(m_dict2),"activate",G_CALLBACK(make_dict),(void *)1);
	g_signal_connect(G_OBJECT(m_adash),"activate",G_CALLBACK(apply_dash),NULL);
	g_signal_connect(G_OBJECT(m_unjaco),"activate",G_CALLBACK(correct_jacosub),NULL);
	g_signal_connect(G_OBJECT(m_unfor),"activate",G_CALLBACK(unformat_text),NULL);
	g_signal_connect(G_OBJECT(m_deline),"activate",G_CALLBACK(editor_delete_line),NULL);
	g_signal_connect(G_OBJECT(m_edswap),"activate",G_CALLBACK(swap_editors),NULL);
	g_signal_connect(G_OBJECT(m_gotodash),"activate",G_CALLBACK(swap_editors),(void *)1);
	g_signal_connect(G_OBJECT(m_font),"activate",G_CALLBACK(ChooseFont),NULL);
	g_signal_connect(G_OBJECT(m_educa),"activate",G_CALLBACK(editor_ulcase),(void *)1);
	g_signal_connect(G_OBJECT(m_edlca),"activate",G_CALLBACK(editor_ulcase),NULL);
	
	g_signal_connect(G_OBJECT(m_find),"activate",G_CALLBACK(StartSearch),NULL);
	g_signal_connect(G_OBJECT(m_find_n),"activate",G_CALLBACK(StartSearch),(void *)1);
	g_signal_connect(G_OBJECT(m_find_p),"activate",G_CALLBACK(StartSearch),(void *)2);
	g_signal_connect(G_OBJECT(m_find_r),"activate",G_CALLBACK(StartReplace),(void *)2);
	g_signal_connect(G_OBJECT(m_findcur),"activate",G_CALLBACK(find_current_word),NULL);
	g_signal_connect(G_OBJECT(m_entity),"activate",G_CALLBACK(find_entity),NULL);
	g_signal_connect(G_OBJECT(m_scanner),"activate",G_CALLBACK(find_scanner_error),NULL);
	g_signal_connect(G_OBJECT(m_dialconn),"activate",G_CALLBACK(find_dialog_connection),NULL);
	g_signal_connect(G_OBJECT(m_gotoline),"activate",G_CALLBACK(go_to_line),NULL);
	g_signal_connect(G_OBJECT(m_scanread),"activate",G_CALLBACK(readScanner),NULL);
	
	g_signal_connect(G_OBJECT(m_play),"activate",G_CALLBACK(speech_play),NULL);
	g_signal_connect(G_OBJECT(m_stop),"activate",G_CALLBACK(speech_stop),NULL);
	g_signal_connect(G_OBJECT(m_mp3),"activate",G_CALLBACK(book_create),NULL);
	g_signal_connect(G_OBJECT(m_amr),"activate",G_CALLBACK(book_create),(void *)1);
	g_signal_connect(G_OBJECT(m_tiface),"toggled",G_CALLBACK(toggle_interface_menu),(void *)1);
	g_signal_connect(G_OBJECT(m_fullscreen),"toggled",G_CALLBACK(toggle_fullscreen),NULL);
	g_signal_connect(G_OBJECT(m_togtool),"toggled",G_CALLBACK(toggle_toolbar),(void *)1);
	g_signal_connect(G_OBJECT(m_keep_audio),"toggled",G_CALLBACK(toggle_audiodevice),NULL);
	g_signal_connect(G_OBJECT(m_voice),"activate",G_CALLBACK(set_mbrola_voice),NULL);
	g_signal_connect(G_OBJECT(m_stereo),"activate",G_CALLBACK(stereo_dialog),NULL);
#ifdef USE_RESAMPLER
	g_signal_connect(G_OBJECT(m_resample),"activate",G_CALLBACK(resampler_dialog),NULL);
#endif
#ifdef USE_SUBMIXER
	g_signal_connect(G_OBJECT(m_video_open),"activate",G_CALLBACK(mplayer_read_movie),NULL);
	g_signal_connect(G_OBJECT(m_video_caud),"activate",G_CALLBACK(mplayer_make_audio),NULL);
	g_signal_connect(G_OBJECT(m_video_cmov),"activate",G_CALLBACK(mplayer_make_video),NULL);
	g_signal_connect(G_OBJECT(m_video_settings),"activate",G_CALLBACK(mplayer_settings),NULL);
#endif	
	g_signal_connect(G_OBJECT(m_quit),"activate",G_CALLBACK(close_main_window),NULL);
	g_signal_connect(G_OBJECT(m_prefs),"activate",G_CALLBACK(save_prefs),NULL);

	g_signal_connect(G_OBJECT(m_ac_go_edi),"activate",G_CALLBACK(acc_go_editor),NULL);
	g_signal_connect(G_OBJECT(m_ac_go_synth),"activate",G_CALLBACK(acc_go_synth),NULL);
	g_signal_connect(G_OBJECT(m_ac_go_conv),"activate",G_CALLBACK(acc_go_conv),NULL);
	g_signal_connect(G_OBJECT(m_ac_nop),"toggled",G_CALLBACK(acc_make_wrap),NULL);
	g_signal_connect(G_OBJECT(m_ac_atk),"toggled",G_CALLBACK(acc_make_wrap),NULL);

	g_signal_connect(G_OBJECT(m_about),"activate",G_CALLBACK(about_me),NULL);
	//g_signal_connect(G_OBJECT(m_hilfe),"activate",G_CALLBACK(help_me),NULL);
	g_signal_connect(G_OBJECT(m_homepage),"activate",G_CALLBACK(show_milena_home),NULL);

	gtk_widget_set_sensitive(m_stop,FALSE);
#ifdef USE_SUBMIXER
	gtk_widget_set_sensitive(m_video_caud,FALSE);
	gtk_widget_set_sensitive(m_video_cmov,FALSE);
#endif
	void speak_and_next()
	{
		if (is_speaking) return;
		start_reading(1);
	}
	GClosure *gcl=g_cclosure_new(speak_and_next,NULL,NULL);
	gtk_accel_group_connect(accel,GDK_p,GDK_CONTROL_MASK | GDK_SHIFT_MASK,0,gcl);
	

//toolbar
	toolbar = gtk_toolbar_new();
	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_BOTH);
	gtk_container_set_border_width(GTK_CONTAINER(toolbar), 1);
	
	t_new= (gpointer)gtk_tool_button_new_from_stock(GTK_STOCK_NEW);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), (gpointer)t_new, -1);

	t_open = (gpointer)gtk_tool_button_new_from_stock(GTK_STOCK_OPEN);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), (gpointer)t_open, -1);
	
	t_save = (gpointer)gtk_tool_button_new_from_stock(GTK_STOCK_SAVE);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), (gpointer)t_save, -1);

	t_saveas = (gpointer)gtk_tool_button_new_from_stock(GTK_STOCK_SAVE_AS);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), (gpointer)t_saveas, -1);

	t_close = (gpointer)gtk_tool_button_new_from_stock(GTK_STOCK_CLOSE);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), (gpointer)t_close, -1);

	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(), -1); 
	
	t_play = (gpointer)gtk_tool_button_new_from_stock(GTK_STOCK_MEDIA_PLAY);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), (gpointer)t_play, -1);
	t_stop = (gpointer)gtk_tool_button_new_from_stock(GTK_STOCK_MEDIA_STOP);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), (gpointer)t_stop, -1);

	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(), -1); 
	t_prop = (gpointer)gtk_toggle_tool_button_new_from_stock(GTK_STOCK_PROPERTIES);
	gtk_toggle_tool_button_set_active((gpointer)t_prop,TRUE);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), (gpointer)t_prop, -1);
	
	
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(), -1); 

	GtkWidget *t_exit = (gpointer)gtk_tool_button_new_from_stock(GTK_STOCK_QUIT);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), (gpointer)t_exit, -1);
	gtk_box_pack_start(GTK_BOX(frame), toolbar, FALSE, FALSE, 1);

	gtk_widget_set_sensitive(t_stop,FALSE);
	
	g_signal_connect(G_OBJECT(t_open),"clicked",G_CALLBACK(open_file),NULL);
	g_signal_connect(G_OBJECT(t_new),"clicked",G_CALLBACK(new_file),NULL);
	g_signal_connect(G_OBJECT(t_save),"clicked",G_CALLBACK(save_file),NULL);
	g_signal_connect(G_OBJECT(t_saveas),"clicked",G_CALLBACK(save_file),(void *)1);
	g_signal_connect(G_OBJECT(t_close),"clicked",G_CALLBACK(close_current_editor),NULL);
	g_signal_connect(G_OBJECT(t_exit),"clicked",G_CALLBACK(close_main_window),NULL);
	g_signal_connect(G_OBJECT(t_play),"clicked",G_CALLBACK(speech_play),NULL);
	g_signal_connect(G_OBJECT(t_stop),"clicked",G_CALLBACK(speech_stop),NULL);
	g_signal_connect(G_OBJECT(t_prop),"toggled",G_CALLBACK(toggle_iface),NULL);

//frame	
	uframe=gtk_hbox_new(0,2);
	gtk_box_pack_start((gpointer)frame,uframe,FALSE,FALSE,0);
	leftframe=gtk_frame_new("Parametry syntezy");
	gtk_box_pack_start(GTK_BOX(uframe),leftframe,FALSE,FALSE,0);
	riteframe=gtk_frame_new("Konwerter MP3/AMR");
	gtk_box_pack_start(GTK_BOX(uframe),riteframe,FALSE,FALSE,0);
	
	lbox=gtk_vbox_new(0,2);
	gtk_container_add(GTK_CONTAINER(leftframe),lbox);
	
	//spins=gtk_table_new(2,2,0);
	spins=gtk_hbox_new(0,2);
	gtk_box_pack_start(GTK_BOX(lbox),spins,FALSE,FALSE,0);
	
	label_tempo=gtk_label_new_with_mnemonic("_Tempo");
	gtk_box_pack_start(GTK_BOX(spins),label_tempo,FALSE,FALSE,0);
	//gtk_table_attach_defaults((gpointer)spins,label_tempo,0,1,0,1);
	spin_tempo=gtk_spin_button_new_with_range(0.5,1.5,0.01);
	gtk_spin_button_set_value((gpointer)spin_tempo,0.9);
	gtk_box_pack_start(GTK_BOX(spins),spin_tempo,FALSE,FALSE,0);
	//gtk_table_attach_defaults((gpointer)spins,spin_tempo,1,2,0,1);
	connect_label(label_tempo,spin_tempo);
	
	label_pitch=gtk_label_new("Wysokość");
	gtk_box_pack_start(GTK_BOX(spins),label_pitch,FALSE,FALSE,0);
	//gtk_table_attach_defaults((gpointer)spins,label_pitch,0,1,1,2);
	spin_pitch=gtk_spin_button_new_with_range(0.5,1.5,0.01);
	gtk_spin_button_set_value((gpointer)spin_pitch,0.9);
	gtk_box_pack_start(GTK_BOX(spins),spin_pitch,FALSE,FALSE,0);
	//gtk_table_attach_defaults((gpointer)spins,spin_pitch,1,2,1,2);
	connect_label(label_pitch,spin_pitch);

	cb_pro=gtk_check_button_new_with_label("Szybsze czytanie");
	gtk_box_pack_start(GTK_BOX(spins),cb_pro,FALSE,FALSE,0);

	cb_lector=gtk_check_button_new_with_label("Lektor");
	gtk_box_pack_start(GTK_BOX(spins),cb_lector,FALSE,FALSE,0);

	cb_ellip=gtk_check_button_new_with_label("Wielokropek");
	gtk_widget_set_tooltip_text((gpointer)cb_ellip,"Wielokropek urywa zdanie");
	gtk_box_pack_start(GTK_BOX(spins),cb_ellip,FALSE,FALSE,0);

	cb_equ=gtk_check_button_new_with_label("EQS");
	gtk_widget_set_tooltip_text((gpointer)cb_equ,"Włączenie equalizera długości sylab");
	gtk_box_pack_start(GTK_BOX(spins),cb_equ,FALSE,FALSE,0);
	
	langframe=gtk_frame_new("Języki");
	gtk_box_pack_start(GTK_BOX(lbox),langframe,FALSE,FALSE,0);
	langtb=gtk_table_new(4,2,0);
	gtk_container_add(GTK_CONTAINER(langframe),langtb);
	for (i=0;i<NUM_LANGS;i++) {
		lang_cb[i]=gtk_check_button_new_with_label(lang_names[i]);
		gtk_table_attach_defaults((gpointer)langtb,lang_cb[i],
			i%4, i%4+1, i/4, i/4+1);
	}

	tfr1=gtk_hbox_new(0,2);
	gtk_box_pack_start(GTK_BOX(lbox),tfr1,FALSE,FALSE,0);
	
	cb_hours=gtk_check_button_new_with_label("Odczytuj godziny");
	gtk_box_pack_start(GTK_BOX(tfr1),cb_hours,FALSE,FALSE,0);
	gtk_toggle_button_set_active((gpointer)cb_hours,TRUE);
		
//	cb_pro=gtk_check_button_new_with_label("Szybsze czytanie");
//	gtk_box_pack_start(GTK_BOX(tfr1),cb_pro,FALSE,FALSE,0);
	
	cb_ign=gtk_check_button_new_with_label("Ignoruj akcenty i separatory");
	gtk_box_pack_start(GTK_BOX(tfr1),cb_ign,FALSE,FALSE,0);

	cb_spell=gtk_check_button_new_with_label("Czytaj większość znaków");
	gtk_box_pack_start(GTK_BOX(tfr1),cb_spell,FALSE,FALSE,0);

	
	tfr2=gtk_hbox_new(0,2);
	gtk_box_pack_start(GTK_BOX(lbox),tfr2,FALSE,FALSE,0);
	
	cb_dic=gtk_check_button_new_with_label("Plik słownika");
	gtk_box_pack_start(GTK_BOX(tfr2),cb_dic,FALSE,FALSE,0);
	g_signal_connect(G_OBJECT(cb_dic),"toggled",G_CALLBACK(set_dicfile_active),NULL);
	dic_filechooser=gtk_file_chooser_button_new("Wybierz plik słownika",
		GTK_FILE_CHOOSER_ACTION_OPEN);
	gtk_file_chooser_set_local_only((gpointer)dic_filechooser,TRUE);
	gtk_file_chooser_button_set_width_chars((gpointer)dic_filechooser,50);
	gtk_box_pack_start(GTK_BOX(tfr2),dic_filechooser,FALSE,FALSE,0);
	gtk_widget_set_sensitive(dic_filechooser,FALSE);

	atk_object_set_name(gtk_widget_get_accessible(dic_filechooser),"Wybierz plik słownika");

	GtkWidget *themframe=gtk_frame_new("Tematy");
	gtk_box_pack_start(GTK_BOX(lbox),themframe,FALSE,FALSE,0);
	tfr3=gtk_hbox_new(0,2);
	gtk_container_add(GTK_CONTAINER(themframe),tfr3);
	
//	gtk_box_pack_start(GTK_BOX(lbox),tfr3,FALSE,FALSE,0);
	
	DIR *dy;
	if (dy=opendir(milena_GetDataPath())) {
		struct dirent *de;
		struct miltheme *mt,**mmt;
		while (de=readdir(dy)) {
			char buf[256],bfn[256];FILE *f;
			int le=strlen(de->d_name);
			if (le<14) continue;
			if (strcmp("_theme.dat",de->d_name+le-10)) continue;
			if (strncmp("pl_",de->d_name,3)) continue;
			strncpy(buf,de->d_name+3,le-13);
			buf[le-13]=0;
			strcpy(bfn,milena_GetDataPath());
			strcat(bfn,"/");
			strcat(bfn,de->d_name);
			mt=malloc(sizeof(*mt));
			mt->fname=strdup(de->d_name);
			mt->tname=strdup(buf);
			mt->cb=gtk_check_button_new_with_label(buf);
			mt->title=NULL;
			f=fopen(bfn,"r");
			if (f) {
				if (fgets(bfn,256,f)) {
					char *c=bfn;
					while (*c && isspace(*c)) c++;
					if (*c && !strncmp(c,"//",2)) {
						c=trim(c+2);
						if (*c) mt->title=to_utf8(c,strlen(c),"ISO-8859-2",0);
					}
				}
				fclose(f);
			}
			if (mt->title) gtk_widget_set_tooltip_text((gpointer)mt->cb,mt->title);
			for (mmt=&themes;*mmt;mmt=&(*mmt)->next) {
				if (strcmp((*mmt)->fname,mt->fname)>0) break;
			}
			
			mt->next=*mmt;
			*mmt=mt;
		}
		closedir(dy);
		for (mt=themes;mt;mt=mt->next) {
			gtk_box_pack_start(GTK_BOX(tfr3),mt->cb,FALSE,FALSE,0);
		}
	}
	
//	cb_lector=gtk_check_button_new_with_label("Tryb lektorski");
//	gtk_box_pack_start(GTK_BOX(tfr3),cb_lector,FALSE,FALSE,0);
//	cb_spell=gtk_check_button_new_with_label("Czytaj większość znaków");
//	gtk_box_pack_start(GTK_BOX(tfr3),cb_spell,FALSE,FALSE,0);
	g_signal_connect(G_OBJECT(cb_lector),"toggled",G_CALLBACK(set_lector_active),NULL);
	g_signal_connect(G_OBJECT(cb_spell),"toggled",G_CALLBACK(set_lector_active),NULL);
	
	/* prawa strona */	
	mp3tb=gtk_table_new(3,7,0);
	gtk_container_add(GTK_CONTAINER(riteframe),mp3tb);

	cb_kontrast=gtk_check_button_new_with_label("Kontrast");
	gtk_table_attach_defaults((gpointer)mp3tb,cb_kontrast,0,1,0,1);
	gtk_toggle_button_set_active((gpointer)cb_kontrast,TRUE);

	spin_kontrast=gtk_spin_button_new_with_range(0.3,1.0,0.05);
	atk_object_set_name(gtk_widget_get_accessible(spin_kontrast),"Kontrast");
	atk_object_set_description(gtk_widget_get_accessible(spin_kontrast),"Ustaw poziom efektu kontrast");
	gtk_spin_button_set_value((gpointer)spin_kontrast,0.9);
	gtk_table_attach_defaults((gpointer)mp3tb,spin_kontrast,1,2,0,1);
	g_signal_connect(G_OBJECT(cb_kontrast),"toggled",G_CALLBACK(set_kontrast_active),NULL);
	
	label=gtk_label_new(
#ifdef USE_FAAC
			    "Format MP3/AAC"
#else
			    "Format MP3"
#endif
			    );
	gtk_table_attach_defaults((gpointer)mp3tb,label,0,1,1,2);
	combo_mp3=combo(0,"MP3 32 kbit","MP3 48 kbit","MP3 64 kbit",
#ifdef USE_FAAC
			"AAC",
#endif
			NULL);
	gtk_table_attach_defaults((gpointer)mp3tb,combo_mp3,1,2,1,2);
	connect_label(label,combo_mp3);
	cb_stereo=gtk_check_button_new_with_label("Stereo");
	gtk_table_attach_defaults((gpointer)mp3tb,cb_stereo,2,3,1,2);
	
	label=gtk_label_new("Tryb AMR");
	gtk_table_attach_defaults((gpointer)mp3tb,label,0,1,2,3);
	combo_amr=combo(1,"2","3","4","5","6",NULL);
	gtk_table_attach_defaults((gpointer)mp3tb,combo_amr,1,2,2,3);
	connect_label(label,combo_amr);
	cb_dtx=gtk_check_button_new_with_label("DTX");
	gtk_table_attach_defaults((gpointer)mp3tb,cb_dtx,2,3,2,3);
	
	label=gtk_label_new("Podział na rozdziały");
	gtk_table_attach_defaults((gpointer)mp3tb,label,0,1,3,4);
	combo_split=combo(0,"Wyłączony","Według znaku",NULL);
	gtk_table_attach_defaults((gpointer)mp3tb,combo_split,1,2,3,4);
	connect_label(label,combo_split);
	char_split=gtk_entry_new();
	gtk_entry_set_max_length((gpointer)char_split,1);
	gtk_entry_set_text((gpointer)char_split,"#");
	gtk_widget_set_sensitive(char_split,FALSE);
	gtk_table_attach_defaults((gpointer)mp3tb,char_split,2,3,3,4);
	gtk_entry_set_width_chars((gpointer)char_split,2);
	g_signal_connect(G_OBJECT(combo_split),"changed",G_CALLBACK(set_autonum_active),NULL);
	atk_object_set_name(gtk_widget_get_accessible(char_split),"Znak");
	atk_object_set_description(gtk_widget_get_accessible(char_split),"Znak podziału na rozdziały");

	label=gtk_label_new("Autonumeracja");
	gtk_table_attach_defaults((gpointer)mp3tb,label,0,1,4,5);
	combo_autonum=combo(0,"Brak","Prosta","Z linią tytułową",NULL);
	gtk_table_attach_defaults((gpointer)mp3tb,combo_autonum,1,3,4,5);
	connect_label(label,combo_autonum);
	g_signal_connect(G_OBJECT(combo_autonum),"changed",G_CALLBACK(set_autonum_active),NULL);
	gtk_widget_set_sensitive(combo_autonum,FALSE);
	
	label=gtk_label_new("Prolog");
	gtk_table_attach_defaults((gpointer)mp3tb,label,0,1,5,6);
	combo_prolog=combo(0,"Brak","PROLOG","Z linią tytułową",NULL);
	gtk_table_attach_defaults((gpointer)mp3tb,combo_prolog,1,3,5,6);
	connect_label(label,combo_prolog);
	gtk_widget_set_sensitive(combo_prolog,FALSE);
	
	label=gtk_label_new("Epilog");
	gtk_table_attach_defaults((gpointer)mp3tb,label,0,1,6,7);
	combo_epilog=combo(0,"Brak","Ostatni rozdział","EPILOG","Z linią tytułową",NULL);
	gtk_table_attach_defaults((gpointer)mp3tb,combo_epilog,1,3,6,7);
	connect_label(label,combo_epilog);
	gtk_widget_set_sensitive(combo_epilog,FALSE);
	
	/* tresc */
	
	notek=gtk_notebook_new();
	gtk_notebook_set_scrollable((gpointer)notek,TRUE);
	gtk_box_pack_start(GTK_BOX(frame),(gpointer)notek,TRUE,TRUE,0);
	g_signal_connect(G_OBJECT(notek),"switch-page",
		G_CALLBACK(menu_active_notek),NULL);
	g_signal_connect(G_OBJECT(notek),"page-added",
		G_CALLBACK(menu_active_notek),NULL);
	g_signal_connect(G_OBJECT(notek),"page-removed",
		G_CALLBACK(menu_active_notek),NULL);
	g_signal_connect(G_OBJECT(notek),"change-current-page",
		G_CALLBACK(menu_active_notek),NULL);
	load_prefs();
	CreateEditor(0,NULL,"nowy");

	/* statusbar */
	
	statusbar=(gpointer)gtk_statusbar_new();
	gtk_box_pack_start(GTK_BOX(frame),statusbar,FALSE,FALSE,0);
	
	statusbar_line=gtk_statusbar_new();
	gtk_widget_set_size_request(statusbar_line,120,-1);
	gtk_box_pack_end(GTK_BOX(statusbar),statusbar_line,FALSE,FALSE,0);
	
	statusbar_ove=gtk_statusbar_new();
	gtk_widget_set_size_request(statusbar_ove,80,-1);
	gtk_box_pack_end(GTK_BOX(statusbar),statusbar_ove,FALSE,FALSE,0);
	
	gtk_statusbar_set_has_resize_grip((gpointer)statusbar,FALSE);
	gtk_statusbar_set_has_resize_grip((gpointer)statusbar_ove,FALSE);
	gtk_statusbar_set_has_resize_grip((gpointer)statusbar_line,FALSE);
	
}

int menu_active_notek_cb(void *data)
{
	struct MyGtkEditor *editor;
	int is_main=0,is_dash=0,is_dic=0,has_dash=0,has_dic=0;
	editor=gimma_current_editor();
	if (editor) {
		if (editor->editor_type == MILEDIT_MODE_NORMAL) {
			is_main=1;
			if (editor->dic_editor) has_dic=1;
			if (editor->dash_editor) has_dash=1;
		}
		else if (editor->editor_type == MILEDIT_MODE_DASH) is_dash=1;
		else if (editor->editor_type == MILEDIT_MODE_DIC) is_dic=1;
	}
	gtk_widget_set_sensitive(m_include,is_main);
	gtk_widget_set_sensitive(m_dash,is_main);
	gtk_widget_set_sensitive(m_dict,is_main);
	gtk_widget_set_sensitive(m_dict2,is_main);
	gtk_widget_set_sensitive(m_adash,is_main && has_dash);
	gtk_widget_set_sensitive(m_unfor,is_main);
	gtk_widget_set_sensitive(m_unjaco,is_main);
	gtk_widget_set_sensitive(m_edswap,(is_main && (has_dic || has_dash)) || is_dic || is_dash);
	gtk_widget_set_sensitive(m_gotodash,(is_main && has_dash) || is_dash);
	gtk_widget_set_sensitive(m_findcur,is_dic || is_dash);
	gtk_widget_set_sensitive(m_entity,is_main);
	gtk_widget_set_sensitive(m_scanner,is_main);
	gtk_widget_set_sensitive(m_amr,is_main);
	gtk_widget_set_sensitive(m_mp3,is_main);
	gtk_widget_set_sensitive(m_play,(is_main | is_dic) && !is_speaking);
	gtk_widget_set_sensitive(t_play,(is_main | is_dic) && !is_speaking);
	gtk_widget_set_sensitive(m_edit,is_main | is_dash | is_dic);
	gtk_widget_set_sensitive(m_search,is_main | is_dash | is_dic);
	gtk_widget_set_sensitive(m_close,is_main | is_dash | is_dic);
	gtk_widget_set_sensitive(t_close,is_main | is_dash | is_dic);
	show_position_on_sb(editor);
#ifdef USE_SUBMIXER
	enable_video_menues(is_main,editor);
#endif
	return FALSE;
}
void menu_active_notek(GtkWidget *widget,void *dummy)
{
	g_timeout_add(200,menu_active_notek_cb,NULL);
}



// get_encoding dialog

char *ask_encoding(void)
{
	GtkWidget *dialog=gtk_dialog_new_with_buttons(
		"Wybierz kodowanie",
		(gpointer)main_window,
		GTK_DIALOG_MODAL,
		GTK_STOCK_OK,GTK_RESPONSE_ACCEPT,
		GTK_STOCK_CANCEL,GTK_RESPONSE_REJECT,
		NULL);
	GtkWidget *box=gtk_dialog_get_content_area((gpointer)dialog);
	GtkWidget *codings=combo(1,
		"ISO-8859-1","ISO-8859-2",
		"ISO-8859-13","ISO-8859-16",
		"CP1250","CP1252","CP1257","IBM852",NULL);
	gtk_box_pack_start(GTK_BOX(box),codings,FALSE,FALSE,0);
	gtk_widget_show_all(box);
	int n=gtk_dialog_run((gpointer)dialog);
	char *c=NULL;
	if (n==GTK_RESPONSE_ACCEPT) {
		c=gtk_combo_box_get_active_text((gpointer)codings);
	}
	gtk_widget_destroy(dialog);
	return c;
}

void show_main_window(void)
{
	gtk_window_present(GTK_WINDOW(main_window));
}

void start_busy_cursor(void)
{
	if (!watch_cursor) watch_cursor=gdk_cursor_new(GDK_WATCH);
	GdkDisplay *display=gtk_widget_get_display((gpointer)main_window);
	gdk_window_set_cursor(GTK_WIDGET(main_window)->window,watch_cursor);
	gdk_display_sync(display);
	yield();
}
void end_busy_cursor(void)
{
	GdkDisplay *display=gtk_widget_get_display((gpointer)main_window);
	gdk_window_set_cursor(GTK_WIDGET(main_window)->window,NULL);
	gdk_display_flush(display);
}


gchar ** initial_fnames=NULL;
gint auto_load_dic;


void acc_make_wrap(void)
{
	struct MyGtkEditor *ed;
	for (ed=editors;ed;ed=ed->next) {
		if (!ed->editor_type) {
			gtk_text_view_set_wrap_mode(ed->view,word_wrap);
		}
	}
}

#include "milena_abc.xpm"
gint old_pdf_behav;
int main(int argc,char *argv[])
{

	int fno;
	gint hide_converter=0;
	gchar *rtf_mode=NULL;
	GOptionContext *optcon=g_option_context_new("[PLIK...]");
	GOptionEntry optries[]={
		{"auto-load-dic",'a',0,G_OPTION_ARG_NONE,&auto_load_dic,"Automatycznie ładuj pliki DIC",NULL},
		{"show-converter",'c',0,G_OPTION_ARG_NONE,&hide_converter,"Pokaż panel ustawień po załadowaniu plików",NULL},
		{"plain-pdf",'p',0,G_OPTION_ARG_NONE,&old_pdf_behav,"Użyj pdftotext zamiast pdftoxml",NULL},
		{"foot-rtf",'f',0,G_OPTION_ARG_STRING,&rtf_mode,"Tryb przypisów RTF (discard, normal, end, inline)",NULL}, 
		{"debug-level",'d',0,G_OPTION_ARG_INT,&debug_level,"Poziom debugowania",NULL},
		{G_OPTION_REMAINING,'x',0,G_OPTION_ARG_FILENAME_ARRAY,&initial_fnames,"Pliki",NULL},
		{NULL}
	};
	gtk_init(&argc,&argv);
	g_option_context_add_main_entries(optcon,optries,NULL);
	g_option_context_add_group (optcon, gtk_get_option_group (TRUE));
	g_option_context_set_summary(optcon,"Milena AudioBook Creator v"GMILENA_VERSION);
	GError *error=NULL;
	if (!g_option_context_parse(optcon,&argc,&argv,&error)) {
		g_print("Options parser failed: %s\n",error->message);
		exit(1);
	}
	if (rtf_mode) {
		setFootNote(rtf_mode);
	}
	if (debug_level <0) debug_level=0;
	main_window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	setlocale(LC_NUMERIC,"C");
	make_main_frame();
	if (initial_fnames) {
		for(fno=0;initial_fnames[fno];fno++) {
			open_file(NULL,initial_fnames[fno]);
		}
		gtk_notebook_set_current_page((gpointer)notek,0);
	}
	GdkPixbuf* pix=gdk_pixbuf_new_from_xpm_data((void *)milena_icon);
	GtkStatusIcon *icon=gtk_status_icon_new_from_pixbuf(pix);
#if GTK_CHECK_VERSION(2,18,0)
	gtk_status_icon_set_title(icon,"Milena ABC");
#endif
	gtk_status_icon_set_tooltip_text(icon,"Milena - Kreator audioksiążek");
	gtk_window_set_icon(GTK_WINDOW(main_window),pix);
	gtk_widget_show_all(main_window);
	toggle_toolbar((gpointer)m_togtool);
	gtk_window_resize(main_window,800,560);
	if (initial_fnames && !hide_converter) {
		gtk_toggle_tool_button_set_active((gpointer)t_prop,FALSE);
		gtk_widget_hide(uframe);
		
	}
	toggle_fullscreen((gpointer)m_fullscreen);
	gtk_check_menu_item_set_active((gpointer)m_tiface,gtk_toggle_tool_button_get_active((gpointer)t_prop));
	g_signal_connect_swapped(G_OBJECT(main_window), "delete-event",
	      G_CALLBACK(close_main_window), NULL);
	g_signal_connect(G_OBJECT(icon), "activate",
	      G_CALLBACK(show_main_window), NULL);
	gtk_main();
	return 0;
}

void save_prefs(GtkWidget *widget,void *dummy)
{
	char path[PATH_MAX+1];
	char line[1024];
	sprintf(path,"%s/.milena_abcrc",getenv("HOME"));
	FILE *f=fopen(path,"w");
	if (!f) {
		g_perror(path);
		return;
	}
	if (font_name[0]) {
		fprintf(f,"font=%s\n",font_name);
	}
	fprintf(f,"tempo=%.3f\n",gtk_spin_button_get_value((gpointer)spin_tempo));
	fprintf(f,"pitch=%.3f\n",gtk_spin_button_get_value((gpointer)spin_pitch));
	fprintf(f,"hours=%d\n",gtk_toggle_button_get_active((gpointer)cb_hours)?1:0);
	fprintf(f,"pro=%d\n",gtk_toggle_button_get_active((gpointer)cb_pro)?1:0);
	fprintf(f,"lector=%d\n",gtk_toggle_button_get_active((gpointer)cb_lector)?1:0);
	fprintf(f,"contrast=%d\n",gtk_toggle_button_get_active((gpointer)cb_kontrast)?1:0);
	fprintf(f,"vcontrast=%.3f\n",gtk_spin_button_get_value((gpointer)spin_kontrast));
	fprintf(f,"mp3=%d\n",gtk_combo_box_get_active((gpointer)combo_mp3));
	fprintf(f,"amr=%d\n",gtk_combo_box_get_active((gpointer)combo_amr));
	fprintf(f,"dtx=%d\n",gtk_toggle_button_get_active((gpointer)cb_dtx));
	fprintf(f,"stereo=%d\n",gtk_toggle_button_get_active((gpointer)cb_stereo));
	fprintf(f,"ellipsis=%d\n",gtk_toggle_button_get_active((gpointer)cb_ellip));
	fprintf(f,"eql=%d\n",gtk_toggle_button_get_active((gpointer)cb_equ));
	fprintf(f,"a11y_search=%d\n",gtk_check_menu_item_get_active((gpointer)m_ac_search));
	fprintf(f,"a11y_atk=%d\n",gtk_check_menu_item_get_active((gpointer)m_ac_atk));
	fprintf(f,"a11y_wrap=%d\n",gtk_check_menu_item_get_active((gpointer)m_ac_nop));
	fprintf(f,"speaker_follow=%d\n",gtk_check_menu_item_get_active((gpointer)m_speaker_follow));
	fprintf(f,"keep_audio=%d\n",gtk_check_menu_item_get_active((gpointer)m_keep_audio));
	fprintf(f,"toolbar_on=%d\n",gtk_check_menu_item_get_active((gpointer)m_togtool));
	fprintf(f,"fullscreen_on=%d\n",gtk_check_menu_item_get_active((gpointer)m_fullscreen));
	fprintf(f,"stereo_rev=%.3f\n",stereo_rev);
	fprintf(f,"stereo_hfd=%.3f\n",stereo_hfd);
	fprintf(f,"stereo_room=%.3f\n",stereo_room);
	fprintf(f,"stereo_depth=%.3f\n",stereo_depth);
	fprintf(f,"mpart_on=%d\n",multipartial_on);
	fprintf(f,"id3mp3=%d\n",id3_tag_mode);
	fprintf(f,"mpart_val=%.3f\n",multipartial_value);
	fprintf(f,"mpart_mrg=%.3f\n",multipartial_margin);
#ifdef USE_RESAMPLER
	fprintf(f,"resam_use=%d\n",use_resampler);
	fprintf(f,"resam_mpx=%.3f\n",computed_resample_factor);
	fprintf(f,"resam_vtl=%.3f\n",vocal_track_length);
#endif
#ifdef USE_SUBMIXER
	fprintf(f,"smix_tempo=%.3f\n",smix_speed);
	fprintf(f,"smix_maxtempo=%.3f\n",smix_maxspeed);
	fprintf(f,"smix_factor=%.3f\n",smix_volume_factor);
	fprintf(f,"smix_maxvol=%.3f\n",smix_max_origin_vol);
	fprintf(f,"smix_drop=%d\n",smix_droptime);
	fprintf(f,"smix_rise=%d\n",smix_risetime);
	fprintf(f,"smix_mindist=%d\n",smix_mindist);
	fprintf(f,"smix_margin=%d\n",smix_audio_margin);
#endif
	if (mbrola_voice[0]) fprintf(f,"voice=%s\n",mbrola_voice);
	if (scanner_path[0]) fprintf(f,"scanpath=%s\n",scanner_path);
	fclose(f);
	Info("Informacja","Ustawienia zostały zapisane");
}


void load_prefs(void)
{
	char path[PATH_MAX+1];
	char line[1024];
	sprintf(path,"%s/.milena_abcrc",getenv("HOME"));
	FILE *f=fopen(path,"r");
	if (!f) {
		sprintf(path,"%s/.milena_bookrc",getenv("HOME"));
		FILE *f=fopen(path,"r");
		if (!f) {
			return;
		}
		while(fgets(line,1024,f)) {
			char *val=strchr(line,'=');
			if (!val) continue;
			*val++=0;
			val=trim(val);
			char *cmd=trim(line);
			if (!strcmp(cmd,"tempo")) {
				gtk_spin_button_set_value((gpointer)spin_tempo,strtod(val,NULL));
			}
			else if (!strcmp(cmd,"pitch")) {
				gtk_spin_button_set_value((gpointer)spin_pitch,strtod(val,NULL));
			}
			else if (!strcmp(cmd,"voice")) {
				strcpy(mbrola_voice,val);
			}
		}
		fclose(f);
		return;
	}
	while(fgets(line,1024,f)) {
		char *val=strchr(line,'=');
		if (!val) continue;
		*val++=0;
		val=trim(val);
		char *cmd=trim(line);
		if (!strcmp(cmd,"font")) {
			strcpy(font_name,val);
			ApplyFont(font_name);
			continue;
		}
		else if (!strcmp(cmd,"tempo")) {
				gtk_spin_button_set_value((gpointer)spin_tempo,strtod(val,NULL));
		}
		else if (!strcmp(cmd,"pitch")) {
			gtk_spin_button_set_value((gpointer)spin_pitch,strtod(val,NULL));
		}
		else if (!strcmp(cmd,"voice")) {
			strcpy(mbrola_voice,val);
		}
		else if (!strcmp(cmd,"scanpath")) {
			strcpy(scanner_path,val);
		}
		else if (!strcmp(cmd,"hours")) {
			gtk_toggle_button_set_active((gpointer)cb_hours,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"pro")) {
			gtk_toggle_button_set_active((gpointer)cb_pro,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"lector")) {
			gtk_toggle_button_set_active((gpointer)cb_lector,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"dtx")) {
			gtk_toggle_button_set_active((gpointer)cb_dtx,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"stereo")) {
			gtk_toggle_button_set_active((gpointer)cb_stereo,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"contrast")) {
			gtk_toggle_button_set_active((gpointer)cb_kontrast,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"vcontrast")) {
			gtk_spin_button_set_value((gpointer)spin_kontrast,strtod(val,NULL));
		}
		else if (!strcmp(cmd,"mp3")) {
			gtk_combo_box_set_active((gpointer)combo_mp3,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"amr")) {
			gtk_combo_box_set_active((gpointer)combo_amr,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"ellipsis")) {
			gtk_toggle_button_set_active((gpointer)cb_ellip,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"eql")) {
			gtk_toggle_button_set_active((gpointer)cb_equ,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"toolbar_on")) {
			gtk_check_menu_item_set_active((gpointer)m_togtool,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"fullscreen_on")) {
			gtk_check_menu_item_set_active((gpointer)m_fullscreen,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"a11y_search")) {
			gtk_check_menu_item_set_active((gpointer)m_ac_search,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"a11y_atk")) {
			gtk_check_menu_item_set_active((gpointer)m_ac_atk,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"a11y_wrap")) {
			gtk_check_menu_item_set_active((gpointer)m_ac_nop,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"speaker_follow")) {
			gtk_check_menu_item_set_active((gpointer)m_speaker_follow,strtol(val,NULL,10));
		}
		else if (!strcmp(cmd,"keep_audio")) {
			gtk_check_menu_item_set_active((gpointer)m_keep_audio,strtol(val,NULL,10));
			dont_close_audiodevice=strtol(val,NULL,10);
		}
		else if (!strcmp(cmd,"stereo_rev")) {
			stereo_rev=strtod(val,NULL);
		}
		else if (!strcmp(cmd,"stereo_hfd")) {
			stereo_hfd=strtod(val,NULL);
		}
		else if (!strcmp(cmd,"stereo_depth")) {
			stereo_depth=strtod(val,NULL);
		}
		else if (!strcmp(cmd,"stereo_room")) {
			stereo_room=strtod(val,NULL);
		}
		else if (!strcmp(cmd,"mpart_val")) {
			multipartial_value=strtod(val,NULL);
		}
		else if (!strcmp(cmd,"mpart_mrg")) {
			multipartial_margin=strtod(val,NULL);
		}
		else if (!strcmp(cmd,"mpart_on")) {
			multipartial_on=strtol(val,NULL,10);
		}
		else if (!strcmp(cmd,"id3mp3")) {
			id3_tag_mode=strtol(val,NULL,10);
		}
#ifdef USE_RESAMPLER
		else if (!strcmp(cmd,"resam_use")) {
			use_resampler=strtol(val,NULL,10);
		}
		else if (!strcmp(cmd,"resam_mpx")) {
			computed_resample_factor=strtod(val,NULL);
		}
		else if (!strcmp(cmd,"resam_vtl")) {
			vocal_track_length=strtod(val,NULL);
		}
#endif
#ifdef USE_SUBMIXER
        else if (!strcmp(cmd,"smix_tempo")) {
            smix_speed=strtod(val,NULL);
        }
        else if (!strcmp(cmd,"smix_maxtempo")) {
            smix_maxspeed=strtod(val,NULL);
        }
        else if (!strcmp(cmd,"smix_drop")) {
            smix_droptime=strtol(val,NULL,10);
        }
        else if (!strcmp(cmd,"smix_rise")) {
            smix_risetime=strtol(val,NULL,10);
        }
        else if (!strcmp(cmd,"smix_mindist")) {
            smix_mindist=strtol(val,NULL,10);
        }
        else if (!strcmp(cmd,"smix_margin")) {
            smix_audio_margin=strtol(val,NULL,10);
        }
        else if (!strcmp(cmd,"smix_factor")) {
            smix_volume_factor=strtod(val,NULL);
        }
        else if (!strcmp(cmd,"smix_maxvol")) {
            smix_max_origin_vol=strtod(val,NULL);
        }
#endif
	}
	fclose(f);
#ifdef USE_RESAMPLER
	compute_resample_par();
#endif
#ifdef USE_SUBMIXER
	if (smix_speed < 0.1) {
		smix_speed = gtk_spin_button_get_value((gpointer)spin_tempo);
		smix_maxspeed = smix_speed * 0.9;
	}
#endif
	acc_make_wrap();
}

void set_mbrola_voice(GtkWidget *widget,void *dummy)
{
	GtkWidget *dialog=gtk_file_chooser_dialog_new("Wybierz plik",
		GTK_WINDOW(main_window),GTK_FILE_CHOOSER_ACTION_OPEN,
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
		NULL);
	GtkFileFilter *filter;
	gtk_file_chooser_set_local_only((gpointer)dialog,TRUE);
	gtk_file_chooser_add_filter((gpointer) dialog,
		filter=make_filter("Pliki Mbroli","pl1",NULL));
	gtk_file_chooser_add_filter((gpointer) dialog,
		make_filter("Wszystkie pliki","*",NULL));
	gtk_file_chooser_set_filter((gpointer) dialog,filter);
	if (gtk_dialog_run((gpointer)dialog)==GTK_RESPONSE_ACCEPT){
		strcpy(mbrola_voice,gtk_file_chooser_get_filename((gpointer)dialog));
	}
	gtk_widget_destroy(dialog);

}
